@@ -3,15 +3,19 @@
 #ifndef KVNC_H
 #define KVNC_H
 
-#include <ktmainwindow.h>
+#include <kmainwindow.h>
 
-class KAction;
 class KToggleAction;
-class KRecentFilesAction;
 class KRFBCanvas;
 class QPushButton;
 
-class KVNC : public KTMainWindow
+/**
+ * Top level window for Keystone.
+ *
+ * @author Richard Moore, rich@kde.org
+ * @version $Id: kvnc.h,v 1.4 2000/09/03 00:43:12 waba Exp $
+ */
+class KVNC : public KMainWindow
 {
   Q_OBJECT
 public: 
@@ -38,6 +42,8 @@ private:
   KRFBCanvas *canvas;
   KPopupMenu *cornerMenu;
   QPushButton *cornerButton;
+  KToggleAction *menubarAction;
+  KToggleAction *statusbarAction;
 };
 
 #endif // KVNC_H
