@@ -38,7 +38,6 @@
 #include "accountdialog.h"
 #include "kmacctlocal.h"
 #include "kmacctmgr.h"
-#include "kmacctpop.h"
 #include "kmacctexppop.h"
 #include "kmfolder.h"
 #include "kmfoldermgr.h"
@@ -58,7 +57,7 @@ AccountDialog::AccountDialog( KMAccount *account, const QStringList &identity,
   {
     makeLocalAccountPage();
   }
-  else if( accountType == "pop" || accountType == "advanced pop" )
+  else if( accountType == "pop" )
   {
     makePopAccountPage();
   }
@@ -130,11 +129,18 @@ void AccountDialog::makeLocalAccountPage()
   topLayout->addWidget( label, 8, 0 );
   mLocal.identityCombo = new QComboBox( false, page );
   topLayout->addWidget( mLocal.identityCombo, 8, 1 );
+  // GS - this was moved inside the commented block 9/30/2000
+  //      (I think Don missed it?)
+  label->setEnabled(false);
   */
 
-  label->setEnabled(false);
   //mLocal.identityCombo->setEnabled(false);
 
+  label = new QLabel( i18n("Precommand:"), page );
+  topLayout->addWidget( label, 8, 0 );
+  mLocal.precommand = new QLineEdit( page );
+  topLayout->addWidget( mLocal.precommand, 8, 1 );
+
   connect(kapp,SIGNAL(kdisplayFontChanged()),SLOT(slotFontChanged()));
 }
 
@@ -234,8 +240,8 @@ void AccountDialog::makePopAccountPage()
   topLayout->addWidget( label, 15, 0 );
   mPop.identityCombo = new QComboBox( false, page );
   topLayout->addWidget( mPop.identityCombo, 15, 1 );
-  */
   label->setEnabled(false);
+  */
   //  mPop.identityCombo->setEnabled(false);
 
   label = new QLabel( i18n("Precommand:"), page );
@@ -249,7 +255,7 @@ void AccountDialog::makePopAccountPage()
 
 void AccountDialog::setupSettings()
 {
-  QComboBox *folderCombo;
+  QComboBox *folderCombo = 0;
   int interval = mAccount->checkInterval();
 
   QString accountType = mAccount->type();
@@ -260,14 +266,15 @@ void AccountDialog::setupSettings()
     mLocal.intervalSpin->setValue( QMAX(1, interval) ); 
     mLocal.intervalCheck->setChecked( interval >= 1 );
     mLocal.excludeCheck->setChecked( mAccount->checkExclude() );
+    mLocal.precommand->setText( mAccount->precommand() );
 
     slotEnableLocalInterval( interval >= 1 );
     folderCombo = mLocal.folderCombo;
     //    mLocal.identityCombo->insertStringList( mIdentityList );
   }
-  else
+  else if( accountType == "pop" )
   {
-    KMAcctPop &ap = *(KMAcctPop*)mAccount;
+    KMAcctExpPop &ap = *(KMAcctExpPop*)mAccount;
     mPop.nameEdit->setText( mAccount->name() );
     mPop.loginEdit->setText( ap.login() );
     mPop.passwordEdit->setText( ap.passwd());
@@ -286,6 +293,8 @@ void AccountDialog::setupSettings()
     folderCombo = mPop.folderCombo;
     //    mPop.identityCombo->insertStringList( mIdentityList );
   }
+  else // Unknown account type
+    return;
 
   KMFolderDir *fdir = (KMFolderDir*)&kernel->folderMgr()->dir();
   KMFolder *acctFolder = mAccount->folder();
@@ -303,7 +312,8 @@ void AccountDialog::setupSettings()
     for( KMFolder *folder = (KMFolder*)fdir->first(); folder != 0;
 	 folder = (KMFolder*)fdir->next() )
     {
-      if( folder->isDir() || folder->isSystemFolder() ) 
+      if( folder->isDir() || 
+	  (folder->isSystemFolder() && (folder->name() != "inbox" ))) 
       {
 	continue;
       }
@@ -325,7 +335,7 @@ void AccountDialog::setupSettings()
 void AccountDialog::slotSSLChanged()
 {
   if (mPop.useSSLCheck->isChecked()) {
-    struct servent *serv = getservbyname("spop3", "tcp");
+    struct servent *serv = getservbyname("pop3s", "tcp");
     if (serv) {
       QString x;
       x.sprintf("%u", ntohs(serv->s_port));
@@ -364,12 +374,14 @@ void AccountDialog::saveSettings()
 			     mLocal.intervalSpin->value() : 0 );
     mAccount->setCheckExclude( mLocal.excludeCheck->isChecked() );
  
+    mAccount->setPrecommand( mLocal.precommand->text() );
+
     KMFolder *folder 
       = kernel->folderMgr()->find( mLocal.folderCombo->currentText() );
     mAccount->setFolder( folder );
 
   }
-  else if( accountType == "pop" || accountType == "advanced pop" )
+  else if( accountType == "pop" )
   {
     mAccount->setName( mPop.nameEdit->text() );
     mAccount->setCheckInterval( mPop.intervalCheck->isChecked() ? 
@@ -380,32 +392,18 @@ void AccountDialog::saveSettings()
       = kernel->folderMgr()->find( mPop.folderCombo->currentText() );
     mAccount->setFolder( folder );
     
-    if( accountType == "pop" )
-    {
-      KMAcctPop &pa = *(KMAcctPop*)mAccount;
-      pa.setHost( mPop.hostEdit->text() );
-      pa.setPort( mPop.portEdit->text().toInt() );
-      pa.setLogin( mPop.loginEdit->text() );
-      pa.setPasswd( mPop.passwordEdit->text(), true );
-      pa.setStorePasswd( mPop.storePasswordCheck->isChecked() );
-      pa.setPasswd( mPop.passwordEdit->text(), pa.storePasswd() );
-      pa.setLeaveOnServer( !mPop.deleteMailCheck->isChecked() );
-      pa.setRetrieveAll( mPop.retriveAllCheck->isChecked() );
-      pa.setPrecommand( mPop.precommand->text() );
-    }
-    else
-    {
-      KMAcctExpPop &epa = *(KMAcctExpPop*)mAccount;
-      epa.setHost( mPop.hostEdit->text() );
-      epa.setPort( mPop.portEdit->text().toInt() );
-      epa.setLogin( mPop.loginEdit->text() );
-      epa.setPasswd( mPop.passwordEdit->text(), true );
-      epa.setUseSSL( mPop.useSSLCheck->isChecked() );
-      epa.setStorePasswd( mPop.storePasswordCheck->isChecked() );
-      epa.setPasswd( mPop.passwordEdit->text(), epa.storePasswd() );
-      epa.setLeaveOnServer( !mPop.deleteMailCheck->isChecked() );
-      epa.setRetrieveAll( mPop.retriveAllCheck->isChecked() );
-    }
+    KMAcctExpPop &epa = *(KMAcctExpPop*)mAccount;
+    epa.setHost( mPop.hostEdit->text() );
+    epa.setPort( mPop.portEdit->text().toInt() );
+    epa.setLogin( mPop.loginEdit->text() );
+    epa.setPasswd( mPop.passwordEdit->text(), true );
+    epa.setUseSSL( mPop.useSSLCheck->isChecked() );
+    epa.setStorePasswd( mPop.storePasswordCheck->isChecked() );
+    epa.setPasswd( mPop.passwordEdit->text(), epa.storePasswd() );
+    epa.setLeaveOnServer( !mPop.deleteMailCheck->isChecked() );
+    epa.setRetrieveAll( mPop.retriveAllCheck->isChecked() ); 
+    epa.setPrecommand( mPop.precommand->text() );
+
   }
   kernel->acctMgr()->writeConfig(TRUE);
 }
