@@ -35,10 +35,12 @@ class KColorButton;
 class KFontChooser;
 class KpgpConfig;
 class ColorListBox;
+class KMAccount;
 
 #include <klistview.h>
 #include <kdialogbase.h>
-
+#include <qguardedptr.h>
+#include <qvaluelist.h>
 
 class NewIdentityDialog : public KDialogBase
 {
@@ -260,6 +262,9 @@ class ConfigureDialog : public KDialogBase
       QListViewItem *mListItemNewFeature;
       QListViewItem *mListItemContrast;
       QPushButton  *profileDeleteButton;
+      QComboBox     *addressbookCombo;
+      QLabel        *addressbookLabel;
+      QStringList   addressbookStrings;
     };
     struct ComposerWidget
     {
@@ -335,6 +340,7 @@ class ConfigureDialog : public KDialogBase
     virtual void slotDefault( void );
     virtual void slotOk( void );
     virtual void slotApply( void );
+    virtual void slotDoApply( bool everything );
     void setup( void );
 
   private:
@@ -359,8 +365,10 @@ class ConfigureDialog : public KDialogBase
     void saveActiveIdentity( void );
     void setIdentityInformation( const QString &identityName );
     QStringList identityStrings( void );
+    QStringList occupiedNames( void );
 
   private slots:
+    void slotCancelOrClose( void );
     void slotNewIdentity( void );
     void slotRenameIdentity( void );
     void slotRemoveIdentity( void );
@@ -378,6 +386,7 @@ class ConfigureDialog : public KDialogBase
     void slotRemoveSelectedAccount( void );
     void slotCustomFontSelectionChanged( void );
     void slotFontSelectorChanged( int index );
+    void slotAddressbookSelectorChanged( int index );
     void slotCustomColorSelectionChanged( void );
     void slotWordWrapSelectionChanged( void );
     void slotMimeHeaderSelectionChanged( void );
@@ -400,6 +409,14 @@ class ConfigureDialog : public KDialogBase
     MiscWidget       mMisc;
 
     IdentityList     mIdentityList;
+    QValueList<QGuardedPtr<KMAccount> > mAccountsToDelete;
+    QValueList<QGuardedPtr<KMAccount> > mNewAccounts;
+    struct mModifiedAccountsType
+    {
+      QGuardedPtr<KMAccount> oldAccount;
+      QGuardedPtr<KMAccount> newAccount;
+    };
+    QValueList<mModifiedAccountsType*> mModifiedAccounts;
 };
 
 
