@@ -19,7 +19,6 @@ KBusyPtr :: KBusyPtr ()
   numCursors = 0;
   frameDelay = 500;
   cursorList = NULL;
-  bitmapList = NULL;
   //animated   = TRUE;
   animated   = FALSE;
 
@@ -33,9 +32,7 @@ KBusyPtr :: KBusyPtr ()
 KBusyPtr :: ~KBusyPtr()
 {
   if (cursorList) delete[] cursorList;
-  if (bitmapList) delete[] bitmapList;
   cursorList = NULL;
-  bitmapList = NULL;
 }
 
 
@@ -55,7 +52,7 @@ void KBusyPtr :: busy (void)
       kapp->setOverrideCursor(cursorList[currentCursor]);
       if (animated) start(frameDelay);
     }
-    kapp->processEvents(200);
+    //    kapp->processEvents(200);
   }
   busyLevel++;
 }
@@ -71,7 +68,7 @@ void KBusyPtr :: idle (void)
   {
     stop();
     kapp->restoreOverrideCursor();
-    kapp->processEvents(200);
+    //    kapp->processEvents(200);
   }
 }
 
@@ -140,9 +137,6 @@ void KBusyPtr :: loadCursor (const char* cursorName,const char* maskName)
   numY = map.height() >> 4;
   numCursors = numX * numY;
 
-  if (bitmapList) delete[] bitmapList;
-  QSize size(16,16);
-  bitmapList = new QBitmap[numCursors](size);
   if (cursorList) delete[] cursorList;
   cursorList = new QCursor[numCursors];
 
