@@ -6,13 +6,8 @@
 
 #include "kmtopwidget.h"
 
-//#include <qstring.h>
 #include <qlabel.h>
 
-//#include <qlist.h>
-//#include <qevent.h>
-//#include <qwidget.h>
-
 #include <qcheckbox.h>
 #include <qpushbutton.h>
 #include <qclipboard.h>
@@ -20,13 +15,12 @@
 #include <qfont.h>
 #include <keditcl.h>
 #include <qlineedit.h>
-
+#include <ktempfile.h>
+#include <kio/job.h>
+#include <kurl.h>
 
 #include "kmmsgpart.h"
-
-
-
-
+#include "kmmsgbase.h"
 
 #ifndef KRN
 class _StringPair {
@@ -56,6 +50,7 @@ class KSpell;
 class KSpellConfig;
 class KMComposeWin;
 class KToggleAction;
+class KSelectAction;
 
 typedef QList<KMMessagePart> KMMsgPartList;
 
@@ -124,6 +119,7 @@ signals:
   void completion();
 
 public slots:
+   void undo();
    void copy();
    void cut();
    void paste();
@@ -137,15 +133,24 @@ protected:
 };
 
 
+class KMHeaders;
+
 //-----------------------------------------------------------------------------
 class KMComposeWin : public KMTopLevelWidget
 {
   Q_OBJECT
+  friend class KMHeaders;         // needed for the digest forward
 
 public:
   KMComposeWin(KMMessage* msg=NULL, QString id = "unknown" );
   ~KMComposeWin();
 
+  /** To catch palette changes */
+  virtual bool event(QEvent *e);
+
+  /** update colors */
+  void readColorConfig();
+
   /** Write settings to app's config file. */
    void writeConfig(void);
 
@@ -184,6 +189,8 @@ public slots:
 
   void slotFind();
   void slotReplace();
+  void slotUndo();
+  void slotRedo();
   void slotCut();
   void slotCopy();
   void slotPaste();
@@ -191,13 +198,24 @@ public slots:
   void slotAddrBook(); // Open addressbook editor dialog.
   void slotInsertFile(); // Insert a file to the end of the text in the editor.
 
+  void slotSetCharset();
+
   void slotSpellcheck(); // Check spelling of text.
   void slotSpellcheckConfig();
+
+  // XML-GUI stuff
+  void slotToggleToolBar();
+  void slotToggleStatusBar();
   void slotEditToolbars();
+  void slotEditKeys();
+
   void readConfig(void); // Read settings from app's config file.
-  
+
   void slotUpdWinTitle(const QString& ); // Change window title to given string.
 
+  /** Switch the icon to lock or unlock respectivly. */
+  void slotEncryptToggled(bool);
+
   /** Append signature file to the end of the text in the editor. */
   void slotAppendSignature();
 
@@ -210,16 +228,13 @@ public slots:
   /** Popup a nice "not implemented" message. */
   void slotToDo();
 
-  /** Show/hide toolbar. */
-  void slotToggleToolBar();
-
   /** Open a popup-menu in the attachments-listbox. */
   void slotAttachPopupMenu(QListViewItem *, const QPoint &, int);
 
   /** Returns the number of the current attachment in the listbox,
   or -1 if there is no current attachment */
   int currentAttachmentNum();
-  
+
   /** Attachment operations. */
   void slotAttachView();
   void slotAttachRemove();
@@ -237,7 +252,7 @@ public slots:
 
   void slotCleanSpace();
 
-  
+
 //  void slotSpellConfigure();
   void slotSpellcheckDone();
 
@@ -246,17 +261,18 @@ public slots:
 
   void updateCursorPosition();
 
-  void slotConfigureCharsets();
-  void slotSetCharsets(const char *message,const char *composer,
-		       bool ascii,bool quote,bool def);
   void slotView();
-  
+
   /** Move focus to next/prev edit widget */
   void focusNextPrevEdit(const QWidget* current, bool next);
 
   /** Update composer field to reflect new identity  */
   void slotIdentityActivated(int idx);
 
+  /** KIO slots for attachment insertion */
+  void slotAttachFileData(KIO::Job *, const QByteArray &);
+  void slotAttachFileResult(KIO::Job *);
+
 protected:
   /** Install grid management and header fields. If fields exist that
     should not be there they are removed. Those that are needed are
@@ -270,13 +286,13 @@ protected:
   void rethinkHeaderLine(int value, int mask, int& row,
 				 const QString labelStr, QLabel* lbl,
 				 QComboBox* cbx, QCheckBox *chk);
-  
+
   /** Initialization methods */
   void setupActions();
   void setupStatusBar();
   void setupEditor();
-  
-  
+
+
   /** Header fields. */
    const QString subject(void) const { return mEdtSubject.text(); }
    const QString to(void) const { return mEdtTo.text(); }
@@ -311,22 +327,20 @@ protected:
 
 private:
   /** Get message including signing and encrypting it */
-   const QString pgpProcessedMsg(void);
-
-#if defined CHARSETS
-  /** Converts message text for sending. */
-  void convertToSend(const QString str);
+  const QString pgpProcessedMsg(void);
 
   /** Test if string has any 8-bit characters */
   bool is8Bit(const QString str);
 
   /** Set edit widget charset */
   void setEditCharset();
-#endif
 
   /** Send the message */
   void doSend(int sendNow=-1);
 
+  /** get default charset from locale settings */
+  const QString defaultCharset(void);
+
 protected:
   QWidget   mMainWidget;
   QComboBox mIdentity, mTransport;
@@ -349,7 +363,7 @@ protected:
   QListView *mAtmListBox;
   QList<QListViewItem> mAtmItemList;
   KMMsgPartList mAtmList;
-  bool mAutoSign, mAutoPgpSign, mShowToolBar, mAutoDeleteMsg;
+  bool mAutoSign, mAutoPgpSign, mAutoDeleteMsg;
   long mShowHeaders;
   QString mDefEncoding;
 #ifndef KRN
@@ -366,7 +380,7 @@ protected:
   QFont mBodyFont;
   //  QList<QLineEdit> mEdtList;
   QList<QWidget> mEdtList;
-  static QString mPathAttach;
+  QList<KTempFile> mAtmTempList;
   QPalette mPalette;
   QString mId, mOldSigText;
   QStringList mTransportHistory;
@@ -375,19 +389,23 @@ protected:
   KToggleAction *confirmReadAction, *urgentAction, *allFieldsAction, *fromAction;
   KToggleAction *replyToAction, *toAction, *ccAction, *bccAction, *subjectAction;
   KToggleAction *identityAction, *transportAction;
+  KToggleAction *toolbarAction, *statusbarAction;
+
+  KSelectAction *encodingAction;
 
-#if defined CHARSETS
-  int m7BitAscii;
-  QString mDefaultCharset;
   QString mCharset;
-  QString mDefComposeCharset;
-  QString mComposeCharset;
-  int mQuoteUnknownCharacters;
+  QString mDefCharset;
   QFont mSavedEditorFont;
-#endif
 
 private:
   QColor foreColor,backColor;
+  struct atmLoadData
+  {
+    KURL url;
+    QByteArray data;
+    bool insert;
+  };
+  QMap<KIO::Job *, atmLoadData> mapAtmLoadData;
 };
 #endif
 
