@@ -1,4 +1,6 @@
 // kmfilteraction.cpp
+// The process methods really should use an enum instead of an int
+// -1 -> status unchanged, 0 -> success, 1 -> failure, 2-> critical failure
 
 #include "kmmessage.h"
 #include "kmfilteraction.h"
@@ -12,7 +14,10 @@
 #include <kconfig.h>
 #include <qcombobox.h>
 #include <qlineedit.h>
+#include <qguardedptr.h>
 #include <ktempfile.h>
+#include <kdebug.h>
+
 #include <signal.h>
 #include <stdlib.h>
 #include <unistd.h> //for alarm (sven)
@@ -24,7 +29,7 @@ static QString resultStr;
 
 
 //-----------------------------------------------------------------------------
-KMFilterAction::KMFilterAction(const char* name): 
+KMFilterAction::KMFilterAction(const char* name):
   KMFilterActionInherited(NULL, name)
 {
 }
@@ -81,8 +86,8 @@ public:
   virtual bool folderRemoved(KMFolder* aFolder, KMFolder* aNewFolder);
   static KMFilterAction* newAction(void);
 protected:
-  KMFolder* mDest;
-  QList<KMFolder> folders;
+  QGuardedPtr<KMFolder> mDest;
+  QValueList<QGuardedPtr<KMFolder> > gfolders;
 };
 
 bool KMFilterActionMove::folderRemoved(KMFolder* aFolder, KMFolder* aNewFolder)
@@ -111,15 +116,15 @@ KMFilterActionMove::KMFilterActionMove(): KMFilterAction("transfer")
 
 int KMFilterActionMove::process(KMMessage* msg, bool&stopIt)
 {
-  if (!mDest) return TRUE;
+  if (!mDest) return 1;
   KMFilterAction::tempOpenFolder(mDest);
   if (msg->parent())
     return 0;
-  if (mDest->moveMsg(msg) == 0) 
+  if (mDest->moveMsg(msg) == 0)
     return 0; // ok, added
   else
   {
-    debug ("KMfilteraction - couldn't move msg");
+    kdDebug() << "KMfilteraction - couldn't move msg" << endl;
     stopIt = TRUE;
     return 2; // critical error: couldn't add
   }
@@ -131,17 +136,19 @@ QWidget* KMFilterActionMove::createParamWidget(KMGFilterDlg* aParent)
   QComboBox* cbx;
   QStringList str;
 
-  folders.clear();
-  kernel->folderMgr()->createFolderList( &str, &folders );
-  cbx = aParent->createFolderCombo( &str, &folders, mDest );
+  gfolders.clear();
+
+  kernel->folderMgr()->createI18nFolderList( &str, &gfolders );
+  cbx = aParent->createFolderCombo( &str, &gfolders, mDest );
+
   return cbx;
 }
 
 void KMFilterActionMove::applyParamWidgetValue(QWidget* aParamWidget)
 {
   QComboBox* cbx = (QComboBox*)aParamWidget;
-  if ((cbx->currentItem() >= 0) && (cbx->currentItem() < (int)folders.count()))
-    mDest = folders.at(cbx->currentItem());
+  if ((cbx->currentItem() >= 0) && (cbx->currentItem() < (int)gfolders.count()))
+    mDest = (*gfolders.at(cbx->currentItem()));
   else
     mDest = 0;
 }
@@ -180,7 +187,7 @@ KMFilterAction* KMFilterActionForward::newAction(void)
 {
   return (new KMFilterActionForward);
 }
- 
+
 const QString KMFilterActionForward::label(void) const
 {
   return i18n("forward to");
@@ -189,44 +196,44 @@ const QString KMFilterActionForward::label(void) const
 KMFilterActionForward::KMFilterActionForward(): KMFilterAction("forward")
 {
 }
- 
+
 int KMFilterActionForward::process(KMMessage* aMsg, bool& /*stop*/)
 {
   KMMessage* msg;
-  if (mTo.isEmpty()) return TRUE;
+  if (mTo.isEmpty()) return 1;
   msg = aMsg->createForward();
   msg->setTo(mTo);
   if (!kernel->msgSender()->send(msg))
   {
-    debug("KMFilterActionForward: could not forward message (sending failed)");
+    kdDebug() << "KMFilterActionForward: could not forward message (sending failed)" << endl;
     return 1; // error: couldn't send
   }
   return -1;
 }
- 
+
 QWidget* KMFilterActionForward::createParamWidget(KMGFilterDlg* aParent)
 {
   QLineEdit* edt;
   edt = aParent->createEdit(mTo);
   return edt;
 }
- 
+
 void KMFilterActionForward::applyParamWidgetValue(QWidget* aParamWidget)
 {
   QLineEdit* w = (QLineEdit*)aParamWidget;
   mTo = w->text();
 }
- 
+
 void KMFilterActionForward::argsFromString(const QString argsStr)
 {
   mTo = argsStr;
 }
- 
+
 const QString KMFilterActionForward::argsAsString(void) const
 {
   return mTo;
 }
- 
+
 //=============================================================================
 // Execute a shell command
 //=============================================================================
@@ -245,38 +252,38 @@ public:
 protected:
   QString mCmd;
 };
- 
+
 KMFilterAction* KMFilterActionExec::newAction(void)
 {
   return (new KMFilterActionExec);
 }
- 
+
 const QString KMFilterActionExec::label(void) const
 {
   return i18n("execute");
 }
- 
+
 KMFilterActionExec::KMFilterActionExec(const char* aName)
   : KMFilterAction(aName)
 {
 }
- 
+
 void KMFilterActionExec::dummySigHandler(int)
 {
 }
- 
+
 int KMFilterActionExec::process(KMMessage* /*aMsg*/, bool& /*stop*/)
 {
   void (*oldSigHandler)(int);
   int rc;
-  if (mCmd.isEmpty()) return TRUE;
+  if (mCmd.isEmpty()) return 1;
   oldSigHandler = signal(SIGALRM, &KMFilterActionExec::dummySigHandler);
   alarm(30);
   rc = system(mCmd);
   alarm(0);
   signal(SIGALRM, oldSigHandler);
   if (rc & 255)
-    return 1; 
+    return 1;
   else
     return -1;
 }
@@ -287,18 +294,18 @@ QWidget* KMFilterActionExec::createParamWidget(KMGFilterDlg* aParent)
   edt = aParent->createEdit(mCmd);
   return edt;
 }
- 
+
 void KMFilterActionExec::applyParamWidgetValue(QWidget* aParamWidget)
 {
   QLineEdit* w = (QLineEdit*)aParamWidget;
   mCmd = w->text();
 }
- 
+
 void KMFilterActionExec::argsFromString(const QString argsStr)
 {
   mCmd = argsStr;
 }
- 
+
 const QString KMFilterActionExec::argsAsString(void) const
 {
   return mCmd;
@@ -316,22 +323,22 @@ public:
   static KMFilterAction* newAction(void);
   virtual int process(KMMessage* msg, bool& stopIt);
 };
- 
+
 KMFilterAction* KMFilterActionExtFilter::newAction(void)
 {
   return (new KMFilterActionExtFilter);
 }
- 
+
 const QString KMFilterActionExtFilter::label(void) const
 {
   return i18n("filter app");
 }
- 
+
 KMFilterActionExtFilter::KMFilterActionExtFilter(const char* aName)
   : KMFilterActionExec(aName)
 {
 }
- 
+
 int KMFilterActionExtFilter::process(KMMessage* aMsg, bool& stop)
 {
   int rc=0, len;
@@ -387,49 +394,6 @@ int KMFilterActionExtFilter::process(KMMessage* aMsg, bool& stop)
   return rc;
 }
 
-//=============================================================================
-// Skip all other filter rules
-//=============================================================================
-class KMFilterActionSkip: public KMFilterAction
-{
-public:
-  KMFilterActionSkip();
-  virtual const QString label(void) const;
-  virtual int process(KMMessage* msg, bool& stopIt);
-  virtual void argsFromString(const QString argsStr);
-  virtual const QString argsAsString(void) const;
-  static KMFilterAction* newAction(void);
-};
-
-KMFilterActionSkip::KMFilterActionSkip(): KMFilterAction("skip rest")
-{
-}
-
-const QString KMFilterActionSkip::label(void) const
-{
-  return i18n("skip rest");
-}
-
-KMFilterAction* KMFilterActionSkip::newAction(void)
-{
-  return (new KMFilterActionSkip);
-}
-
-int KMFilterActionSkip::process(KMMessage*, bool& stopIt)
-{
-  stopIt = TRUE;
-  return -1;
-}
-
-void KMFilterActionSkip::argsFromString(const QString)
-{
-}
-
-const QString KMFilterActionSkip::argsAsString(void) const
-{
-  return "";
-}
-
 //=============================================================================
 // Specify Identity to be used when replying to a message
 //=============================================================================
@@ -461,7 +425,7 @@ const QString KMFilterActionIdentity::label(void) const
 
 KMFilterActionIdentity::KMFilterActionIdentity(): KMFilterAction("set identity")
 {
-  id = i18n( "unknown" );
+  id = i18n( "Default" );
 }
 
 int KMFilterActionIdentity::process(KMMessage* msg, bool& )
@@ -486,12 +450,81 @@ void KMFilterActionIdentity::applyParamWidgetValue(QWidget* aParamWidget)
 
 void KMFilterActionIdentity::argsFromString(const QString argsStr)
 {
-  id = argsStr;
+  if (argsStr.isEmpty())
+    id = i18n( "Default" );
+  else
+    id = argsStr;
 }
 
 const QString KMFilterActionIdentity::argsAsString(void) const
 {
-  return id;
+  if (id == i18n( "Default" ))
+    return "";
+  else
+    return id;
+}
+
+
+//=============================================================================
+// Specify mail transport (smtp server) to be used when replying to a message
+//=============================================================================
+class KMFilterActionTransport: public KMFilterAction
+{
+public:
+  KMFilterActionTransport();
+  virtual const QString label(void) const;
+  virtual int process(KMMessage* msg, bool& stopIt);
+  virtual QWidget* createParamWidget(KMGFilterDlg* parent);
+  virtual void applyParamWidgetValue(QWidget* paramWidget);
+  virtual void argsFromString(const QString argsStr);
+  virtual const QString argsAsString(void) const;
+  static KMFilterAction* newAction(void);
+protected:
+  QString mTransport;
+};
+
+KMFilterAction* KMFilterActionTransport::newAction(void)
+{
+  return (new KMFilterActionTransport);
+}
+
+const QString KMFilterActionTransport::label(void) const
+{
+  return i18n("set transport");
+}
+
+KMFilterActionTransport::KMFilterActionTransport(): KMFilterAction("set transport")
+{
+  mTransport = kernel->msgSender()->transportString();
+}
+
+int KMFilterActionTransport::process(KMMessage* msg, bool& )
+{
+  msg->setHeaderField( "X-KMail-Transport", mTransport );
+  return -1;
+}
+
+QWidget* KMFilterActionTransport::createParamWidget(KMGFilterDlg* aParent)
+{
+  QLineEdit* edt;
+  edt = aParent->createEdit(mTransport);
+  return edt;
+}
+
+void KMFilterActionTransport::applyParamWidgetValue(QWidget* aParamWidget)
+{
+  QLineEdit* w = (QLineEdit*)aParamWidget;
+  mTransport = w->text();
+}
+
+void KMFilterActionTransport::argsFromString(const QString argsStr)
+{
+  mTransport = argsStr;
+}
+
+const QString KMFilterActionTransport::argsAsString(void) const
+{
+  return mTransport;
 }
 
 
@@ -504,10 +537,10 @@ void KMFilterActionDict::init(void)
 {
   insert("set identity", i18n("set identity"),
 	 KMFilterActionIdentity::newAction);
+  insert("set transport", i18n("set transport"),
+	 KMFilterActionTransport::newAction);
   insert("transfer", i18n("transfer"),
 	 KMFilterActionMove::newAction);
-  insert("skip rest", i18n("skip rest"),
-	 KMFilterActionSkip::newAction);
   insert("forward", i18n("forward to"),
          KMFilterActionForward::newAction);
   insert("execute", i18n("execute"),
