@@ -11,6 +11,7 @@
 #include "kmfoldermgr.h"
 
 #include <kapp.h>
+#include <kdebug.h>
 #include <kbuttonbox.h>
 #include <qbuttongroup.h>
 #include <qframe.h>
@@ -25,9 +26,9 @@
 #include <assert.h>
 #include <qstrlist.h>
 #include <klocale.h>
-#include <kwm.h>
+#include <kwin.h>
 
-static QStrList sFilterOpList, sFilterFuncList, sFilterFieldList, 
+static QStringList sFilterOpList, sFilterFuncList, sFilterFieldList,
                 sFilterActionList;
 
 //=============================================================================
@@ -61,7 +62,7 @@ KMFilterDlg::KMFilterDlg(QWidget* parent, const char* name):
   mFilter = 0;
 
   setCaption(i18n("Filter Rules"));
-  KWM::setMiniIcon(winId(), kapp->miniIcon());
+  KWin::setIcons(winId(), kapp->icon(), kapp->miniIcon());
 
   mFilterList = new QListBox(this);
   mFilterList->setMinimumSize(100, 200);
@@ -81,23 +82,23 @@ KMFilterDlg::KMFilterDlg(QWidget* parent, const char* name):
   mCbxHeight = h;
   mRuleFuncA->setMinimumSize(110, h);
   mRuleFuncA->setMaximumSize(32767, h);
-  mRuleFuncA->insertStrList(&sFilterFuncList);
+  mRuleFuncA->insertStringList(sFilterFuncList);
   fgrid->addWidget(mRuleFuncA, 0, 1);
 
   mRuleFuncB = new QComboBox(false, this);
   mRuleFuncB->setMinimumSize(110, h);
   mRuleFuncB->setMaximumSize(32767, h);
-  mRuleFuncB->insertStrList(&sFilterFuncList);
+  mRuleFuncB->insertStringList(sFilterFuncList);
   fgrid->addWidget(mRuleFuncB, 2, 1);
 
   mRuleFieldA = new QComboBox(true, this);
-  mRuleFieldA->insertStrList(&sFilterFieldList);
+  mRuleFieldA->insertStringList(sFilterFieldList);
   mRuleFieldA->setMinimumSize(100, h);
   mRuleFieldA->setMaximumSize(32767, h);
   fgrid->addWidget(mRuleFieldA, 0, 0);
 
   mRuleFieldB = new QComboBox(true, this);
-  mRuleFieldB->insertStrList(&sFilterFieldList);
+  mRuleFieldB->insertStringList(sFilterFieldList);
   mRuleFieldB->setMinimumSize(100, h);
   mRuleFieldB->setMaximumSize(32767, h);
   fgrid->addWidget(mRuleFieldB, 2, 0);
@@ -113,7 +114,7 @@ KMFilterDlg::KMFilterDlg(QWidget* parent, const char* name):
   fgrid->addWidget(mRuleValueB, 2, 2);
 
   mRuleOp = new QComboBox(false, this);
-  mRuleOp->insertStrList(&sFilterOpList);
+  mRuleOp->insertStringList(sFilterOpList);
   mRuleOp->setMinimumSize(50, h);
   mRuleOp->setMaximumSize(32767, h);
   fgrid->addMultiCellWidget(mRuleOp, 1, 1, 0, 1);
@@ -124,7 +125,7 @@ KMFilterDlg::KMFilterDlg(QWidget* parent, const char* name):
   for (i=0; i<FILTER_MAX_ACTIONS; i++)
   {
     mFaType[i] = new KMFaComboBox(this);
-    mFaType[i]->insertStrList(&sFilterActionList);
+    mFaType[i]->insertStringList(sFilterActionList);
     mFaType[i]->setMinimumSize(80, h);
     mFaType[i]->setMaximumSize(32767, h);
     agrid->addWidget(mFaType[i], i, 0);
@@ -321,8 +322,8 @@ void KMFilterDlg::showFilter(KMFilter* aFilter)
 //-----------------------------------------------------------------------------
 void KMFilterDlg::resizeEvent(QResizeEvent *qre)
 {
-  debug( "KMFilterDlg::resizeEvent" );
-  debug( QString( "width %1" ).arg( qre->size().width() ));
+  kdDebug() << "KMFilterDlg::resizeEvent" << endl;
+  kdDebug() << QString( "width %1" ).arg( qre->size().width() ) << endl;
   int i, w;
   QWidget* pwidg;
   QSize sz;
@@ -338,10 +339,20 @@ void KMFilterDlg::resizeEvent(QResizeEvent *qre)
       sz.setHeight(pwidg->height());
       pwidg->resize(sz);
       pwidg->move(pos);
-      debug( QString( "posx %1" ).arg( pos.x() ));
+      kdDebug() << QString( "posx %1" ).arg( pos.x() ) << endl;
     }
 }
 
+//-----------------------------------------------------------------------------
+QString KMFilterDlg::ruleFieldToEnglish(const QString & i18nVal)
+{
+  if (i18nVal == i18n("<message>")) return QString("<message>");
+  if (i18nVal == i18n("<body>")) return QString("<body>");
+  if (i18nVal == i18n("<any header>")) return QString("<any header>");
+  if (i18nVal == i18n("<To or Cc>")) return QString("<To or Cc>");
+  return i18nVal;
+}
+
 //-----------------------------------------------------------------------------
 void KMFilterDlg::applyFilterChanges(void)
 {
@@ -350,10 +361,10 @@ void KMFilterDlg::applyFilterChanges(void)
 
   if (!mFilter || !updown_move_semaphore) return;
 
-  mFilter->ruleA().init(mRuleFieldA->currentText(), 
+  mFilter->ruleA().init(ruleFieldToEnglish(mRuleFieldA->currentText()),
 			(KMFilterRule::Function)mRuleFuncA->currentItem(),
 			mRuleValueA->text());
-  mFilter->ruleB().init(mRuleFieldB->currentText(), 
+  mFilter->ruleB().init(ruleFieldToEnglish(mRuleFieldB->currentText()),
 			(KMFilterRule::Function)mRuleFuncB->currentItem(),
 			mRuleValueB->text());
   mFilter->setOper((KMFilter::Operator)mRuleOp->currentItem());
@@ -371,7 +382,7 @@ void KMFilterDlg::updateCurFilterName(const QString &/*text*/)
 {
   if (mCurFilterIdx < 0)
     return;
-  mFilter->setName(QString("<") + mRuleFieldA->currentText() + ">:" 
+  mFilter->setName(QString("<") + mRuleFieldA->currentText() + ">:"
 		   + mRuleValueA->text());
   QObject::disconnect(mFilterList,SIGNAL(highlighted(int)),this,SLOT(slotFilterSelected(int)));
   mFilterList->changeItem(mFilter->name(), mCurFilterIdx);
@@ -381,7 +392,7 @@ void KMFilterDlg::updateCurFilterName(const QString &/*text*/)
 //-----------------------------------------------------------------------------
 bool KMFilterDlg::testOpts(const QWidget* w) const
 {
-  if (!w) debug("KMFilterDlg: no widget given");
+  if (!w) kdDebug() << "KMFilterDlg: no widget given" << endl;
   return (w!=0);
 }
 
@@ -409,9 +420,9 @@ QLineEdit* KMFilterDlg::createEdit(const QString aTxt)
 }
 
 //-----------------------------------------------------------------------------
-QComboBox* KMFilterDlg::createFolderCombo( QStringList *str, 
-					  QList<KMFolder> *folders,
-					  KMFolder *curFolder )
+QComboBox* KMFilterDlg::createFolderCombo( QStringList *str,
+				  QValueList< QGuardedPtr<KMFolder> > *folders,
+				  QGuardedPtr<KMFolder> curFolder )
 {
   QComboBox* cbx = new QComboBox(false, this);
   int i=0,idx=-1;
@@ -422,10 +433,13 @@ QComboBox* KMFilterDlg::createFolderCombo( QStringList *str,
   for( st = str->begin(); st != str->end(); ++st)
     cbx->insertItem(*st);
 
-  KMFolder *folder;
-  for( folder = folders->first(); folder; folder = folders->next(), ++i )
+  QGuardedPtr<KMFolder> folder;
+  while (folders->at(i) != folders->end()) {
+    folder = *folders->at(i);
     if (folder == curFolder)
       idx = i;
+    ++i;
+  }
   if (idx>=0) cbx->setCurrentItem(idx);
   return cbx;
 }
@@ -468,18 +482,18 @@ void KMFilterDlg::slotActionTypeSelected(KMFaComboBox* cbx, int idx)
 
   if (mFilter->action(i)) delete mFilter->action(i);
   action = kernel->filterActionDict()->create(
-              kernel->filterActionDict()->nameOf(sFilterActionList.at(idx)));
+              kernel->filterActionDict()->nameOf(*(sFilterActionList.at(idx))));
   mFilter->setAction(i, action);
-  if (!action || idx < 0) 
+  if (!action || idx < 0)
   {
-    debug("no action selected");
+    kdDebug() << "no action selected" << endl;
     return;
   }
 
   widg = action->createParamWidget(this);
   mFaField[i] = widg;
   if (!widg) return;
-  
+
   pos = mFaType[i]->pos();
   pos.setX(pos.x() + mFaType[idx]->width() + 4);
   widg->move(pos);
@@ -496,7 +510,7 @@ void KMFilterDlg::slotFilterSelected(int idx)
 {
   KMFilter* filter;
 
-  debug( QString( "mCurFilterIdx %1 idx %2" ).arg( mCurFilterIdx ).arg( idx ) );
+  kdDebug() << QString( "mCurFilterIdx %1 idx %2" ).arg( mCurFilterIdx ).arg( idx ) << endl;
   if (mCurFilterIdx == idx)
     return;
 
@@ -617,20 +631,24 @@ void KMFilterDlg::slotBtnDelete()
 //-----------------------------------------------------------------------------
 void KMFilterDlg::slotBtnOk()
 {
-  if (mFilter) 
-  {
-    applyFilterChanges();
-    kernel->filterMgr()->writeConfig();
-  }
+  if (mFilter) applyFilterChanges();
+  kernel->filterMgr()->writeConfig();
 
-  accept();
+  KMFilterDlgInherited::close();
 }
 
 //-----------------------------------------------------------------------------
 void KMFilterDlg::slotBtnCancel()
 {
   kernel->filterMgr()->readConfig();
-  reject();
+  KMFilterDlgInherited::close();
+}
+
+//-----------------------------------------------------------------------------
+void KMFilterDlg::closeEvent( QCloseEvent *e )
+{
+  kernel->filterMgr()->readConfig();
+  KMFilterDlgInherited::closeEvent(e);
 }
 
 //-----------------------------------------------------------------------------
@@ -646,7 +664,7 @@ int KMFilterDlg::indexOfRuleField(const QString aName) const
 
   for (i=sFilterFieldList.count()-1; i>=0; i--)
   {
-    if (sFilterFieldList.at(i)==aName) break;
+    if (*(sFilterFieldList.at(i))==i18n(aName)) break;
   }
   return i;
 }
@@ -660,7 +678,7 @@ void KMFilterDlg::initLists()
   if (sFilterActionList.count() <= 0)
   {
     sFilterActionList.append(i18n("<nothing>"));
-    for (name=kernel->filterActionDict()->first(); !name.isEmpty(); 
+    for (name=kernel->filterActionDict()->first(); !name.isEmpty();
 	 name=kernel->filterActionDict()->next())
     {
       sFilterActionList.append(kernel->filterActionDict()->currentLabel());
@@ -690,7 +708,8 @@ void KMFilterDlg::initLists()
   if (sFilterFieldList.count() <= 0)
   {
     sFilterFieldList.append(" ");
-    // also see KMFilterRule::matches() if you change the following strings!
+    // also see KMFilterRule::matches() and KMFilterDlg::ruleFieldToEnglish()
+    // if you change the following strings!
     sFilterFieldList.append(i18n("<message>"));
     sFilterFieldList.append(i18n("<body>"));
     sFilterFieldList.append(i18n("<any header>"));
@@ -717,7 +736,7 @@ void KMFilterDlg::enableControls()
     deleteEnabled = TRUE;
   if (deleteEnabled && (mFilterList->currentItem() != 0))
     upEnabled = TRUE;
-  if (deleteEnabled && 
+  if (deleteEnabled &&
       (mFilterList->currentItem() != (int)mFilterList->count() - 1))
     downEnabled = TRUE;
   mBtnUp->setEnabled( upEnabled );
