@@ -34,19 +34,6 @@ QPixmap* KMFolderTree::pixTr = 0;
 QPixmap* KMFolderTree::pixSent = 0;
 
 //-----------------------------------------------------------------------------
-// KMFolderTreeItem methods
-// Begin this code may be relicensed by Troll Tech
-static QBitmap * verticalLine = 0;
-static QBitmap * horizontalLine = 0;
-
-static void cleanupBitmapLines()
-{
-    delete verticalLine;
-    delete horizontalLine;
-    verticalLine = 0;
-    horizontalLine = 0;
-}
-// End this code may be relicensed by Troll Tech
 
 class KMFolderTreeItem : public QListViewItem
 {
@@ -61,7 +48,7 @@ public:
 		    KMPaintInfo *aPaintInfo )
     : QListViewItem( parent, i18n("Mail") ),
       folder( 0 ),
-      unread( 0 ),
+      unread( QString::null ),
       mPaintInfo( aPaintInfo )
     {}
 
@@ -71,7 +58,7 @@ public:
 		    KMPaintInfo *aPaintInfo )
     : QListViewItem( parent, folder->label() ),
       folder( folder ),
-      unread( 0 ),
+      unread( QString::null ),
       mPaintInfo( aPaintInfo )
     {}
 
@@ -79,10 +66,6 @@ public:
   void paintBranches( QPainter * p, const QColorGroup & cg,
 		      int w, int y, int h, GUIStyle s )
 {
-  /******************************************************
-   This code has been replaced due to licensing problems
-   -sanders
-  *******************************************************/
   QListViewItem::paintBranches( p, cg, w, y, h, s);
 }
 
@@ -117,7 +100,7 @@ void paintCell( QPainter * p, const QColorGroup & cg,
 
   QString t = text( column );
   if ( !t.isEmpty() ) {
-    if( folder && folder->countUnread() > 0 ) {
+    if( folder && (folder->countUnread() > 0) ) {
       QFont f = p->font();
       f.setWeight(QFont::Bold);
       p->setFont(f);
@@ -129,7 +112,7 @@ void paintCell( QPainter * p, const QColorGroup & cg,
       p->setPen( mPaintInfo->colUnread );
     if (column == 0)
       p->drawText( br.right(), 0, width-marg-br.right(), height(),
-		   align | AlignVCenter, unread);
+		   align | AlignVCenter, unread );
 
   }
 }
@@ -165,10 +148,12 @@ KMFolderTree::KMFolderTree(QWidget *parent,const char *name)
   : QListView( parent, name ), mList()
 {
   static bool pixmapsLoaded = FALSE;
+  oldSelected = 0;
+  oldCurrent = 0;
 
   initMetaObject();
 
-  // Espen 2000-05-14: Getting rid of thick ugly frames 
+  // Espen 2000-05-14: Getting rid of thick ugly frames
   setLineWidth(0);
 
   mUpdateTimer = NULL;
@@ -182,6 +167,7 @@ KMFolderTree::KMFolderTree(QWidget *parent,const char *name)
   readConfig();
 
   addColumn( i18n("Folders"), 400 );
+  setShowSortIndicator(TRUE);
 
   if (!pixmapsLoaded)
   {
@@ -214,6 +200,41 @@ KMFolderTree::KMFolderTree(QWidget *parent,const char *name)
 	   this, SLOT( rightButtonPressed( QListViewItem*, const QPoint &, int)));
 }
 
+bool KMFolderTree::event(QEvent *e)
+{
+  if (e->type() == QEvent::ApplicationPaletteChange)
+  {
+     readColorConfig();
+     return true;
+  }
+  return QListView::event(e);
+}
+
+//-----------------------------------------------------------------------------
+void KMFolderTree::readColorConfig (void)
+{
+  KConfig* conf = kapp->config();
+  // Custom/System color support
+  conf->setGroup("Reader");
+  QColor c1=QColor(kapp->palette().normal().text());
+  QColor c2=QColor("blue");
+  QColor c4=QColor(kapp->palette().normal().base());
+
+  if (!conf->readBoolEntry("defaultColors",TRUE)) {
+    mPaintInfo.colFore = conf->readColorEntry("ForegroundColor",&c1);
+    mPaintInfo.colUnread = conf->readColorEntry("UnreadMessage",&c2);
+    mPaintInfo.colBack = conf->readColorEntry("BackgroundColor",&c4);
+  }
+  else {
+    mPaintInfo.colFore = c1;
+    mPaintInfo.colUnread = c2;
+    mPaintInfo.colBack = c4;
+  }
+  QPalette newPal = kapp->palette();
+  newPal.setColor( QColorGroup::Base, mPaintInfo.colBack );
+  setPalette( newPal );
+}
+
 //-----------------------------------------------------------------------------
 void KMFolderTree::readConfig (void)
 {
@@ -229,27 +250,7 @@ void KMFolderTree::readConfig (void)
     mPaintInfo.pixmap = QPixmap( pixmapFile );
   }
 
-  // Custom/System color support
-  conf->setGroup("Reader");
-  QColor c1=QColor(kapp->palette().normal().text());
-  QColor c2=QColor("blue");
-  QColor c4=QColor(kapp->palette().normal().base());
-
-  if (!conf->readBoolEntry("defaultColors",TRUE)) {
-    mPaintInfo.colFore = conf->readColorEntry("ForegroundColor",&c1);
-    mPaintInfo.colBack = conf->readColorEntry("BackgroundColor",&c4);
-    QPalette newPal = palette();
-    newPal.setColor( QColorGroup::Base, mPaintInfo.colBack );
-    setPalette( newPal );
-    mPaintInfo.colUnread = conf->readColorEntry("UnreadMessage",&c2);
-  }
-  else {
-    mPaintInfo.colFore = c1;
-    mPaintInfo.colUnread = c2;
-    QPalette newPal = palette();
-    newPal.setColor( QColorGroup::Base, c4 );
-    setPalette( newPal );
-  }
+  readColorConfig();
 
   // Custom/Ssystem font support
   conf->setGroup("Fonts");
@@ -286,7 +287,7 @@ void KMFolderTree::updateUnreadAll()
     folderNode =fdir->next())
   {
     if (!folderNode->isDir()) {
-      folder = static_cast<KMFolder*>(folderNode);
+      folder = dynamic_cast<KMFolder*>(folderNode);
 
       folder->open();
       folder->countUnread();
@@ -317,8 +318,9 @@ void KMFolderTree::writeConfig()
 {
   QListViewItemIterator it( this );
   while (it.current()) {
-    KMFolderTreeItem* fti = static_cast<KMFolderTreeItem*>(it.current());
-    writeIsListViewItemOpen(fti);
+    KMFolderTreeItem* fti = dynamic_cast<KMFolderTreeItem*>(it.current());
+    if (fti)
+      writeIsListViewItemOpen(fti);
     ++it;
   }
 }
@@ -329,15 +331,13 @@ void KMFolderTree::reload(void)
 {
   KMFolderDir* fdir;
   QString str;
-  bool upd = isUpdatesEnabled();
 
-  setUpdatesEnabled(FALSE);
   writeConfig();
 
   QListViewItemIterator it( this );
   while (it.current()) {
-    KMFolderTreeItem* fti = static_cast<KMFolderTreeItem*>(it.current());
-    if (fti->folder)
+    KMFolderTreeItem* fti = dynamic_cast<KMFolderTreeItem*>(it.current());
+    if (fti && fti->folder)
       disconnect(fti->folder,SIGNAL(numUnreadMsgsChanged(KMFolder*)),
 		 this,SLOT(refresh(KMFolder*)));
     ++it;
@@ -353,15 +353,12 @@ void KMFolderTree::reload(void)
 
   QListViewItemIterator jt( this );
   while (jt.current()) {
-    KMFolderTreeItem* fti = static_cast<KMFolderTreeItem*>(jt.current());
-    if (fti->folder)
+    KMFolderTreeItem* fti = dynamic_cast<KMFolderTreeItem*>(jt.current());
+    if (fti && fti->folder)
       connect(fti->folder,SIGNAL(numUnreadMsgsChanged(KMFolder*)),
 	      this,SLOT(refresh(KMFolder*)));
     ++jt;
   }
-
-  setUpdatesEnabled(upd);
-  if (upd) repaint();
   refresh(0);
 }
 
@@ -377,7 +374,7 @@ void KMFolderTree::addDirectory( KMFolderDir *fdir, QListViewItem* parent )
        folderNode != NULL;
        folderNode = fdir->next())
     if (!folderNode->isDir()) {
-      folder = static_cast<KMFolder*>(folderNode);
+      folder = dynamic_cast<KMFolder*>(folderNode);
       ASSERT(parent);
       fti = new KMFolderTreeItem( parent, folder, &mPaintInfo );
 
@@ -424,8 +421,8 @@ void KMFolderTree::delayedUpdate()
   QListViewItemIterator it( this );
   while (it.current()) {
     bool repaintRequired = false;
-    KMFolderTreeItem* fti = static_cast<KMFolderTreeItem*>(it.current());
-    if (!fti->folder) {
+    KMFolderTreeItem* fti = dynamic_cast<KMFolderTreeItem*>(it.current());
+    if (!fti || !fti->folder) {
       ++it;
       continue;
     }
@@ -436,13 +433,13 @@ void KMFolderTree::delayedUpdate()
 	num.setNum(fti->folder->countUnread());
       extendedName = " (" + num + ")";
       if (!fti->folder->isSystemFolder() &&
-	  !(fti->folder->label() == "inbox"))
+	  !(fti->folder->label() == i18n("inbox")))
 	fti->setPixmap( 0, *pixFull );
     }
     else {
       extendedName = "";
       if (!fti->folder->isSystemFolder() &&
-	  !(fti->folder->label() == "inbox"))
+	  !(fti->folder->label() == i18n("inbox")))
 	fti->setPixmap( 0, *pixPlain );
     }
 
@@ -463,7 +460,9 @@ void KMFolderTree::delayedUpdate()
 // Folders have been added/deleted update the tree of folders
 void KMFolderTree::doFolderListChanged()
 {
-  KMFolderTreeItem* fti = static_cast< KMFolderTreeItem* >(currentItem());
+  KMFolderTreeItem* fti = dynamic_cast< KMFolderTreeItem* >(currentItem());
+  if (!fti || !fti->folder)
+    return;
   KMFolder* folder = fti->folder;
   reload();
   QListViewItem *qlvi = indexOfFolder(folder);
@@ -473,22 +472,43 @@ void KMFolderTree::doFolderListChanged()
   }
 }
 
+
+void KMFolderTree::setSelected( QListViewItem *i, bool select )
+{
+    clearSelection();
+    QListView::setSelected( i, select );
+}
+
 //-----------------------------------------------------------------------------
 // When not dragging and dropping a change in the selected item
 // indicates the user has changed the active folder emit a signal
 // so that the header list and reader window can be udpated.
 void KMFolderTree::doFolderSelected( QListViewItem* qlvi )
 {
-  KMFolderTreeItem* fti = static_cast< KMFolderTreeItem* >(qlvi);
-  KMFolder* folder = fti->folder;
+  KMFolderTreeItem* fti = dynamic_cast< KMFolderTreeItem* >(qlvi);
+  KMFolder* folder = 0;
+  if (fti)
+    folder = fti->folder;
 
   clearSelection();
   setCurrentItem( qlvi );
   setSelected( qlvi, TRUE );
   if (!folder || folder->isDir()) {
-    emit folderSelected(NULL); // Root has been selected
+    emit folderSelected(0); // Root has been selected
+  }
+  else {
+      QString extendedName;
+      emit folderSelected(folder);
+      if (folder && (folder->countUnread() > 0) ) {
+	  QString num;
+	  num.setNum(folder->countUnread());
+	  extendedName = " (" + num + ")";
+      }
+      if (extendedName != fti->unread) {
+	  fti->unread = extendedName;
+	  fti->repaint();
+      }	
   }
-  else emit folderSelected(folder);
 }
 
 //-----------------------------------------------------------------------------
@@ -499,8 +519,8 @@ void KMFolderTree::resizeEvent(QResizeEvent* e)
   conf->setGroup("Geometry");
   conf->writeEntry(name(), size().width());
 
-  setColumnWidth( 0, size().width() );
   KMFolderTreeInherited::resizeEvent(e);
+  setColumnWidth( 0, visibleWidth() - 1 );
 }
 
 //-----------------------------------------------------------------------------
@@ -508,8 +528,8 @@ QListViewItem* KMFolderTree::indexOfFolder(const KMFolder* folder)
 {
   QListViewItemIterator it( this );
   while (it.current()) {
-    KMFolderTreeItem* fti = static_cast<KMFolderTreeItem*>(it.current());
-    if (fti->folder == folder) {
+    KMFolderTreeItem* fti = dynamic_cast<KMFolderTreeItem*>(it.current());
+    if (fti && fti->folder == folder) {
       return it.current();
     }
     ++it;
@@ -529,24 +549,25 @@ void KMFolderTree::rightButtonPressed(QListViewItem *lvi, const QPoint &p, int)
   if (!topLevelWidget()) return; // safe bet
 
   QPopupMenu *folderMenu = new QPopupMenu;
-  KMFolderTreeItem* fti = static_cast<KMFolderTreeItem*>(lvi);
-  if (!fti->folder)
+  KMFolderTreeItem* fti = dynamic_cast<KMFolderTreeItem*>(lvi);
+  if (!fti || !fti->folder)
     return;
 
-  if (fti->folder && !fti->folder->isSystemFolder()) {
-    folderMenu->insertItem(i18n("&Create Child Folder..."), this,
-			   SLOT(addChildFolder()));
-    folderMenu->insertItem(i18n("&Modify..."), topLevelWidget(),
-			   SLOT(slotModifyFolder()));
-  }
+  int m1 =folderMenu->insertItem(i18n("&Create Child Folder..."), this,
+				   SLOT(addChildFolder()));
+  int m2 = folderMenu->insertItem(i18n("&Modify..."), topLevelWidget(),
+				  SLOT(slotModifyFolder()));
   folderMenu->insertItem(i18n("C&ompact"), topLevelWidget(),
 			 SLOT(slotCompactFolder()));
   folderMenu->insertSeparator();
   folderMenu->insertItem(i18n("&Empty"), topLevelWidget(),
 			 SLOT(slotEmptyFolder()));
-  if (fti->folder && !fti->folder->isSystemFolder()) {
-  folderMenu->insertItem(i18n("&Remove"), topLevelWidget(),
-			 SLOT(slotRemoveFolder()));
+  int m3 = folderMenu->insertItem(i18n("&Remove"), topLevelWidget(),
+				  SLOT(slotRemoveFolder()));
+  if (fti->folder->isSystemFolder()) {
+    folderMenu->setItemEnabled(m1,false);
+    folderMenu->setItemEnabled(m2,false);
+    folderMenu->setItemEnabled(m3,false);
   }
   folderMenu->exec (p, 0);
   triggerUpdate();
@@ -558,7 +579,9 @@ void KMFolderTree::rightButtonPressed(QListViewItem *lvi, const QPoint &p, int)
 // Requires creating the appropriate subdirectory and show a dialog
 void KMFolderTree::addChildFolder()
 {
-  KMFolderTreeItem *fti = static_cast<KMFolderTreeItem*>(currentItem());
+  KMFolderTreeItem *fti = dynamic_cast<KMFolderTreeItem*>(currentItem());
+  if (!fti)
+    return;
   KMFolder *aFolder = fti->folder;
   if (fti->folder)
     if (!fti->folder->createChildFolder())
@@ -568,7 +591,7 @@ void KMFolderTree::addChildFolder()
   if (fti->folder)
     dir = fti->folder->child();
 
-  KMFolderDialog *d = 
+  KMFolderDialog *d =
     new KMFolderDialog(0, dir, topLevelWidget(), i18n("Create Child Folder") );
 
   if (d->exec()) {
@@ -639,6 +662,13 @@ void KMFolderTree::contentsDragEnterEvent( QDragEnterEvent *e )
   }
 
   oldCurrent = currentItem();
+  oldSelected = 0;
+  QListViewItemIterator it( this );
+  while (it.current()) {
+    if (it.current()->isSelected())
+      oldSelected = it.current();
+    ++it;
+  }
   setFocus();
 
   QListViewItem *i = itemAt( contentsToViewport(e->pos()) );
@@ -748,8 +778,9 @@ void KMFolderTree::contentsDragLeaveEvent( QDragLeaveEvent * )
     stopAutoScroll();
     dropItem = 0L;
 
-    //    setCurrentItem( oldCurrent );
-    //    setSelected( oldCurrent, TRUE );
+    setCurrentItem( oldCurrent );
+    if (oldSelected)
+      setSelected( oldSelected, TRUE );
     connect(this, SIGNAL(currentChanged(QListViewItem*)),
 	    this, SLOT(doFolderSelected(QListViewItem*)));
 }
@@ -762,9 +793,12 @@ void KMFolderTree::contentsDropEvent( QDropEvent *e )
     QListViewItem *item = itemAt( contentsToViewport(e->pos()) );
     if ( item ) {
         QString str;
-	KMFolderTreeItem *fti = static_cast<KMFolderTreeItem*>(item);
-	if ((fti != oldCurrent) && (fti->folder))
-	  emit folderDrop(fti->folder);
+	KMFolderTreeItem *fti = dynamic_cast<KMFolderTreeItem*>(item);
+	if (fti && (fti != oldSelected) && (fti->folder))
+        {
+          if (e->action() == QDropEvent::Copy) emit folderDropCopy(fti->folder);
+	  if (e->action() == QDropEvent::Move) emit folderDrop(fti->folder);
+        }
 	e->accept();
     } else
       e->ignore();
@@ -774,7 +808,8 @@ void KMFolderTree::contentsDropEvent( QDropEvent *e )
 
     clearSelection();
     setCurrentItem( oldCurrent );
-    setSelected( oldCurrent, TRUE );
+    if ( oldSelected )
+      setSelected( oldSelected, TRUE );
     connect(this, SIGNAL(currentChanged(QListViewItem*)),
 	    this, SLOT(doFolderSelected(QListViewItem*)));
     // End this wasn't necessary in QT 2.0.2
@@ -806,7 +841,7 @@ void KMFolderTree::keyPressEvent( QKeyEvent * e )
     }
 
     //Seems to behave sensibly even if ShiftButton is down, suprising
-    if (cntrl) {  
+    if (cntrl) {
       disconnect(this,SIGNAL(currentChanged(QListViewItem*)),
 		 this,SLOT(doFolderSelected(QListViewItem*)));
       switch (e->key()) {
@@ -830,22 +865,27 @@ void KMFolderTree::contentsMousePressEvent( QMouseEvent * e )
 {
   int b = e->state() & !ShiftButton & !ControlButton;
   QMouseEvent *f = new QMouseEvent( QEvent::MouseButtonPress,
-				    e->pos(),
-				    e->globalPos(),
-				    e->button(),
-				    b );
+                                   e->pos(),
+                                   e->globalPos(),
+                                   e->button(),
+                                   b );
   clearSelection();
   KMFolderTreeInherited::contentsMousePressEvent( f );
+  // Force current item to be selected for some reason in certain weird
+  // circumstances this is not always the case
+
+  if (currentItem())
+    setSelected( currentItem(), true );
 }
 
 void KMFolderTree::contentsMouseReleaseEvent( QMouseEvent * e )
 {
   int b = e->state() & !ShiftButton & !ControlButton;
   QMouseEvent *f = new QMouseEvent( QEvent::MouseButtonRelease,
-				    e->pos(),
-				    e->globalPos(),
-				    e->button(),
-				    b );
+                                   e->pos(),
+                                   e->globalPos(),
+                                   e->button(),
+                                   b );
   KMFolderTreeInherited::contentsMouseReleaseEvent( f );
 }
 
