@@ -114,6 +114,9 @@ public:
 
   /** Read config options. */
   virtual void readConfig(void);
+
+  /** Read color options and set palette. */
+  virtual void readColorConfig(void);
   
   // Refresh the list of message headers shown
   virtual void reset(void);
@@ -135,6 +138,8 @@ public:
   // top most visible item.
   virtual void setTopItemByIndex( int aMsgIdx );
   virtual void setNestedOverride( bool override );
+  // Double force items to always be open
+  virtual void setOpen ( QListViewItem *, bool );
 
 signals:
   // emitted when the list view item corresponding to this message
@@ -143,6 +148,8 @@ signals:
   // emitted when the list view item corresponding to this message
   // has been double clicked
   virtual void activated(KMMessage *);
+  // emitted when we might be about to delete messages
+  virtual void maybeDeleting();
 
 public slots:
   void workAroundQListViewLimitation();
@@ -171,17 +178,23 @@ public slots:
   void nextUnreadMessage();
   // Make the previous message header visible scrolling if necessary
   void prevUnreadMessage();
-  
+ 
 protected:
   static QPixmap *pixNew, *pixUns, *pixDel, *pixOld, *pixRep, *pixSent, 
     *pixQueued, *pixFwd;
 
+  // Look for color changes
+  virtual bool event(QEvent *e);
+
   // Overrided to support backing pixmap
   virtual void paintEmptyArea( QPainter * p, const QRect & rect );
 
   // Ensure the current item is visible
   void makeHeaderVisible();
 
+  /** Auxillary method to findUnread */
+  void findUnreadAux( KMHeaderItem*&, bool &, bool, bool );
+
   /** Find next/prev unread message. Starts at currentItem() if startAt
     is unset. */
   virtual int findUnread(bool findNext, int startAt=-1, bool onlyNew = FALSE);
@@ -259,7 +272,7 @@ private:
   static QIconSet *up, *down;   // Icons shown in header
   KMMenuToFolder mMenuToFolder; // Map menu id into a folder
 
-  int mousePressed;             // Drag and drop support
+  bool mousePressed;             // Drag and drop support
   QPoint presspos;              // ditto
 };
 
