@@ -5,7 +5,6 @@
 #include <kmfldsearch.h>
 
 #ifdef MALLOC_DEBUG
-#include <qmessagebox.h>
 #include <malloc.h>
 #endif
 
@@ -35,6 +34,12 @@
 
 #include <kparts/browserextension.h>
 
+#include <kaction.h>
+#include <kstdaction.h>
+#include <kedittoolbar.h>
+#include <kkeydialog.h>
+#include <kcharsets.h>
+
 #include "configuredialog.h"
 #include "kmbroadcaststatus.h"
 #include "kmfoldermgr.h"
@@ -55,8 +60,7 @@
 #include "kmsender.h"
 #include "kmaddrbookdlg.h"
 #include "kmaddrbook.h"
-#include "kwm.h"
-#include "addtoaddressbook.h"
+#include "kwin.h"
 
 #include <errno.h>
 #include <stdlib.h>
@@ -72,6 +76,7 @@ KMMainWin::KMMainWin(QWidget *, char *name) :
   // must be the first line of the constructor:
   searchWin = 0;
   mStartupDone = FALSE;
+  mbNewMBVisible = false;
   QListViewItem* idx;
   mIntegrated  = TRUE;
   mFolder = NULL;
@@ -86,13 +91,17 @@ KMMainWin::KMMainWin(QWidget *, char *name) :
 
   readPreConfig();
   createWidgets();
-  readConfig();
-  activatePanners();
 
   setupMenuBar();
-  setupToolBar();
   setupStatusBar();
 
+  applyMainWindowSettings(kapp->config(), "Main Window");
+  toolbarAction->setChecked(!toolBar()->isHidden());
+  statusbarAction->setChecked(!statusBar()->isHidden());
+
+  readConfig();
+  activatePanners();
+
   idx = mFolderTree->indexOfFolder(kernel->inboxFolder());
   if (idx!=0) {
     mFolderTree->setCurrentItem(idx);
@@ -101,8 +110,8 @@ KMMainWin::KMMainWin(QWidget *, char *name) :
 
   connect(kernel->msgSender(), SIGNAL(statusMsg(const QString&)),
 	  SLOT(statusMsg(const QString&)));
-  connect(kernel->acctMgr(), SIGNAL( newMail()),
-          SLOT( slotNewMail()));
+  connect(kernel->acctMgr(), SIGNAL( checkedMail(bool)),
+          SLOT( slotMailChecked(bool)));
 
   setCaption( i18n("KDE Mail Client") );
 
@@ -116,12 +125,13 @@ KMMainWin::~KMMainWin()
 {
   if (searchWin)
     delete searchWin;
-  writeConfig();      
+  writeConfig();
   writeFolderConfig();
-      
+
+  saveMainWindowSettings(kapp->config(), "Main Window");
+  kapp->config()->sync();
+
   if (mHeaders)    delete mHeaders;
-  if (mToolBar)    delete mToolBar;
-  if (mMenuBar)    delete mMenuBar;
   if (mStatusBar)  delete mStatusBar;
   if (mFolderTree) delete mFolderTree;
 }
@@ -135,6 +145,9 @@ void KMMainWin::readPreConfig(void)
 
   config->setGroup("Geometry");
   mLongFolderList = config->readBoolEntry("longFolderList", false);
+
+  config->setGroup("General");
+  mEncodingStr = config->readEntry("encoding", "");
 }
 
 
@@ -198,7 +211,7 @@ void KMMainWin::readConfig(void)
   }
 
   config->setGroup("Reader");
-  mHtmlPref = config->readBoolEntry( "htmlMail", true );
+  mHtmlPref = config->readBoolEntry( "htmlMail", false );
 
   config->setGroup("Geometry");
   mThreadPref = config->readBoolEntry( "nestedMessages", false );
@@ -226,8 +239,6 @@ void KMMainWin::readConfig(void)
   mBoxOnNew = config->readBoolEntry("msgbox-on-mail", false);
   mExecOnNew = config->readBoolEntry("exec-on-mail", false);
   mNewMailCmd = config->readEntry("exec-on-mail-cmd", "");
-  mUseKab = config->readBoolEntry("use-kab", false);
-  kernel->setUseKAB( mUseKab );
 
   // Re-activate panners
   if (mStartupDone)
@@ -287,7 +298,7 @@ void KMMainWin::writeConfig(void)
   config->writeEntry("Panners", s);
 
   config->setGroup("General");
-  config->writeEntry("use-kab", mUseKab);
+  config->writeEntry("encoding", mEncodingStr);
 }
 
 
@@ -330,6 +341,9 @@ void KMMainWin::createWidgets(void)
   accel->connectItem(accel->insertItem(Key_Right),
 		     mHeaders, SLOT(nextMessage()));
 
+  QString charset("iso8859-1");
+  mCodec = KGlobal::charsets()->codecForName(charset);
+
   // create HTML reader widget
   mMsgView = new KMReaderWin(pnrMsgView);
   connect(mMsgView, SIGNAL(statusMsg(const QString&)),
@@ -340,6 +354,8 @@ void KMMainWin::createWidgets(void)
 	  this, SLOT(slotUrlClicked(const KURL&,int)));
   connect(mMsgView, SIGNAL(showAtmMsg(KMMessage *)),
 	  this, SLOT(slotAtmMsg(KMMessage *)));
+  connect(mHeaders, SIGNAL(maybeDeleting()),
+	  mMsgView, SLOT(clearCache()));
   accel->connectItem(accel->insertItem(Key_Up),
 		     mMsgView, SLOT(slotScrollUp()));
   accel->connectItem(accel->insertItem(Key_Down),
@@ -361,6 +377,8 @@ void KMMainWin::createWidgets(void)
 	  this, SLOT(folderSelected(KMFolder*)));
   connect(mFolderTree, SIGNAL(folderDrop(KMFolder*)),
 	  this, SLOT(slotMoveMsgToFolder(KMFolder*)));
+  connect(mFolderTree, SIGNAL(folderDropCopy(KMFolder*)),
+          this, SLOT(slotCopyMsgToFolder(KMFolder*)));
 }
 
 
@@ -374,14 +392,14 @@ void KMMainWin::activatePanners(void)
     mMsgView->reparent( mHorizPanner, 0, QPoint( 0, 0 ) );
     mFolderTree->reparent( mVertPanner, 0, QPoint( 0, 0 ) );
     mVertPanner->moveToFirst( mFolderTree );
-    setView(mVertPanner);
+    setCentralWidget(mVertPanner);
   }
   else
   {
     mFolderTree->reparent( mVertPanner, 0, QPoint( 0, 0 ) );
     mHeaders->reparent( mVertPanner, 0, QPoint( 0, 0 ) );
     mMsgView->reparent( mHorizPanner, 0, QPoint( 0, 0 ) );
-    setView(mHorizPanner);
+    setCentralWidget(mHorizPanner);
   }
   mHorizPanner->setSizes( *mHorizPannerSep );
   mVertPanner->setSizes( *mVertPannerSep );
@@ -389,7 +407,7 @@ void KMMainWin::activatePanners(void)
   mVertPanner->setResizeMode( mFolderTree, QSplitter::KeepSize);
   if( mLongFolderList )
   {
-    mHorizPanner->setResizeMode( mHeaders, QSplitter::KeepSize);    
+    mHorizPanner->setResizeMode( mHeaders, QSplitter::KeepSize);
   }
   else
   {
@@ -397,15 +415,29 @@ void KMMainWin::activatePanners(void)
   }
 }
 
+//-----------------------------------------------------------------------------
+void KMMainWin::slotSetEncoding()
+{
+     mEncodingStr = mEncoding->currentText();
+     if (mEncoding->currentItem() == 0) // Auto
+       mCodec = 0;
+     else
+       mCodec = KGlobal::charsets()->codecForName( mEncodingStr );
+     mMsgView->setCodec(mCodec);
+     return;
+}
 
 //-----------------------------------------------------------------------------
 void KMMainWin::statusMsg(const QString& aText)
 {
-  mStatusBar->changeItem(" " + aText + " ", mMessageStatusId);
-  /* Just causes to much trouble with event driven repainting.
-  kapp->flushX();
-  kapp->processEvents(100);
-  */
+  QString text = " " + aText + " ";
+  int statusWidth = mStatusBar->width() - littleProgress->width()
+    - fontMetrics().maxWidth();
+
+  while (!text.isEmpty() && fontMetrics().width( text ) >= statusWidth)
+    text.truncate( text.length() - 1);
+
+  mStatusBar->changeItem( text, mMessageStatusId);
 }
 
 
@@ -432,7 +464,8 @@ void KMMainWin::slotClose()
 }
 
 //-------------------------------------------------------------------------
-void KMMainWin::slotSearch() {
+void KMMainWin::slotSearch()
+{
   if(!searchWin) {
     QString curFolder = "";
     if( mFolder )
@@ -440,12 +473,13 @@ void KMMainWin::slotSearch() {
     searchWin = new KMFldSearch(this, "Search", curFolder, false);
     connect(searchWin, SIGNAL(destroyed()),
 	    this, SLOT(slotSearchClosed()));
-  } 
+  }
 
   searchWin->show();
   KWin::setActiveWindow(searchWin->winId());
 }
 
+
 //-------------------------------------------------------------------------
 void KMMainWin::slotSearchClosed() {
   if(searchWin)
@@ -453,6 +487,13 @@ void KMMainWin::slotSearchClosed() {
 }
 
 
+//-------------------------------------------------------------------------
+void KMMainWin::slotFind() {
+  if( mMsgView )
+    mMsgView->slotFind();
+}
+
+
 //-----------------------------------------------------------------------------
 void KMMainWin::slotHelp()
 {
@@ -494,28 +535,14 @@ void KMMainWin::slotFilter()
 //-----------------------------------------------------------------------------
 void KMMainWin::slotAddrBook()
 {
-  if (!kernel->useKAB()) {
-    KMAddrBookEditDlg dlg( kernel->addrBook(), this );
-    dlg.exec();
-  }
-  else
-    debug("KMMainWin::slotAddrBook: not implemented.");
-}
-
-
-//-----------------------------------------------------------------------------
-void KMMainWin::slotUseKab()
-{
-  mUseKab = !mUseKab;
-  fileMenu->setItemChecked( mUseKabId, mUseKab );
-  kernel->setUseKAB( mUseKab );
+  KMAddrBookExternal::launch(this);
 }
 
 
 //-----------------------------------------------------------------------------
 void KMMainWin::slotUnimplemented()
 {
-  warning(i18n("Sorry, but this feature\nis still missing"));
+  qWarning(i18n("Sorry, but this feature\nis still missing"));
 }
 
 //-----------------------------------------------------------------------------
@@ -549,20 +576,10 @@ void KMMainWin::slotCheckMail()
 
  kernel->acctMgr()->checkMail(true);
 
- if(mSendOnCheck)
-   slotSendQueued();
  kernel->setCheckingMail(false);
 }
 
 
-//-----------------------------------------------------------------------------
-void KMMainWin::slotMenuActivated()
-{
-  if ( !actMenu->isVisible() )
-      getAccountMenu();
-}
-
-
 //-----------------------------------------------------------------------------
 void KMMainWin::slotCheckOneAccount(int item)
 {
@@ -579,26 +596,31 @@ void KMMainWin::slotCheckOneAccount(int item)
   kernel->acctMgr()->intCheckMail(item);
   // kbp->idle();
 
-  if(mSendOnCheck)
-    slotSendQueued();
-
   kernel->setCheckingMail(false);
 }
 
-void KMMainWin::slotNewMail() {
+void KMMainWin::slotMailChecked(bool newMail) {
+  if(mSendOnCheck)
+    slotSendQueued();
+
+  if (!newMail)
+    return;
 
   if (mBeepOnNew) {
     KApplication::beep();
   }
 
+  // FIXME: change system() to a KProcess
   if (mExecOnNew) {
     if (mNewMailCmd.length() > 0)
       system((const char *)mNewMailCmd);
   }
 
-  if (mBoxOnNew) {
+  if (mBoxOnNew && !mbNewMBVisible) {
+    mbNewMBVisible = true;
     KMessageBox::information(this, QString(i18n("You have new mail!")),
                                    QString(i18n("New Mail")));
+    mbNewMBVisible = false;
   }
 
   // Todo:
@@ -647,6 +669,8 @@ void KMMainWin::slotEmptyFolder()
 
   if (!mFolder) return;
 
+  mMsgView->clearCache();
+
   kernel->kbp()->busy();
 
   // begin of critical part
@@ -686,7 +710,7 @@ void KMMainWin::slotRemoveFolder()
   if (!mFolder) return;
   if (mFolder->isSystemFolder() || strcmp(mFolder->type(),"plain")!=0)
   {
-    warning(i18n("Cannot remove a\nsystem folder."));
+    qWarning(i18n("Cannot remove a\nsystem folder."));
     return;
   }
 
@@ -835,7 +859,7 @@ void KMMainWin::slotUndo()
 void KMMainWin::slotShowMsgSrc()
 {
   KMMessage* msg = mHeaders->getMsg(-1);
-  if (msg) msg->viewSource(i18n("Message as Plain Text"));
+  if (msg) msg->viewSource(i18n("Message as Plain Text"),mCodec);
 }
 
 
@@ -857,6 +881,12 @@ void KMMainWin::slotMoveMsgToFolder( KMFolder *dest)
   mHeaders->moveMsgToFolder(dest);
 }
 
+//-----------------------------------------------------------------------------
+void KMMainWin::slotCopyMsgToFolder( KMFolder *dest)
+{
+  mHeaders->copyMsgToFolder(dest);
+}
+
 //-----------------------------------------------------------------------------
 void KMMainWin::slotApplyFilters()
 {
@@ -889,10 +919,7 @@ void KMMainWin::slotSaveMsg()
 //-----------------------------------------------------------------------------
 void KMMainWin::slotSendQueued()
 {
-  if (kernel->msgSender()->sendQueued())
-    statusMsg(i18n("Queued messages successfully sent."));
-  else
-    statusMsg(i18n("Failed to send (some) queued messages."));
+  kernel->msgSender()->sendQueued();
 }
 
 
@@ -932,7 +959,6 @@ void KMMainWin::slotSetHeaderStyle(int id)
   readConfig(); // added this so _all_ the other widgets get this information
 }
 
-
 //-----------------------------------------------------------------------------
 void KMMainWin::folderSelected(KMFolder* aFolder)
 {
@@ -996,10 +1022,10 @@ void KMMainWin::slotMsgActivated(KMMessage *msg)
 void KMMainWin::slotAtmMsg(KMMessage *msg)
 {
   KMReaderWin *win;
-
   assert(msg != NULL);
-
   win = new KMReaderWin;
+  connect(win, SIGNAL(showAtmMsg(KMMessage *)),
+	  this, SLOT(slotAtmMsg(KMMessage *)));
   win->setAutoDelete(true); //delete on end
   showMsg(win, msg);
 }
@@ -1007,10 +1033,8 @@ void KMMainWin::slotAtmMsg(KMMessage *msg)
 
 void KMMainWin::showMsg(KMReaderWin *win, KMMessage *msg)
 {
-  KWM::setMiniIcon(win->winId(), kapp->miniIcon());
-  win->setCaption(msg->subject());
-
-  win->setMsg(msg);
+  KWin::setIcons(win->winId(), kapp->icon(), kapp->miniIcon());
+  win->setMsg(msg, true); // hack to work around strange QTimer bug
   win->resize(550,600);
 
   connect(win, SIGNAL(statusMsg(const QString&)),
@@ -1027,8 +1051,8 @@ void KMMainWin::showMsg(KMReaderWin *win, KMMessage *msg)
                      win, SLOT(slotScrollDown()));
   accel->connectItem(accel->insertItem(Key_Prior),
                      win, SLOT(slotScrollPrior()));
-  accel->connectItem(accel->insertItem(Key_S),
-                     win, SLOT(slotCopyMessage()));
+  //accel->connectItem(accel->insertItem(Key_S),
+  //                   win, SLOT(slotCopyMsg()));
   win->show();
 }
 
@@ -1061,16 +1085,30 @@ void KMMainWin::slotUrlClicked(const KURL &aUrl, int)
     msg->initHeader();
     msg->setTo(aUrl.path());
     QString query=aUrl.query();
-    if (query.left(9) == "?subject=")
-    {
-       msg->setSubject( KURL::decode_string(query.mid(9)) ); 
+    while (!query.isEmpty()) {
+      QString queryPart;
+      int secondQuery = query.find('?',1);
+      if (secondQuery != -1)
+	queryPart = query.left(secondQuery);
+      else
+	queryPart = query;
+      query = query.mid(queryPart.length());
+
+      if (queryPart.left(9) == "?subject=")
+	msg->setSubject( KURL::decode_string(queryPart.mid(9)) );
+      else if (queryPart.left(6) == "?body=")
+	// It is correct to convert to latin1() as URL should not contain
+	// anything except ascii.
+	msg->setBody( KURL::decode_string(queryPart.mid(6)).latin1() );
+      else if (queryPart.left(6) == "?cc=")
+	msg->setCc( KURL::decode_string(queryPart.mid(4)) );
     }
 
     win = new KMComposeWin(msg);
     win->show();
   }
-  else if ((aUrl.protocol() == "http") || (aUrl.protocol() ==  "ftp") ||
-           (aUrl.protocol() == "file"))
+  else if ((aUrl.protocol() == "http") || (aUrl.protocol() == "https") ||
+	   (aUrl.protocol() ==  "ftp") || (aUrl.protocol() == "file"))
   {
     statusMsg(i18n("Opening URL..."));
     // -- David : replacement for KFM::openURL
@@ -1128,15 +1166,7 @@ void KMMainWin::slotMailtoForward()
 //-----------------------------------------------------------------------------
 void KMMainWin::slotMailtoAddAddrBook()
 {
-  if (!kernel->useKAB()) {
-    if (mUrlCurrent.isEmpty()) return;
-    kernel->addrBook()->insert(mUrlCurrent.path());
-    statusMsg(i18n("Address added to addressbook."));
-  }
-  else {
-    AddToKabDialog dialog(mUrlCurrent.url(), kernel->KABaddrBook(), this);
-    dialog.exec();
-  }
+  KMAddrBookExternal::addEmail(mUrlCurrent.path(), this);
 }
 
 
@@ -1205,22 +1235,17 @@ void KMMainWin::slotMsgPopup(const KURL &aUrl, const QPoint& aPoint)
   else
   {
     // popup somewhere else on the document
-    menu->insertItem(i18n("&Reply..."), this,
-		     SLOT(slotReplyToMsg()));
-    menu->insertItem(i18n("Reply &All..."), this,
-		     SLOT(slotReplyAllToMsg()));
-    menu->insertItem(i18n("&Forward..."), this,
-		     SLOT(slotForwardMsg()), Key_F);
-    menu->insertItem(i18n("R&edirect..."), this,
-		     SLOT(slotRedirectMsg()));
+    replyAction->plug(menu);
+    replyAllAction->plug(menu);
+    forwardAction->plug(menu);
+    redirectAction->plug(menu);
     menu->insertSeparator();
     menu->insertItem(i18n("&Move..."), this,
-		     SLOT(slotMoveMsg()), Key_M);
+                     SLOT(slotMoveMsg()), Key_M);
     menu->insertItem(i18n("&Copy..."), this,
-		     SLOT(slotCopyMsg()), Key_C);
+                     SLOT(slotCopyMsg()), Key_C); 
     menu->insertSeparator();
-    menu->insertItem(i18n("&Delete"), this,
-		     SLOT(slotDeleteMsg()), Key_D);
+    deleteAction->plug(menu);
     menu->popup(aPoint, 0);
   }
 }
@@ -1229,14 +1254,14 @@ void KMMainWin::slotMsgPopup(const KURL &aUrl, const QPoint& aPoint)
 //-----------------------------------------------------------------------------
 void KMMainWin::getAccountMenu()
 {
-  QStrList actList;
+  QStringList actList;
 
   actMenu->clear();
   actList = kernel->acctMgr()->getAccounts();
-  QString tmp;
+  QStringList::Iterator it;
   int id = 0;
-  for(tmp = actList.first(); tmp ; tmp = actList.next(), id++)
-    actMenu->insertItem(tmp, id);
+  for(it = actList.begin(); it != actList.end() ; ++it, id++)
+    actMenu->insertItem(*it, id);
 }
 
 
@@ -1244,89 +1269,127 @@ void KMMainWin::getAccountMenu()
 void KMMainWin::setupMenuBar()
 {
   //----- File Menu
-  fileMenu = new QPopupMenu();
-  fileMenu->insertItem(i18n("&New Mail Client"), this,
-		       SLOT(slotNewMailReader()));
-  fileMenu->insertSeparator();
-  fileMenu->insertItem(i18n("Save &As..."), this,
-			   SLOT(slotSaveMsg()), KStdAccel::key(KStdAccel::Save));
-  fileMenu->insertItem(i18n("&Print..."), this,
-			   SLOT(slotPrintMsg()), KStdAccel::key(KStdAccel::Print));
-  fileMenu->insertSeparator();
-  fileMenu->insertItem(i18n("Compact all &folders"), kernel->folderMgr(),
-                        SLOT(compactAll()));
-  fileMenu->insertSeparator();
-  fileMenu->insertItem(i18n("Check &Mail"), this,
-		       SLOT(slotCheckMail()), CTRL+Key_L);
-  actMenu = new QPopupMenu();
+  (void) new KAction( i18n("&New Mail Client..."), 0, this, SLOT(slotNewMailReader()),
+		      actionCollection(), "new_mail_client" );
 
-  getAccountMenu();
+  saveAsAction = new KAction( i18n("Save &As..."), "filesave",
+    KStdAccel::key(KStdAccel::Save),
+    this, SLOT(slotSaveMsg()), actionCollection(), "save_as" );
 
+  KStdAction::print (this, SLOT(slotPrintMsg()), actionCollection());
+
+  (void) new KAction( i18n("Compact all &folders"), 0,
+		      kernel->folderMgr(), SLOT(compactAll()),
+		      actionCollection(), "compact_all_folders" );
+
+  (void) new KAction( i18n("Check &Mail"), "mail_get", CTRL+Key_L,
+		      this, SLOT(slotCheckMail()),
+		      actionCollection(), "check_mail" );
+
+  KActionMenu *actActionMenu = new
+    KActionMenu( i18n("Check Mail in"), actionCollection(), "check_mail_in" );
+
+  actMenu = actActionMenu->popupMenu();
   connect(actMenu,SIGNAL(activated(int)),this,SLOT(slotCheckOneAccount(int)));
-  connect(fileMenu,SIGNAL(highlighted(int)),this,SLOT(slotMenuActivated()));
 
-  fileMenu->insertItem(i18n("Check Mail in..."),actMenu);
+  (void) new KAction( i18n("&Send Queued"), 0, this,
+		      SLOT(slotSendQueued()), actionCollection(), "send_queued");
 
-  fileMenu->insertItem(i18n("Send &Queued"), this,
-		       SLOT(slotSendQueued()));
-  fileMenu->insertSeparator();
-  fileMenu->insertItem(i18n("Settings..."), this,
-		       SLOT(slotSettings()));
-  fileMenu->insertItem(i18n("&Addressbook..."), this,
-		       SLOT(slotAddrBook()));
-  mUseKabId = fileMenu->insertItem(i18n("Use KAB"), this,
-				   SLOT(slotUseKab()));
-  fileMenu->setItemChecked( mUseKabId, mUseKab );
-  fileMenu->insertItem(i18n("F&ilter..."), this,
-		       SLOT(slotFilter()));
-  fileMenu->insertSeparator();
-  fileMenu->insertItem(i18n("&Close"), this,
-			   SLOT(slotClose()), KStdAccel::key(KStdAccel::Close));
-  fileMenu->insertItem(i18n("&Quit"), this,
-			   SLOT(quit()), KStdAccel::key(KStdAccel::Quit));
+  (void) new KAction( i18n("Address &Book..."), "contents", 0, this,
+		      SLOT(slotAddrBook()), actionCollection(), "addressbook" );
+
+  KStdAction::close( this, SLOT(slotClose()), actionCollection());
+  //KStdAction::quit( this, SLOT(quit()), actionCollection());
 
   //----- Edit Menu
-  QPopupMenu *editMenu = new QPopupMenu();
-  editMenu->insertItem(i18n("&Undo"), this,
-                       SLOT(slotUndo()), KStdAccel::key(KStdAccel::Undo));
-  editMenu->insertItem(i18n("&Copy text"), this, SLOT(slotCopyText()),
-			   KStdAccel::key(KStdAccel::Copy));
-  editMenu->insertItem(i18n("&Delete"), this,
-			  SLOT(slotDeleteMsg()), Key_D);
-  // GS - should we put menu items to copy, cut & paste emails?
-  editMenu->insertSeparator();
-  editMenu->insertItem(i18n("&Search..."), this,
-                       SLOT(slotSearch()), Key_S);
-  editMenu->insertSeparator();
-  editMenu->insertItem(i18n("Mar&k all messages"), this,
-	               SLOT(slotMarkAll()), Key_K);
-#ifdef BROKEN
-  editMenu->insertItem(i18n("&Find..."), this,
-		       SLOT(slotUnimplemented()), KStdAccel::key(KStdAccel::Find));
-#endif
-  //----- Folder Menu
-  mFolderMenu = new QPopupMenu();
-  mFolderMenu->insertItem(i18n("&Create..."), this,
-			 SLOT(slotAddFolder()));
-  mFolderMenu->insertItem(i18n("&Modify..."), this,
-			 SLOT(slotModifyFolder()));
-  mFolderMenu->insertItem(i18n("C&ompact"), this,
-			 SLOT(slotCompactFolder()));
-  mFolderMenu->insertSeparator();
-  mFolderMenu->insertItem(i18n("&Empty"), this,
-			 SLOT(slotEmptyFolder()));
-  mFolderMenu->insertItem(i18n("&Remove"), this,
-			 SLOT(slotRemoveFolder()));
-  mFolderMenu->insertSeparator();
-  htmlId = mFolderMenu->insertItem("", this,
-				  SLOT(slotOverrideHtml()));
-  threadId = mFolderMenu->insertItem("", this,
-				    SLOT(slotOverrideThread()));
-  QObject::connect( mFolderMenu, SIGNAL( aboutToShow() ), 
-		    this, SLOT( updateFolderMenu() ));
+  KStdAction::undo( this, SLOT(slotUndo()), actionCollection());
 
-  //----- Message-Status Submenu
-  QPopupMenu *msgStatusMenu = new QPopupMenu;
+  (void) new KAction( i18n("&Copy text"), KStdAccel::key(KStdAccel::Copy), this,
+		      SLOT(slotCopyText()), actionCollection(), "copy_text" );
+
+  deleteAction = new KAction( i18n("&Delete"), "editdelete", Key_D, this,
+		      SLOT(slotDeleteMsg()), actionCollection(), "delete" );
+
+  (void) new KAction( i18n("&Search messages..."), "find", Key_S, this,
+		      SLOT(slotSearch()), actionCollection(), "search_messages" );
+
+  (void) new KAction( i18n("&Find in message..."), KStdAccel::key(KStdAccel::Find), this,
+		      SLOT(slotFind()), actionCollection(), "find_in_messages" );
+
+  (void) new KAction( i18n("Select all messages"), Key_K, this,
+		      SLOT(slotMarkAll()), actionCollection(), "mark_all_messages" );
+
+  //----- Folder Menu
+  (void) new KAction( i18n("&Create..."), 0, this,
+		      SLOT(slotAddFolder()), actionCollection(), "create" );
+
+  modifyFolderAction = new KAction( i18n("&Modify..."), 0, this,
+		      SLOT(slotModifyFolder()), actionCollection(), "modify" );
+
+  (void) new KAction( i18n("C&ompact"), 0, this,
+		      SLOT(slotCompactFolder()), actionCollection(), "compact" );
+
+  (void) new KAction( i18n("&Empty"), 0, this,
+		      SLOT(slotEmptyFolder()), actionCollection(), "empty" );
+
+  removeFolderAction = new KAction( i18n("&Remove..."), 0, this,
+		      SLOT(slotRemoveFolder()), actionCollection(), "remove" );
+
+  preferHtmlAction = new KToggleAction( i18n("Prefer HTML to plain text"), 0, this,
+		      SLOT(slotOverrideHtml()), actionCollection(), "prefer_html" );
+
+  threadMessagesAction = new KToggleAction( i18n("Thread messages"), 0, this,
+		      SLOT(slotOverrideThread()), actionCollection(), "thread_messages" );
+
+  //----- Message Menu
+  (void) new KAction( i18n("New Message..."), "filenew", KStdAccel::key(KStdAccel::New), this,
+		      SLOT(slotCompose()), actionCollection(), "new_message" );
+
+  (void) new KAction( i18n("&Next"), Key_N, mHeaders,
+		      SLOT(nextMessage()), actionCollection(), "next" );
+
+  (void) new KAction( i18n("Next unread"), "next", Key_Plus, mHeaders,
+		      SLOT(nextUnreadMessage()), actionCollection(), "next_unread" );
+
+  (void) new KAction( i18n("&Previous"), Key_P, mHeaders,
+		      SLOT(prevMessage()), actionCollection(), "previous" );
+
+  (void) new KAction( i18n("Previous unread"), "previous", Key_Minus, mHeaders,
+		      SLOT(prevUnreadMessage()), actionCollection(), "previous_unread" );
+
+  replyAction = new KAction( i18n("&Reply..."), "mail_reply", Key_R, this,
+		      SLOT(slotReplyToMsg()), actionCollection(), "reply" );
+
+  replyAllAction = new KAction( i18n("Reply &All..."), "mail_replyall",
+     Key_A, this, SLOT(slotReplyAllToMsg()), actionCollection(), "reply_all" );
+
+  forwardAction = new KAction( i18n("&Forward..."), "mail_forward", Key_F, this,
+		      SLOT(slotForwardMsg()), actionCollection(), "forward" );
+
+  redirectAction = new KAction( i18n("R&edirect..."), Key_E, this,
+		      SLOT(slotRedirectMsg()), actionCollection(), "redirect" );
+
+  bounceAction = new KAction( i18n("&Bounce..."), 0, this,
+		      SLOT(slotBounceMsg()), actionCollection(), "bounce" );
+
+  (void) new KAction( i18n("Send again..."), 0, this,
+		      SLOT(slotResendMsg()), actionCollection(), "send_again" );
+
+  //----- Message-Encoding Submenu
+  mEncoding = new KSelectAction( i18n( "Set &Encoding.." ), 0, this, SLOT( slotSetEncoding() ), actionCollection(), "encoding" );
+  QStringList encodings = KGlobal::charsets()->availableEncodingNames();
+  encodings.prepend( i18n( "Auto" ) );
+  mEncoding->setItems( encodings );
+  mEncoding->setCurrentItem(0);
+  if (encodings.findIndex( mEncodingStr ) != -1)
+      mEncoding->setCurrentItem(encodings.findIndex( mEncodingStr ));
+
+  (void) new KAction( i18n("Edi&t..."), Key_T, this,
+		      SLOT(slotEditMsg()), actionCollection(), "edit" );
+
+  KActionMenu *setStatusAction = new KActionMenu( i18n("Set status" ),
+					     actionCollection(), "set_status" );
+  QPopupMenu *msgStatusMenu = setStatusAction->popupMenu();
   connect(msgStatusMenu, SIGNAL(activated(int)), this,
 	  SLOT(slotSetMsgStatus(int)));
   msgStatusMenu->insertItem(i18n("New"), (int)KMMsgStatusNew);
@@ -1336,64 +1399,33 @@ void KMMainWin::setupMenuBar()
   msgStatusMenu->insertItem(i18n("Queued"), (int)KMMsgStatusQueued);
   msgStatusMenu->insertItem(i18n("Sent"), (int)KMMsgStatusSent);
 
-  //----- Message Menu
-  messageMenu = new QPopupMenu;
-  QObject::connect( messageMenu, SIGNAL( aboutToShow() ),
-		    this, SLOT( updateMessageMenu() ));
+  KActionMenu *moveActionMenu = new KActionMenu( i18n("&Move to" ),
+					     actionCollection(), "move_to" );
+  moveMenu = moveActionMenu->popupMenu();
+
+  KActionMenu *copyActionMenu = new KActionMenu( i18n("&Copy to" ),
+					     actionCollection(), "copy_to" );
+  copyMenu = copyActionMenu->popupMenu();
+
+  (void) new KAction( i18n("Apply filters"), CTRL+Key_J, this,
+		      SLOT(slotApplyFilters()), actionCollection(), "apply_filters" );
+
+  (void) new KAction( i18n("View Source..."), 0, this,
+		      SLOT(slotShowMsgSrc()), actionCollection(), "view_source" );
 
-  messageMenu->insertItem(i18n("New &Message"), this,
-			  SLOT(slotCompose()), KStdAccel::key(KStdAccel::New));
-  messageMenu->insertSeparator();
-  messageMenu->insertItem(i18n("&Next"), mHeaders,
-			  SLOT(nextMessage()), Key_N);
-  messageMenu->insertItem(i18n("Next unread"), mHeaders,
-			  SLOT(nextUnreadMessage()), Key_Plus);
-  messageMenu->insertItem(i18n("&Previous"), mHeaders,
-			  SLOT(prevMessage()), Key_P);
-  messageMenu->insertItem(i18n("Previous unread"), mHeaders,
-			  SLOT(prevUnreadMessage()), Key_Minus);
-  messageMenu->insertSeparator();
-  messageMenu->insertItem(i18n("&Reply..."), this,
-			  SLOT(slotReplyToMsg()), Key_R);
-  messageMenu->insertItem(i18n("Reply &All..."), this,
-			  SLOT(slotReplyAllToMsg()), Key_A);
-  messageMenu->insertItem(i18n("&Forward..."), this,
-			  SLOT(slotForwardMsg()), Key_F);
-  messageMenu->insertItem(i18n("R&edirect..."), this,
-			  SLOT(slotRedirectMsg()), Key_E);
-  messageMenu->insertItem(i18n("&Bounce..."), this,
-			  SLOT(slotBounceMsg()));
-  messageMenu->insertItem(i18n("Send again..."), this,
-			  SLOT(slotResendMsg()));
-  messageMenu->insertSeparator();
-  messageMenu->insertItem(i18n("Edi&t..."),this,
-			  SLOT(slotEditMsg()), Key_T);
-  messageMenu->insertItem(i18n("&Set Status"), msgStatusMenu);
-  moveId = messageMenu->insertItem(i18n("&Move..."), this,
-			  SLOT(slotMoveMsg()));
-  copyId = messageMenu->insertItem(i18n("&Copy..."), this,
-			  SLOT(slotCopyMsg()));
-  messageMenu->insertSeparator();
-  messageMenu->insertItem(i18n("Apply filters"), this,
-			  SLOT(slotApplyFilters()), CTRL+Key_J);
-  messageMenu->insertSeparator();
-  messageMenu->insertItem(i18n("View Source..."), this,
-			  SLOT(slotShowMsgSrc()));
 
   //----- View Menu
-  mViewMenu = new QPopupMenu();
+  KActionMenu *viewMenuAction = new
+    KActionMenu( i18n("things to show", "&View"), actionCollection(), "view" );
+
+  mViewMenu = viewMenuAction->popupMenu();
   mViewMenu->setCheckable(TRUE);
   connect(mViewMenu,SIGNAL(activated(int)),SLOT(slotSetHeaderStyle(int)));
-  mViewMenu->insertItem(i18n("&Brief Headers"),
-		       KMReaderWin::HdrBrief);
-  mViewMenu->insertItem(i18n("&Fancy Headers"),
-		       KMReaderWin::HdrFancy);
-  mViewMenu->insertItem(i18n("&Standard Headers"),
-		       KMReaderWin::HdrStandard);
-  mViewMenu->insertItem(i18n("&Long Headers"),
-		       KMReaderWin::HdrLong);
-  mViewMenu->insertItem(i18n("&All Headers"),
-		       KMReaderWin::HdrAll);
+  mViewMenu->insertItem(i18n("&Brief Headers"), KMReaderWin::HdrBrief);
+  mViewMenu->insertItem(i18n("&Fancy Headers"), KMReaderWin::HdrFancy);
+  mViewMenu->insertItem(i18n("&Standard Headers"), KMReaderWin::HdrStandard);
+  mViewMenu->insertItem(i18n("&Long Headers"), KMReaderWin::HdrLong);
+  mViewMenu->insertItem(i18n("&All Headers"), KMReaderWin::HdrAll);
   mViewMenu->insertSeparator();
   mViewMenu->insertItem(i18n("Iconic Attachments"),
 		       KMReaderWin::HdrAll + KMReaderWin::IconicAttmnt);
@@ -1404,86 +1436,67 @@ void KMMainWin::setupMenuBar()
   mViewMenu->setItemChecked((int)mMsgView->headerStyle(), TRUE);
   mViewMenu->setItemChecked((int)mMsgView->attachmentStyle()+5, TRUE);
 
-  //----- Help Menu
-  QPopupMenu *mHelpMenu = helpMenu(aboutText, false);
+  //----- Settings Menu
+  toolbarAction = KStdAction::showToolbar(this, SLOT(slotToggleToolBar()),
+    actionCollection());
+  statusbarAction = KStdAction::showStatusbar(this, SLOT(slotToggleStatusBar()),
+    actionCollection());
+  KStdAction::keyBindings(this, SLOT(slotEditKeys()), actionCollection());
+  KStdAction::configureToolbars(this, SLOT(slotEditToolbars()), actionCollection());
+  //  KStdAction::preferences(this, SLOT(slotSettings()), actionCollection());
+ (void) new KAction( i18n("Configuration..."), 0, this,
+		     SLOT(slotSettings()), actionCollection(), "settings" );
 
-  //----- Menubar
-  mMenuBar  = new KMenuBar(this);
-  mMenuBar->insertItem(i18n("&File"), fileMenu);
-  mMenuBar->insertItem(i18n("&Edit"), editMenu);
-  mMenuBar->insertItem(i18n("F&older"), mFolderMenu);
-  mMenuBar->insertItem(i18n("&Message"), messageMenu);
-  mMenuBar->insertItem(i18n("&View"), mViewMenu);
-  mMenuBar->insertSeparator();
-  mMenuBar->insertItem(i18n("&Help"), mHelpMenu);
 
-  setMenu(mMenuBar);
+  (void) new KAction( i18n("F&ilter Rules..."), 0, this,
+ 		      SLOT(slotFilter()), actionCollection(), "filter" );
+
+  createGUI( "kmmainwin.rc", false );
+
+  QObject::connect( guiFactory()->container("file", this),
+		    SIGNAL( aboutToShow() ), this, SLOT( getAccountMenu() ));
+
+  QObject::connect( guiFactory()->container("folder", this),
+		    SIGNAL( aboutToShow() ), this, SLOT( updateFolderMenu() ));
+
+  QObject::connect( guiFactory()->container("message", this),
+		    SIGNAL( aboutToShow() ), this, SLOT( updateMessageMenu() ));
+
+  conserveMemory();
 }
 
 
 //-----------------------------------------------------------------------------
-void KMMainWin::setupToolBar()
+
+void KMMainWin::slotToggleToolBar()
 {
-  mToolBar = new KToolBar(this);
+  if(toolBar("mainToolBar")->isVisible())
+    toolBar("mainToolBar")->hide();
+  else
+    toolBar("mainToolBar")->show();
+}
 
-#ifdef MALLOC_DEBUG
-  mToolBar->insertButton(BarIcon("filenew"), 0,
-			SIGNAL(clicked()), this,
-			SLOT(slotMemInfo()), TRUE,
-			i18n("Malloc info"));
-#endif
+void KMMainWin::slotToggleStatusBar()
+{
+  if (statusBar()->isVisible())
+    statusBar()->hide();
+  else
+    statusBar()->show();
+}
 
-  mToolBar->insertButton(BarIcon("filenew"), 0,
-			SIGNAL(clicked()), this,
-			SLOT(slotCompose()), TRUE,
-			i18n("Compose new message"));
+void KMMainWin::slotEditToolbars()
+{
+  KEditToolbar dlg(actionCollection(), "kmmainwin.rc");
 
-  mToolBar->insertButton(BarIcon("filesave"), 0,
-			SIGNAL(clicked()), this,
-			SLOT(slotSaveMsg()), TRUE,
-			i18n("Save message to file"));
+  if (dlg.exec() == true)
+  {
+    createGUI("kmmainwin.rc");
+  }
+}
 
-  mToolBar->insertButton(BarIcon("fileprint"), 0,
-			SIGNAL(clicked()), this,
-			SLOT(slotPrintMsg()), TRUE,
-			i18n("Print message"));
-
-  mToolBar->insertSeparator();
-
-  mToolBar->insertButton(BarIcon("message_get"), 0,
-			SIGNAL(clicked()), this,
-			SLOT(slotCheckMail()), TRUE,
-			i18n("Get new messages"));
-  mToolBar->insertSeparator();
-
-  mToolBar->insertButton(BarIcon("message_reply"), 0,
-			SIGNAL(clicked()), this,
-			SLOT(slotReplyToMsg()), TRUE,
-			i18n("Reply to author"));
-
-  mToolBar->insertButton(BarIcon("message_replyall"), 0,
-			SIGNAL(clicked()), this,
-			SLOT(slotReplyAllToMsg()), TRUE,
-			i18n("Reply to all recipients"));
-
-  mToolBar->insertButton(BarIcon("message_forward"), 0,
-			SIGNAL(clicked()), this,
-			SLOT(slotForwardMsg()), TRUE,
-			i18n("Forward message"));
-
-  mToolBar->insertButton(BarIcon("message_delete"), 0,
-			SIGNAL(clicked()), this,
-			SLOT(slotDeleteMsg()), TRUE,
-			i18n("Delete message"));
-
-  mToolBar->insertSeparator();
-  mToolBar->insertButton(BarIcon("contents"), 0,
-			SIGNAL(clicked()), this,
-			SLOT(slotAddrBook()), TRUE,
-			i18n("Open addressbook..."));
-
-
-  addToolBar(mToolBar);
+void KMMainWin::slotEditKeys()
+{
+  KKeyDialog::configureKeys(actionCollection(), xmlFile(), true, this);
 }
 
 
@@ -1538,76 +1551,71 @@ void KMMainWin::copySelectedToFolder(int menuId )
 QPopupMenu* KMMainWin::folderToPopupMenu(KMFolderDir* aFolderDir,
 					 bool move,
 					 QObject *receiver,
-					 KMMenuToFolder *aMenuToFolder)
+					 KMMenuToFolder *aMenuToFolder,
+					 QPopupMenu *menu )
 {
   KMFolderNode *folderNode;
   KMFolder* folder;
 
-  QPopupMenu *msgMoveMenu = new QPopupMenu;
   if (move)
-    connect(msgMoveMenu, SIGNAL(activated(int)), receiver,
-	    SLOT(moveSelectedToFolder(int)));
-  else
-    connect(msgMoveMenu, SIGNAL(activated(int)), receiver,
-	    SLOT(copySelectedToFolder(int)));
+  {
+    disconnect(menu, SIGNAL(activated(int)), receiver,
+           SLOT(moveSelectedToFolder(int)));
+    connect(menu, SIGNAL(activated(int)), receiver,
+             SLOT(moveSelectedToFolder(int)));
+  } else {
+    disconnect(menu, SIGNAL(activated(int)), receiver,
+           SLOT(copySelectedToFolder(int)));
+    connect(menu, SIGNAL(activated(int)), receiver,
+             SLOT(copySelectedToFolder(int)));
+  }
   for (folderNode = aFolderDir->first();
        folderNode != NULL;
        folderNode = aFolderDir->next())
     if (!folderNode->isDir()) {
       folder = static_cast<KMFolder*>(folderNode);
 
-      int menuId = msgMoveMenu->insertItem(folder->label());
+      int menuId = menu->insertItem(folder->label());
       aMenuToFolder->insert( menuId, folder );
 
       KMFolderDir *child = folder->child();
       if (child && child->first()) {
 	QPopupMenu *subMenu = folderToPopupMenu( child, move, receiver,
-						 aMenuToFolder );
+						 aMenuToFolder,
+						 new QPopupMenu() );
 	// add an item to the top of the submenu somehow subMenu
-	msgMoveMenu->insertItem(i18n(folder->label() + " child"), subMenu);
+	menu->insertItem(i18n("%1 child").arg(folder->label()), subMenu);
       }
     }
-  return msgMoveMenu;
+  return menu;
 }
 
 
 //-----------------------------------------------------------------------------
 void KMMainWin::updateMessageMenu()
 {
-  int moveIndex = messageMenu->indexOf( moveId );
-  int copyIndex = messageMenu->indexOf( copyId );
-  messageMenu->removeItem( moveId );
-  messageMenu->removeItem( copyId );
-
   KMFolderDir *dir = &kernel->folderMgr()->dir();
   mMenuToFolder.clear();
-  QPopupMenu *msgMoveMenu;
-  msgMoveMenu =  folderToPopupMenu( dir, TRUE, this, &mMenuToFolder );
-  QPopupMenu *msgCopyMenu;
-  msgCopyMenu = folderToPopupMenu( dir, FALSE, this, &mMenuToFolder );
-  moveId = messageMenu->insertItem(i18n("&Move to"), msgMoveMenu, -1,
-				   moveIndex );
-  copyId = messageMenu->insertItem(i18n("&Copy to"), msgCopyMenu, -1,
-				   copyIndex );
+  moveMenu->clear();
+  folderToPopupMenu( dir, TRUE, this, &mMenuToFolder, moveMenu );
+  copyMenu->clear();
+  folderToPopupMenu( dir, FALSE, this, &mMenuToFolder, copyMenu );
 }
 
 
+//-----------------------------------------------------------------------------
 void KMMainWin::updateFolderMenu()
 {
-  mFolderMenu->setItemEnabled(htmlId, mFolder ? true : false);
-  mFolderMenu->setItemEnabled(threadId, mFolder ? true : false);
+  modifyFolderAction->setEnabled( mFolder ? !mFolder->isSystemFolder()
+    : false );
+  removeFolderAction->setEnabled( mFolder ? !mFolder->isSystemFolder()
+    : false );
+  preferHtmlAction->setEnabled( mFolder ? true : false );
+  threadMessagesAction->setEnabled( true );
+  threadMessagesAction->setEnabled( mFolder ? true : false );
 
-  if (mHtmlPref)
-    mFolderMenu->changeItem(htmlId, i18n( "Prefer HTML to plain text" ));
-  else
-    mFolderMenu->changeItem(htmlId, i18n( "Prefer plain text to HTML" ));
-  mFolderMenu->setItemChecked(htmlId, !mFolderHtmlPref);
-  
-  if (mThreadPref)
-    mFolderMenu->changeItem(threadId, i18n( "Thread messages" ));
-  else
-    mFolderMenu->changeItem(threadId, i18n( "Don't thread messages" ));
-  mFolderMenu->setItemChecked(threadId, !mFolderThreadPref);
+  preferHtmlAction->setChecked( mHtmlPref ? !mFolderHtmlPref : mFolderHtmlPref );
+  threadMessagesAction->setChecked( mThreadPref ? !mFolderThreadPref : mFolderThreadPref );
 }
 
 #ifdef MALLOC_DEBUG
@@ -1639,7 +1647,7 @@ void KMMainWin::slotMemInfo() {
 		      "Max. freeable memory      : %5\n")
     .arg(mi.hblks).arg(fmt(mi.uordblks)).arg(fmt(mi.fordblks))
     .arg(fmt(mi.arena)).arg(fmt(mi.keepcost));
-  QMessageBox::information(0, "Malloc information", s);
+  KMessageBox::information(0, s, "Malloc information", s);
 #endif
 }
 
