@@ -9,6 +9,7 @@
 #include "kmmsginfo.h"
 #include "kmreaderwin.h"
 #include "kpgp.h"
+
 #ifndef KRN
 #include "kmfolder.h"
 #include "kmundostack.h"
@@ -33,7 +34,8 @@
 #include <time.h>
 #include <klocale.h>
 #include <kglobal.h>
-#include <kwm.h>
+#include <kcharsets.h>
+#include <kwin.h>
 
 #include "kmmsgpart.h" // for encodeBase64
 
@@ -44,12 +46,15 @@
 
 
 static DwString emptyString("");
-static QCString result;
 
 // Values that are set from the config file with KMMessage::readConfig()
-static QString sReplyStr, sForwardStr, sReplyAllStr, sIndentPrefixStr;
-static bool sSmartQuote;
-static int sWrapCol, sHdrStyle;
+static QString sReplyStr, sReplyAllStr, sIndentPrefixStr;
+static bool sSmartQuote, sReplaceSubjPrefix;
+static int sWrapCol;
+static QStringList sReplySubjPrefixes;
+
+QString KMMessage::sForwardStr = "";
+int KMMessage::sHdrStyle = KMReaderWin::HdrFancy;
 
 /* Start functions added for KRN */
 
@@ -68,7 +73,7 @@ const QString KMMessage::followup(void) const
     return decodeRFC1522String(header.FollowupTo().AsString().c_str());
   else
   {
-    if (header.HasNewsgroups()) 
+    if (header.HasNewsgroups())
       return decodeRFC1522String(header.Newsgroups().AsString().c_str());
     else return "";
   }
@@ -194,7 +199,7 @@ bool KMMessage::isMessage(void) const
 {
   return TRUE;
 }
-   
+
 
 //-----------------------------------------------------------------------------
 const QString KMMessage::asString(void)
@@ -204,8 +209,7 @@ const QString KMMessage::asString(void)
     mNeedsAssembly = FALSE;
     mMsg->Assemble();
   }
-  result = mMsg->AsString().c_str();
-  return result;
+  return mMsg->AsString().c_str();
 }
 
 //-----------------------------------------------------------------------------
@@ -236,6 +240,7 @@ void KMMessage::fromString(const QString& aStr, bool aSetStatus)
   const char* strPos;
   char* resultPos;
   char ch;
+  QCString result;
 
   if (mMsg) delete mMsg;
   mMsg = new DwMessage;
@@ -258,6 +263,7 @@ void KMMessage::fromString(const QString& aStr, bool aSetStatus)
     setStatus(headerField("Status"), headerField("X-Status"));
 
   mNeedsAssembly = FALSE;
+  KMMessageInherited::setDate(date());
 }
 
 
@@ -265,63 +271,61 @@ void KMMessage::fromString(const QString& aStr, bool aSetStatus)
 const QString KMMessage::formatString(const QString& aStr) const
 {
   QString result, str;
-  const char* pos;
-  char ch, cstr[64];
-  time_t tm;
-  int i;
+  QChar ch;
+  uint j;
 
-  if (aStr.isEmpty()) return aStr;
+  if (aStr.isEmpty())
+    return aStr;
 
-  for (pos=aStr.data(); *pos; )
-  {
-    ch = *pos++;
-    if (ch=='%')
-    {
-      ch = *pos++;
-      switch (ch)
-      {
+  for (uint i=0; i<aStr.length();) {
+    ch = aStr[i++];
+    if (ch == '%') {
+      ch = aStr[i++];
+      switch ((char)ch) {
       case 'D':
 	/* I'm not too sure about this change. Is it not possible
 	   to have a long form of the date used? I don't
 	   like this change to a short XX/XX/YY date format.
-	   At least not for the default. -sanders
+	   At least not for the default. -sanders */
+        {
         QDateTime datetime;
 	datetime.setTime_t(date());
-        result += KGlobal::locale()->formatDate(datetime.date());
-	*/
-	tm = date();
-	strftime(cstr, 63, "%a, %d %b %Y", localtime(&tm));
-	result += cstr;
-	break;
+        result += KGlobal::locale()->formatDateTime(datetime, false);
+        }
+        break;
       case 'F':
-	result += stripEmailAddr(from());
-	break;
+        result += stripEmailAddr(from());
+        break;
       case 'f':
-	str = stripEmailAddr(from());
-	for (i=0; str[i]>' '; i++)
-	  ;
-	for (; i < (int)str.length() && str[i] <= ' '; i++)
-	  ;
-	result += str[0];
-	if (str[i]>' ') result += str[i];
-	else if (str[1]>' ') result += str[1];
-	break;
+        str = stripEmailAddr(from());
+
+        for (j=0; str[j]>' '; j++)
+          ;
+        for (; j < str.length() && str[j] <= ' '; j++)
+          ;
+        result += str[0];
+        if (str[j]>' ')
+          result += str[j];
+        else
+          if (str[1]>' ')
+            result += str[1];
+        break;
       case 'S':
-	result += subject();
-	break;
+        result += subject();
+        break;
       case '_':
-	result += ' ';
-	break;
+        result += ' ';
+        break;
       case '%':
-	result += '%';
-	break;
+        result += '%';
+        break;
       default:
-	result += '%';
-	result += ch;
-	break;
+        result += '%';
+        result += ch;
+        break;
       }
-    }
-    else result += ch;
+    } else
+      result += ch;
   }
   return result;
 }
@@ -347,7 +351,7 @@ static QString splitLine( QString &line)
     {
        QChar c = line[i];
        if ((c == '>') || (c == ':') || (c == '|'))
-          j = i+1;  
+          j = i+1;
        else if ((c != ' ') && (c != '\t'))
           break;
        i++;
@@ -372,7 +376,7 @@ static QString splitLine( QString &line)
 static QString flowText(QString &text, const QString& indent, int maxLength)
 {
 // printf("flowText: \"%s\"\n", text.ascii());
-   maxLength--; 
+   maxLength--;
    if (text.isEmpty())
    {
       return indent+"<NULL>\n";
@@ -402,17 +406,17 @@ static QString flowText(QString &text, const QString& indent, int maxLength)
       QString line = text.left(i);
       if (i < (int) text.length())
          text = text.mid(i);
-      else 
+      else
          text = QString::null;
 
       result += indent + line + '\n';
-      
-      if (text.isEmpty()) 
+
+      if (text.isEmpty())
          return result;
    }
 }
 
-static bool flushPart(QString &msg, QStringList &part, 
+static bool flushPart(QString &msg, QStringList &part,
                       const QString &indent, int maxLength)
 {
    maxLength -= indent.length();
@@ -423,9 +427,9 @@ static bool flushPart(QString &msg, QStringList &part,
    {
       part.remove(part.fromLast());
    }
- 
+
 //printf("Start of part.\n");
-      
+
    QString text;
    for(QStringList::Iterator it2 = part.begin();
        it2 != part.end();
@@ -463,12 +467,12 @@ static bool flushPart(QString &msg, QStringList &part,
 }
 
 
-static void smartQuote( QString &msg, const QString &ownIndent, 
+static void smartQuote( QString &msg, const QString &ownIndent,
                         int maxLength, bool aStripSignature )
 {
   QStringList part;
   QString startOfSig1;
-  QString startOfSig2 = ownIndent+"--";
+  QString startOfSig2 = ownIndent+"-- ";
   QString oldIndent;
   bool firstPart = true;
   int i = 0;
@@ -476,7 +480,7 @@ static void smartQuote( QString &msg, const QString &ownIndent,
 
 //printf("Smart Quoting.\n");
 
-  while( startOfSig2.left(1) == "\n") 
+  while( startOfSig2.left(1) == "\n")
      startOfSig2 = startOfSig2.mid(1);
   startOfSig1 = startOfSig2 + ' ';
 
@@ -484,13 +488,13 @@ static void smartQuote( QString &msg, const QString &ownIndent,
   while ((i = msg.find('\n', l)) != -1)
   {
      if (i == l)
-        lines.append( QString::null);
+        lines.append(QString::null);
      else
-        lines.append( msg.mid(l, i-l));
+        lines.append(msg.mid(l, i-l));
      l = i+1;
   }
-  if (l <= (int) msg.length());
-     lines.append( msg.mid(l));
+  if (l <= (int)msg.length())
+    lines.append(msg.mid(l));
   msg = QString::null;
   for(QStringList::Iterator it = lines.begin();
       it != lines.end();
@@ -500,20 +504,20 @@ static void smartQuote( QString &msg, const QString &ownIndent,
 
      if (aStripSignature)
      {
-        if (line == startOfSig1) break; // Start of signature 
-        if (line == startOfSig2) break; // Start of malformed signature 
+        if (line == startOfSig1) break; // Start of signature
+        if (line == startOfSig2) break; // Start of malformed signature
      }
 
      QString indent = splitLine( line );
 
 //     printf("Quoted Line = \"%s\" \"%s\"\n", line.ascii(), indent.ascii());
-     if ( line.isEmpty()) 
+     if ( line.isEmpty())
      {
         if (!firstPart)
            part.append(QString::null);
         continue;
      };
- 
+
      if (firstPart)
      {
         oldIndent = indent;
@@ -541,7 +545,7 @@ static void smartQuote( QString &msg, const QString &ownIndent,
            if (oldIndent.length() > indent.length())
               msg += indent + "\n";
            else
-              msg += oldIndent + "\n";  
+              msg += oldIndent + "\n";
         }
         if (!fromLine.isEmpty())
         {
@@ -557,7 +561,7 @@ static void smartQuote( QString &msg, const QString &ownIndent,
 
 
 //-----------------------------------------------------------------------------
-const QString KMMessage::asQuotedString(const QString& aHeaderStr,
+const QCString KMMessage::asQuotedString(const QString& aHeaderStr,
 					const QString& aIndentStr,
 					bool aIncludeAttach,
                                         bool aStripSignature) const
@@ -566,87 +570,94 @@ const QString KMMessage::asQuotedString(const QString& aHeaderStr,
   QString headerStr;
   KMMessagePart msgPart;
   QRegExp reNL("\\n");
-  QString nlIndentStr;
+  QString indentStr;
   bool isInline;
   int i;
 
-  nlIndentStr = "\n" + formatString(aIndentStr);
+  QString cset = charset();
+  QTextCodec* codec;
+  if (!cset.isEmpty() && !cset.isNull())
+    codec = KGlobal::charsets()->codecForName(charset());
+  else
+    codec = QTextCodec::codecForLocale();
+
+  indentStr = formatString(aIndentStr);
   headerStr = formatString(aHeaderStr);
 
+
   // Quote message. Do not quote mime message parts that are of other
   // type than "text".
-  if (numBodyParts() == 0)
-  {
-     Kpgp* pgp = Kpgp::getKpgp();
-     assert(pgp != NULL);
-     result = bodyDecoded();
-     pgp->setMessage(result);
-     if(pgp->isEncrypted())
-     {
-       pgp->decrypt();
-       result = QString(pgp->message()).stripWhiteSpace();
-     } else {
-       result = result.stripWhiteSpace();
-     }
-     result.replace(reNL,nlIndentStr);
-     result = formatString(aIndentStr) + result + '\n';
-     if (sSmartQuote)
-        smartQuote(result, nlIndentStr, sWrapCol, aStripSignature);
-  }
-  else
-  {
+  if (numBodyParts() == 0) {
+    Kpgp* pgp = Kpgp::getKpgp();
+    assert(pgp != NULL);
+    result = codec->toUnicode(bodyDecoded());
+
+    pgp->setMessage(result);
+    if(pgp->isEncrypted()) {
+      pgp->decrypt();
+      result = pgp->message().stripWhiteSpace();
+    } else
+      result = result.stripWhiteSpace();
+
+    result.replace(reNL, '\n' + indentStr);
+    result = indentStr + result + '\n';
+
+    if (sSmartQuote)
+      smartQuote(result, '\n' + indentStr, sWrapCol, aStripSignature);
+  } else {
     result = "";
-    for (i = 0; i < numBodyParts(); i++)
-    {
+    for (i = 0; i < numBodyParts(); i++) {
       bodyPart(i, &msgPart);
 
-      if (i==0) isInline = TRUE;
-      else isInline = (stricmp(msgPart.contentDisposition(),"inline")==0);
+      if (i==0)
+        isInline = TRUE;
+      else
+        isInline = (stricmp(msgPart.contentDisposition(), "inline") == 0);
 
-      if (isInline)
-      {
-	if (stricmp(msgPart.typeStr(),"text")==0 || 
-	    stricmp(msgPart.typeStr(),"message")==0)
-	{
-	  result += nlIndentStr;
+      if (isInline) {
+        if (stricmp(msgPart.typeStr(),"text") == 0 ||
+            stricmp(msgPart.typeStr(),"message") == 0) {
+          result += '\n';
+          result += indentStr;
           Kpgp* pgp = Kpgp::getKpgp();
           assert(pgp != NULL);
-	  QString part;
-          if ((pgp->setMessage(msgPart.bodyDecoded())) &&
+          QString part;
+          if ((pgp->setMessage(codec->toUnicode(msgPart.bodyDecoded()))) &&
               (pgp->isEncrypted()) &&
-              (pgp->decrypt()))
-	  {
-	    part = QString(pgp->message());
-	  }
-          else
-	  {
-	    part = QString(msgPart.bodyDecoded());
-	    //	    debug ("part\n" + part ); inexplicably crashes -sanders
-	  }
-          part.replace(reNL,nlIndentStr);
-          part = formatString(aIndentStr) + part + '\n';
+              (pgp->decrypt())) {
+            part = pgp->message();
+          } else {
+            part = codec->toUnicode(msgPart.bodyDecoded());
+            //	    debug ("part\n" + part ); inexplicably crashes -sanders
+          }
+          part.replace(reNL, '\n' + indentStr);
+          part = indentStr + part + '\n';
           if (sSmartQuote)
-             smartQuote(part, nlIndentStr, sWrapCol, aStripSignature);
+            smartQuote(part, '\n' + indentStr, sWrapCol, aStripSignature);
           result += part;
-	}
-	else isInline = FALSE;
+        } else
+          isInline = FALSE;
       }
-      if (!isInline && aIncludeAttach)
-      {
-	result += QString("\n----------------------------------------") +
-	  "\nContent-Type: " + msgPart.typeStr() + "/" + msgPart.subtypeStr();
-	if (!msgPart.name().isEmpty())
-	  result += "; name=\"" + msgPart.name() + '"';
+      if (!isInline && aIncludeAttach) {
+        result += QString("\n----------------------------------------") +
+                  "\nContent-Type: " + msgPart.typeStr() + "/" + msgPart.subtypeStr();
+        if (!msgPart.charset().isEmpty())
+          result += "; charset=\"" + msgPart.charset() + '"';
 
-	result += QString("\nContent-Transfer-Encoding: ")+ 
-	  msgPart.cteStr() + "\nContent-Description: " + 
-	  msgPart.contentDescription() +
-	  "\n----------------------------------------\n";
+        if (!msgPart.name().isEmpty())
+          result += "; name=\"" + msgPart.name() + '"';
+
+        result += QString("\nContent-Transfer-Encoding: ")+
+                  msgPart.cteStr() + "\nContent-Description: " +
+                  msgPart.contentDescription() +
+                  "\n----------------------------------------\n";
       }
     }
   }
 
-  return headerStr + "\n" + result;
+  QCString c = codec->fromUnicode(headerStr + "\n" + result);
+
+  return c;
 }
 
 
@@ -657,6 +668,8 @@ KMMessage* KMMessage::createReply(bool replyToAll)
   QString str, replyStr, mailingListStr, replyToStr, toStr, refStr;
 
   msg->initHeader(headerField("X-KMail-Identity"));
+  if (!headerField("X-KMail-Transport").isEmpty())
+    msg->setHeaderField("X-KMail-Transport", headerField("X-KMail-Transport"));
 
   mailingListStr = headerField("X-Mailing-List");
   replyToStr = replyTo();
@@ -673,7 +686,7 @@ KMMessage* KMMessage::createReply(bool replyToAll)
 
     if (!from().isEmpty()) toStr += from() + ", ";
 
-    // -sanders only include the to address if it's != replyTo address 
+    // -sanders only include the to address if it's != replyTo address
     if(!to().isEmpty() && (rep.isEmpty() || to().find(rep) == -1))
       toStr += to() + ", ";
 
@@ -689,9 +702,10 @@ KMMessage* KMMessage::createReply(bool replyToAll)
       pos1 = toStr.findRev(", ", i);
       if( pos1 == -1 ) pos1 = 0;
       pos2 = toStr.find(", ", i);
-      toStr = toStr.left(pos1) + toStr.right(toStr.length() - pos2); 
+      toStr = toStr.left(pos1) + toStr.right(toStr.length() - pos2);
     }
-    toStr.truncate(toStr.length()-2);
+    else
+      toStr.truncate(toStr.length()-2);
     // same for the cc field
     QString ccStr = cc().simplifyWhiteSpace() + ", ";
     if((i = ccStr.find(f)) != -1)
@@ -702,7 +716,8 @@ KMMessage* KMMessage::createReply(bool replyToAll)
       pos2 = ccStr.find(", ", i);
       ccStr = ccStr.left(pos1) + ccStr.right(ccStr.length() - pos2 - 1); //Daniel
     }
-    ccStr.truncate(ccStr.length()-2);
+    else
+      ccStr.truncate(ccStr.length()-2);
 
     // remove leading or trailing "," and spaces - might confuse some MTAs
     if (!ccStr.isEmpty())
@@ -712,7 +727,7 @@ KMMessage* KMMessage::createReply(bool replyToAll)
         ccStr = ccStr.simplifyWhiteSpace(); //mAybe it was ",  "
         if (ccStr[ccStr.length()-1] == ',')
           ccStr.truncate(ccStr.length()-1);
-	msg->setCc(ccStr);
+        msg->setCc(ccStr);
       }
 
   }
@@ -732,7 +747,7 @@ KMMessage* KMMessage::createReply(bool replyToAll)
 	toStr.truncate(toStr.length()-1);
       msg->setTo(toStr);
     }
-  
+
   refStr = getRefStr();
   if (!refStr.isEmpty())
     msg->setReferences(refStr);
@@ -744,13 +759,32 @@ KMMessage* KMMessage::createReply(bool replyToAll)
 
   msg->setBody(asQuotedString(replyStr, sIndentPrefixStr));
 
-  if (strnicmp(subject(), "Re:", 3)!=0)
+  QStringList::Iterator it;
+  bool recognized = false;
+  for (it = sReplySubjPrefixes.begin(); !recognized && (it != sReplySubjPrefixes.end()); ++it)
+  {
+    QString prefix = subject().left((*it).length());
+    if (prefix.lower() == (*it).lower()) //recognized
+    {
+      if (!sReplaceSubjPrefix || (prefix == "Re:"))
+        msg->setSubject(subject());
+      else
+      {
+        //replace recognized prefix with "Re: "
+        //handle crappy subjects Re:  blah blah (note double space)
+        int subjStart = (*it).length();
+        while (subject()[subjStart].isSpace()) //strip only from beginning
+          subjStart++;
+        msg->setSubject("Re: " + subject().mid(subjStart,
+                                   subject().length() - subjStart));
+      }
+      recognized = true;
+    }
+  }
+  if (!recognized)
     msg->setSubject("Re: " + subject());
-  else msg->setSubject(subject());
-#if defined CHARSETS
-  printf("Setting reply charset: %s\n",(const char *)charset());
+
   msg->setCharset(charset());
-#endif
   setStatus(KMMsgStatusReplied);
 
   return msg;
@@ -794,10 +828,11 @@ KMMessage* KMMessage::createRedirect(void)
   int i;
 
   str = asQuotedString(str, "", FALSE, false);
-  msg->setBody(str);
+
+  msg->setBody(str.latin1());
   if (numBodyParts() > 0)
   {
-    msgPart.setBody(str);
+    msgPart.setBody(str.latin1());
     msgPart.setTypeStr("text");
     msgPart.setSubtypeStr("plain");
     msg->addBodyPart(&msgPart);
@@ -812,15 +847,13 @@ KMMessage* KMMessage::createRedirect(void)
 	msg->addBodyPart(&msgPart);
       }
     }
-  }    
+  }
 
-#warning TODO: insert sender here
+//TODO: insert sender here
   msg->setHeaderField("X-KMail-Redirect-From", from());
   msg->setSubject(subject());
   msg->setFrom(from());
-#if defined CHARSETS
   msg->setCharset(charset());
-#endif
   setStatus(KMMsgStatusForwarded);
 
   return msg;
@@ -832,28 +865,31 @@ KMMessage* KMMessage::createForward(void)
 {
   KMMessage* msg = new KMMessage;
   KMMessagePart msgPart;
-  QString str;
+  QCString str;
+  QString s;
   int i;
 
   msg->initHeader(headerField("X-KMail-Identity"));
 
   if (sHdrStyle == KMReaderWin::HdrAll) {
-    str = "\n\n----------  " + sForwardStr + "  ----------\n";
-    str += asString();
-    str = asQuotedString(str, "", FALSE, false);
+    s = "\n\n----------  " + sForwardStr + "  ----------\n";
+    s += asString();
+    str = asQuotedString(s, "", FALSE, false);
     str += "\n-------------------------------------------------------\n";
   } else {
-    str = "\n\n----------  " + sForwardStr + "  ----------\n";
-    str += "Subject: " + subject() + "\n";
-    str += "Date: " + dateStr() + "\n";
-    str += "From: " + from() + "\n";
-    str += "To: " + to() + "\n";
-    str += "\n";
-    str = asQuotedString(str, "", FALSE, false);
+    s = "\n\n----------  " + sForwardStr + "  ----------\n";
+    s += "Subject: " + subject() + "\n";
+    s += "Date: " + dateStr() + "\n";
+    s += "From: " + from() + "\n";
+    s += "To: " + to() + "\n";
+    s += "\n";
+    str = asQuotedString(s, "", FALSE, false);
     str += "\n-------------------------------------------------------\n";
   }
 
+  msg->setCharset(charset());
   msg->setBody(str);
+
   if (numBodyParts() > 0)
   {
     msgPart.setBody(str);
@@ -868,7 +904,7 @@ KMMessage* KMMessage::createForward(void)
 	  (stricmp(msgPart.typeStr(),"text")!=0 &&
 	   stricmp(msgPart.typeStr(),"message")!=0))
       {
-	msg->addBodyPart(&msgPart);
+        msg->addBodyPart(&msgPart);
       }
     }
   }
@@ -876,9 +912,7 @@ KMMessage* KMMessage::createForward(void)
   if (strnicmp(subject(), "Fwd:", 4)!=0)
     msg->setSubject("Fwd: " + subject());
   else msg->setSubject(subject());
-#if defined CHARSETS
   msg->setCharset(charset());
-#endif
   setStatus(KMMsgStatusForwarded);
 
   return msg;
@@ -888,10 +922,10 @@ KMMessage* KMMessage::createForward(void)
 //-----------------------------------------------------------------------------
 void KMMessage::initHeader( QString id )
 {
-  QString identStr = "unknown";
+  QString identStr = i18n( "Default" );
   if (!id.isEmpty() && KMIdentity::identities().contains(id))
     identStr = id;
-    
+
   KMIdentity ident( identStr );
   ident.readConfig();
   if(ident.fullEmailAddr().isEmpty())
@@ -899,7 +933,7 @@ void KMMessage::initHeader( QString id )
   else
     setFrom(ident.fullEmailAddr());
 
-  if(ident.replyToAddr().isEmpty()) 
+  if(ident.replyToAddr().isEmpty())
     setReplyTo("");
   else
     setReplyTo(ident.replyToAddr());
@@ -983,9 +1017,8 @@ const QString KMMessage::dateShortStr(void) const
   if (!header.HasDate()) return "";
   unixTime = header.Date().AsUnixTime();
 
-  
-  result = ctime(&unixTime);
-  
+  QCString result = ctime(&unixTime);
+
   if (result[result.length()-1]=='\n')
     result.truncate(result.length()-1);
 
@@ -993,6 +1026,21 @@ const QString KMMessage::dateShortStr(void) const
 }
 
 
+//-----------------------------------------------------------------------------
+const QString KMMessage::dateIsoStr(void) const
+{
+  DwHeaders& header = mMsg->Headers();
+  time_t unixTime;
+
+  if (!header.HasDate()) return "";
+  unixTime = header.Date().AsUnixTime();
+
+  char cstr[64];
+  strftime(cstr, 63, "%Y-%m-%d %H:%M:%S", localtime(&unixTime));
+  return QString(cstr);
+}
+
+
 //-----------------------------------------------------------------------------
 time_t KMMessage::date(void) const
 {
@@ -1125,8 +1173,11 @@ const QString KMMessage::from(void) const
 
 
 //-----------------------------------------------------------------------------
-void KMMessage::setFrom(const QString& aStr)
+void KMMessage::setFrom(const QString& bStr)
 {
+  QString aStr = bStr;
+  if (aStr.isNull())
+      aStr = "";
   setHeaderField("From", aStr);
   mDirty = TRUE;
 }
@@ -1138,6 +1189,12 @@ const QString KMMessage::fromStrip(void) const
   return stripEmailAddr(decodeRFC1522String(headerField("From")));
 }
 
+//-----------------------------------------------------------------------------
+const QString KMMessage::fromEmail(void) const
+{
+  return getEmailAddr(decodeRFC1522String(headerField("From")));
+}
+
 
 //-----------------------------------------------------------------------------
 const QString KMMessage::subject(void) const
@@ -1172,14 +1229,26 @@ void KMMessage::setXMark(const QString& aStr)
 //-----------------------------------------------------------------------------
 const QString KMMessage::replyToId(void) const
 {
-  int rightAngle;
-  QString replyTo = headerField("In-Reply-To");
-  if (replyTo.isEmpty())
-    replyTo = headerField("References");
+  int leftAngle, rightAngle;
+  QString replyTo, references;
 
+  replyTo = headerField("In-Reply-To");
   rightAngle = replyTo.find( '>' );
   if (rightAngle != -1)
     replyTo.truncate( rightAngle + 1 );
+
+  references = headerField("References");
+  leftAngle = references.findRev( '<' );
+  if (leftAngle != -1)
+    references = references.mid( leftAngle );
+  rightAngle = references.find( '>' );
+  if (rightAngle != -1)
+    references.truncate( rightAngle + 1 );
+
+  if ((replyTo.isEmpty() || replyTo[0] != '<') &&
+      !references.isEmpty() && references[0] == '<')
+    replyTo = references;
+
   return replyTo;
 }
 
@@ -1264,12 +1333,13 @@ const QString KMMessage::headerField(const QString& aName) const
 {
   DwHeaders& header = mMsg->Headers();
   DwField* field;
+  QCString result;
 
   if (aName.isEmpty() || !(field = header.FindField((const char*)aName)))
     result = "";
-  else 
+  else
     result = decodeRFC1522String(header.FieldBody((const char*)aName).
-                    AsString().c_str());  
+                    AsString().c_str());
   return result;
 }
 
@@ -1289,13 +1359,17 @@ void KMMessage::removeHeaderField(const QString& aName)
 
 
 //-----------------------------------------------------------------------------
-void KMMessage::setHeaderField(const QString& aName, const QString& aValue)
+void KMMessage::setHeaderField(const QString& aName, const QString& bValue)
 {
   DwHeaders& header = mMsg->Headers();
   DwString str;
   DwField* field;
+  QString aValue = "";
+  if (!bValue.isEmpty())
+    aValue = encodeRFC2047String(bValue);
 
   if (aName.isEmpty()) return;
+  if (aValue.isEmpty()) return;
 
   str = aName;
   if (str[str.length()-1] != ':') str += ": ";
@@ -1420,16 +1494,16 @@ void KMMessage::setContentTransferEncoding(int aCte)
 
 
 //-----------------------------------------------------------------------------
-const QString KMMessage::body(void) const
+QCString KMMessage::body(void) const
 {
-  QString str;
+  QCString str;
   str = mMsg->Body().AsString().c_str();
   return str;
 }
 
 
 //-----------------------------------------------------------------------------
-const QString KMMessage::bodyDecoded(void) const
+QCString KMMessage::bodyDecoded(void) const
 {
   DwString dwsrc, dwstr;
   QString result;
@@ -1448,16 +1522,21 @@ const QString KMMessage::bodyDecoded(void) const
     break;
   }
   // Should probably be returning a QByteArray, if it may contain NUL
+
   QCString ba(dwstr.c_str(), dwstr.size() + 1);
-  return QString((const QByteArray&)ba);
+
+  return ba;
 }
 
 
 //-----------------------------------------------------------------------------
-void KMMessage::setBodyEncoded(const QString& aStr)
+void KMMessage::setBodyEncoded(const QCString& aStr)
 {
-  int len = aStr.length();
-  DwString dwSrc(aStr.data(), len);
+  QCString bStr = aStr;
+  if (aStr.isNull())
+    bStr = "";
+  int len = bStr.length();
+  DwString dwSrc(bStr.data(), len);
   DwString dwResult;
 
   switch (cte())
@@ -1479,21 +1558,50 @@ void KMMessage::setBodyEncoded(const QString& aStr)
 
 
 //-----------------------------------------------------------------------------
-void KMMessage::setBody(const QString& aStr)
+void KMMessage::setBody(const QCString& aStr)
 {
-  mMsg->Body().FromString((const char*)aStr);
+  mMsg->Body().FromString(aStr.data());
   mNeedsAssembly = TRUE;
 }
 
 
+// Patched by Daniel Moisset <dmoisset@grulic.org.ar>
+// modified numbodyparts, bodypart to take nested body parts as
+// a linear sequence.
+// third revision, Sep 26 2000
+
+// this is support structure for traversing tree without recursion
+
 //-----------------------------------------------------------------------------
 int KMMessage::numBodyParts(void) const
 {
-  int count;
+  int count = 0;
   DwBodyPart* part = mMsg->Body().FirstBodyPart();
+  QList< DwBodyPart > parts;
+  QString mp = "multipart";
 
-  for (count=0; part; count++)
-    part = part->Next();
+  while (part) 
+  {
+     //dive into multipart messages
+     while ( part && part->Headers().HasContentType() &&
+             (mp == part->Headers().ContentType().TypeStr().c_str()) )
+     {
+	 parts.append( part );
+	 part = part->Body().FirstBodyPart();
+     }
+     // this is where currPart->msgPart contains a leaf message part
+     count++;
+     // go up in the tree until reaching a node with next
+     // (or the last top-level node)
+     while (part && !(part->Next()) && !(parts.isEmpty()))
+     {
+	part = parts.getLast();
+	parts.removeLast();
+     } ;
+     
+     if (part)
+	 part = part->Next();
+  }
 
   return count;
 }
@@ -1502,14 +1610,38 @@ int KMMessage::numBodyParts(void) const
 //-----------------------------------------------------------------------------
 void KMMessage::bodyPart(int aIdx, KMMessagePart* aPart) const
 {
-  DwBodyPart* part;
+  DwBodyPart *part, *curpart;
+  QList< DwBodyPart > parts;
+  QString mp = "multipart";
   DwHeaders* headers;
-  int curIdx;
-
+  int curIdx = 0;
   // Get the DwBodyPart for this index
-  part = mMsg->Body().FirstBodyPart();
-  for (curIdx=0; curIdx < aIdx && part; ++curIdx)
-    part = part->Next();
+
+  curpart = mMsg->Body().FirstBodyPart();
+  part = 0;
+
+  while (curpart && !part) {
+     //dive into multipart messages
+     while ( curpart && curpart->Headers().HasContentType() &&
+             (mp == curpart->Headers().ContentType().TypeStr().c_str()) )
+     {
+	 parts.append( curpart );
+	 curpart = curpart->Body().FirstBodyPart();
+     }
+     // this is where currPart->msgPart contains a leaf message part
+     if (curIdx==aIdx)
+        part = curpart;
+     curIdx++;
+     // go up in the tree until reaching a node with next
+     // (or the last top-level node)
+     while (curpart && !(curpart->Next()) && !(parts.isEmpty()))
+     {
+	curpart = parts.getLast();
+	parts.removeLast();
+     } ;
+     if (curpart)
+	 curpart = curpart->Next();
+  }  
 
   // If the DwBodyPart was found get the header fields and body
   if (part)
@@ -1521,13 +1653,11 @@ void KMMessage::bodyPart(int aIdx, KMMessagePart* aPart) const
     {
       aPart->setTypeStr(headers->ContentType().TypeStr().c_str());
       aPart->setSubtypeStr(headers->ContentType().SubtypeStr().c_str());
-#if defined CHARSETS
       DwParameter *param=headers->ContentType().FirstParameter();
       while(param)
           if (param->Attribute()=="charset") break;
-          else param=param->Next(); 
+          else param=param->Next();
       if (param) aPart->setCharset(param->Value().c_str());
-#endif       
     }
     else
     {
@@ -1536,26 +1666,28 @@ void KMMessage::bodyPart(int aIdx, KMMessagePart* aPart) const
     }
     // Modification by Markus
     if (!headers->ContentType().Name().empty())
-      aPart->setName(headers->ContentType().Name().c_str());
+	aPart->setName(headers->ContentType().Name().c_str());
+    else if (!headers->Subject().AsString().empty())
+	aPart->setName( headers->Subject().AsString().c_str() );
     else
-      aPart->setName("unnamed");
+      aPart->setName( i18n("Attachment: ") + QString( "%1" ).arg( aIdx ) );
 
     // Content-transfer-encoding
     if (headers->HasContentTransferEncoding())
       aPart->setCteStr(headers->ContentTransferEncoding().AsString().c_str());
-    else 
+    else
       aPart->setCteStr("7bit");
 
     // Content-description
     if (headers->HasContentDescription())
       aPart->setContentDescription(headers->ContentDescription().AsString().c_str());
-    else 
+    else
       aPart->setContentDescription("");
 
     // Content-disposition
     if (headers->HasContentDisposition())
       aPart->setContentDisposition(headers->ContentDisposition().AsString().c_str());
-    else 
+    else
       aPart->setContentDisposition("");
 
     // Body
@@ -1612,15 +1744,12 @@ void KMMessage::setBodyPart(int aIdx, const KMMessagePart* aPart)
   const DwString contDesc = (const char*)aPart->contentDescription();
   const DwString contDisp = (const char*)aPart->contentDisposition();
   const DwString bodyStr  = (const char*)aPart->body();
-#if defined CHARSETS
   const DwString charset  = (const char*)aPart->charset();
-#endif
   DwHeaders& headers = part->Headers();
   if (type != "" && subtype != "")
   {
     headers.ContentType().SetTypeStr(type);
     headers.ContentType().SetSubtypeStr(subtype);
-#if defined CHARSETS
     if (!charset.empty())
     {
       DwParameter *param=headers.ContentType().FirstParameter();
@@ -1634,10 +1763,9 @@ void KMMessage::setBodyPart(int aIdx, const KMMessagePart* aPart)
 	param=new DwParameter;
 	param->SetAttribute("charset");
 	headers.ContentType().AddParameter(param);
-      }   
+      }
       param->SetValue(charset);
     }
-#endif
   }
   if (cte != "")
     headers.Cte().FromString(cte);
@@ -1648,7 +1776,7 @@ void KMMessage::setBodyPart(int aIdx, const KMMessagePart* aPart)
   if (contDisp != "")
     headers.ContentDisposition().FromString(contDisp);
 
-  part->Body().FromString(bodyStr); 
+  part->Body().FromString(bodyStr);
   mNeedsAssembly = TRUE;
 }
 
@@ -1668,19 +1796,18 @@ void KMMessage::addBodyPart(const KMMessagePart* aPart)
   QString type     = aPart->typeStr();
   QString subtype  = aPart->subtypeStr();
   QString cte      = aPart->cteStr();
-  QString contDesc = aPart->contentDescription();
+  QString contDesc = KMMsgBase::encodeRFC2047String(aPart->
+    contentDescription());
   QString contDisp = aPart->contentDisposition();
-  QString name     = aPart->name();
-#if defined CHARSETS
-   QString charset  = aPart->charset();
-#endif
+  QString name     = KMMsgBase::encodeRFC2231String(aPart->name());
+  bool RFC2231encoded = aPart->name() != name;
+  QString charset  = aPart->charset();
 
   DwHeaders& headers = part->Headers();
   if (type != "" && subtype != "")
   {
     headers.ContentType().SetTypeStr((const char*)type);
     headers.ContentType().SetSubtypeStr((const char*)subtype);
-#if defined CHARSETS
     if (!charset.isEmpty()){
          DwParameter *param;
          param=new DwParameter;
@@ -1688,11 +1815,19 @@ void KMMessage::addBodyPart(const KMMessagePart* aPart)
          param->SetValue((const char *)charset);
          headers.ContentType().AddParameter(param);
     }
-#endif
   }
 
-  if(!name.isEmpty())
-    headers.ContentType().SetName((const char*)name);
+  if (RFC2231encoded)
+  {
+    DwParameter *nameParam;
+    nameParam = new DwParameter;
+    nameParam->SetAttribute("name*");
+    nameParam->SetValue((const char*)name);
+    headers.ContentType().AddParameter(nameParam);
+  } else {
+    if(!name.isEmpty())
+      headers.ContentType().SetName((const char*)name);
+  }
 
   if (!cte.isEmpty())
     headers.Cte().FromString(cte);
@@ -1711,26 +1846,26 @@ void KMMessage::addBodyPart(const KMMessagePart* aPart)
 
 
 //-----------------------------------------------------------------------------
-void KMMessage::viewSource(const QString& aCaption) const
+void KMMessage::viewSource(const QString& aCaption, QTextCodec *codec) const
 {
   QString str = ((KMMessage*)this)->asString();
+  if (codec) str=codec->toUnicode(str);
 
 #if ALLOW_GUI
   QMultiLineEdit* edt;
 
   edt = new QMultiLineEdit;
-  KWM::setMiniIcon(edt->winId(), kapp->miniIcon());
+  KWin::setIcons(edt->winId(), kapp->icon(), kapp->miniIcon());
   if (aCaption) edt->setCaption(aCaption);
 
   edt->insertLine(str);
   edt->setReadOnly(TRUE);
-  edt->resize(KApplication::desktop()->width()/2, 
+  edt->resize(KApplication::desktop()->width()/2,
 	      2*KApplication::desktop()->height()/3);
   edt->setCursorPosition(0, 0);  edt->show();
 
 #else //not ALLOW_GUI
-  debug("Message source: %s\n%s\n--- end of message ---", 
-	aCaption.isEmpty() ? "" : (const char*)aCaption, str);
+  kdDebug() << "Message source: " << (aCaption.isEmpty() ? "" : (const char*)aCaption) << "\n" << str << "\n--- end of message ---" << endl;
 
 #endif
 }
@@ -1769,11 +1904,40 @@ const QString KMMessage::stripEmailAddr(const QString& aStr)
   return result;
 }
 
+//-----------------------------------------------------------------------------
+const QString KMMessage::getEmailAddr(const QString& aStr)
+{
+  int a, i, j, len, found = 0;
+  QChar c;
+  QString result;
+  // Find the '@' in the email address:
+  a = aStr.find('@');
+  if (a<0) return aStr;
+  // Loop backwards until we find '<', '(', ' ', or beginning of string.
+  for (i = a - 1; i >= 0; i--) {
+    c = aStr[i];
+    if (c == '<' || c == '(' || c == ' ') found = 1;
+    if (found) break;
+  }
+  // Reset found for next loop.
+  found = 0;
+  // Loop forwards until we find '>', ')', ' ', or end of string.
+  for (j = a + 1; j < (int)aStr.length(); j++) {
+    c = aStr[j];
+    if (c == '>' || c == ')' || c == ' ') found = 1;
+    if (found) break;
+  }
+  // Calculate the length and return the result.
+  len = j - (i + 1);
+  result = aStr.mid(i+1,len);
+  return result;
+}
+
 
 //-----------------------------------------------------------------------------
 const QString KMMessage::emailAddrAsAnchor(const QString& aEmail, bool stripped)
 {
-  QString result, addr, tmp;
+  QString result, addr, tmp, tmp2;
   const char *pos;
   char ch;
   QString email = decodeRFC1522String(aEmail);
@@ -1789,14 +1953,18 @@ const QString KMMessage::emailAddrAsAnchor(const QString& aEmail, bool stripped)
     else if (ch == '&') addr += "&amp;";
     else if (ch != ',') addr += ch;
 
+    if (ch != ',')
+      tmp2 += ch;
+
     if (ch == ',' || !pos[1])
     {
       tmp = addr.copy();
       result += tmp.replace(QRegExp("\""),"");
       result = result.replace(QRegExp("\n"),"");
       result += "\">";
-      if (stripped) result += KMMessage::stripEmailAddr(aEmail);
+      if (stripped) result += KMMessage::stripEmailAddr(tmp2);
       else result += addr;
+      tmp2 = "";
       result += "</a>";
       if (ch == ',')
       {
@@ -1818,12 +1986,16 @@ void KMMessage::readConfig(void)
   /* They are taken from kmail's dialog */
 
   KConfig *config=kapp->config();
-    
+
   config->setGroup("KMMessage");
   sReplyStr = config->readEntry("phrase-reply",i18n("On %D, you wrote:"));
   sReplyAllStr = config->readEntry("phrase-reply-all",i18n("On %D, %F wrote:"));
   sForwardStr = config->readEntry("phrase-forward",i18n("Forwarded Message"));
   sIndentPrefixStr = config->readEntry("indent-prefix",">%_");
+  sReplySubjPrefixes = config->readListEntry("reply-prefixes", ',');
+  if (sReplySubjPrefixes.count() == 0)
+    sReplySubjPrefixes.append("Re:");
+  sReplaceSubjPrefix = config->readBoolEntry("replace-prefix", true);
 
   config->setGroup("Reader");
   sHdrStyle = config->readNumEntry("hdr-style", KMReaderWin::HdrFancy);
@@ -1837,34 +2009,33 @@ void KMMessage::readConfig(void)
      sWrapCol = 60;
 }
 
-#if defined CHARSETS
+
 //-----------------------------------------------------------------------------
 const QString KMMessage::charset(void) const
 {
-   printf("Checking charset...\n");
    DwMediaType &mType=mMsg->Headers().ContentType();
    mType.Parse();
    DwParameter *param=mType.FirstParameter();
    while(param){
-      printf("%s=%s\n",param->Attribute().c_str(),param->Value().c_str());
       if (param->Attribute()=="charset")
         return QString(param->Value().c_str());
-      else param=param->Next(); 
-   }   
+      else param=param->Next();
+   }
    return ""; // us-ascii, but we don't have to specify it
 }
 
 //-----------------------------------------------------------------------------
-void KMMessage::setCharset(const QString& aStr)
+void KMMessage::setCharset(const QString& bStr)
 {
-   printf("Setting charset to: %s\n",(const char *)aStr);
+   QString aStr = bStr;
+   if (aStr.isNull())
+       aStr = "";
    DwMediaType &mType=mMsg->Headers().ContentType();
    mType.Parse();
-   printf("mType: %s\n",mType.AsString().c_str());
    DwParameter *param=mType.FirstParameter();
    while(param)
       if (param->Attribute()=="charset") break;
-      else param=param->Next(); 
+      else param=param->Next();
    if (!param){
       param=new DwParameter;
       param->SetAttribute("charset");
@@ -1872,6 +2043,4 @@ void KMMessage::setCharset(const QString& aStr)
    }
    param->SetValue((const char *)aStr);
    mType.Assemble();
-   printf("mType: %s\n",mType.AsString().c_str());
-}		
-#endif
+}
