@@ -1,5 +1,7 @@
 // kmmsgbase.cpp
 
+#include <kdebug.h>
+
 #include "kmmsgbase.h"
 #include <mimelib/mimepp.h>
 #include <qregexp.h>
@@ -8,6 +10,7 @@
 #endif
 
 #include <ctype.h>
+#include <stdlib.h>
 
 #define NUM_STATUSLIST 9
 static KMMsgStatus sStatusList[NUM_STATUSLIST] =
@@ -66,7 +69,7 @@ bool KMMsgBase::isMessage(void) const
 
 
 //-----------------------------------------------------------------------------
-void KMMsgBase::setStatus(KMMsgStatus aStatus)
+void KMMsgBase::setStatus(const KMMsgStatus aStatus)
 {
   if (mParent) mParent->msgStatusChanged( mStatus, aStatus );
   mStatus = aStatus;
@@ -148,7 +151,7 @@ const char* KMMsgBase::statusToStr(KMMsgStatus aStatus)
 
 
 //-----------------------------------------------------------------------------
-void KMMsgBase::setDate(const time_t aUnixTime)
+void KMMsgBase::setDate(time_t aUnixTime)
 {
   mDate  = aUnixTime;
   mDirty = TRUE;
@@ -190,14 +193,14 @@ const QString KMMsgBase::asIndexString(void) const
 //  dateTen %= 10000000000; // In index only 10 chars are reserved for the date
 //  This is nonsense because 10000000000 is bigger than the highest unsigned
 //  long. (Or is there any compiler that defines unsigned long as something
-//  really huge??)
+//  really huge?? )
 
   QString a((const char*)decodeQuotedPrintableString(subject()));
   a.truncate(100);
   QString b((const char*)decodeQuotedPrintableString(fromStrip()));
   b.truncate(50);
   QString c((const char*)decodeQuotedPrintableString(toStrip()));
-  c.truncate(50);
+  c.truncate(47);
   QString d((const char*)replyToIdMD5());
   d.truncate(22);
   QString e((const char*)msgIdMD5());
@@ -208,7 +211,7 @@ const QString KMMsgBase::asIndexString(void) const
 	      (char)status(), folderOffset(), msgSize(), dateTen,
 	      (const char*)xmark() );
   if (str.length() != 37)
-    debug( "Invalid length " + str );
+    kdDebug() << "Invalid length " << endl;
   str += a.rightJustify( 100, ' ' );
   str += " ";
   str += b.rightJustify( 50, ' ' );
@@ -225,8 +228,8 @@ const QString KMMsgBase::asIndexString(void) const
       str[i] = ' ';
 
   if (str.length() != 285) {
-    debug( QString( "Error invalid index entry %1").arg(str.length()) );
-    debug( str );
+    kdDebug() << QString( "Error invalid index entry %1").arg(str.length()) << endl;
+    kdDebug() << str << endl;
   }
   return str;
 }
@@ -359,6 +362,142 @@ const QString KMMsgBase::decodeRFC1522String(const QString& _str)
 }
 
 
+//-----------------------------------------------------------------------------
+const char especials[17] = "()<>@,;:\"/[]?.= ";
+
+const QString KMMsgBase::encodeRFC2047String(const QString& _str)
+{
+  if (_str.isEmpty()) return _str;
+  QCString latin = _str.local8Bit();
+  int cr, start, stop, pos = 0;
+  int latinLen = latin.length();
+  char hexcode;
+  int numQuotes, i;
+  QString result = QString();
+  while (pos < latinLen)
+  {
+    cr = pos;
+    start = pos;
+    while (cr < latinLen)
+    {
+      if (latin[cr] == 32) start = cr + 1;
+      if (latin[cr] < 0) break;
+      cr++;
+    }
+    if (cr < latinLen)
+    {
+      numQuotes = 1;
+      while (cr < latinLen)
+      {
+        /* The encoded word must be limited to 75 character */
+        for (i = 0; i < 16; i++) if (latin[cr] == especials[i]) numQuotes++;
+        if (latin[cr] < 0) numQuotes++;
+        /* Stop after 58 = 75 - 17 characters or at "<user@host..." */
+        if (cr - start + 2 * numQuotes >= 58 || latin[cr] == 60) break;
+        cr++;
+      }
+      if (cr < latinLen)
+      {
+        stop = cr - 1;
+        while (stop >= start && latin[stop] != 32) stop--;
+        if (stop <= start) stop = cr;
+      } else stop = cr;
+      while (pos < start) { result += latin[pos]; pos++; }
+      result += QString("=?iso-8859-1?q?");
+      while (pos < stop)
+      {
+        numQuotes = 0;
+        for (i = 0; i < 16; i++) if (latin[pos] == especials[i]) numQuotes = 1;
+        if (latin[pos] < 0) numQuotes = 1;
+        if (numQuotes)
+        {
+          result += "=";
+          hexcode = ((latin[pos] & 0xF0) >> 4) + 48;
+          if (hexcode >= 58) hexcode += 7;
+          result += hexcode;
+          hexcode = (latin[pos] & 0x0F) + 48;
+          if (hexcode >= 58) hexcode += 7;
+          result += hexcode;
+        } else {
+          result += latin[pos];
+        }
+        pos++;
+      }
+      result += "?=";
+    } else {
+      while (pos < latinLen) { result += latin[pos]; pos++; }
+    }
+  }
+  return result;
+}
+
+
+//-----------------------------------------------------------------------------
+const QString KMMsgBase::encodeRFC2231String(const QString& _str)
+{
+  if (_str.isEmpty()) return _str;
+  char *latin = (char *)calloc(1, _str.length() + 1);
+  strcpy(latin, _str.latin1());
+  char *l = latin;
+  char hexcode;
+  int i;
+  bool quote;
+  while (*l)
+  {
+    if (*l < 0) break;
+    l++;
+  }
+  if (!*l) return _str;
+  QString result = QString("iso-8859-1''");
+  l = latin;
+  while (*l)
+  {
+    quote = *l < 0;
+    for (i = 0; i < 16; i++) if (*l == especials[i]) quote = true;
+    if (quote)
+    {
+      result += "%";
+      hexcode = ((*l & 0xF0) >> 4) + 48;
+      if (hexcode >= 58) hexcode += 7;
+      result += hexcode;
+      hexcode = (*l & 0x0F) + 48;
+      if (hexcode >= 58) hexcode += 7;
+      result += hexcode;
+    } else {
+      result += *l;
+    }
+    l++;
+  }
+  free(latin);
+  return result;
+}
+
+
+//-----------------------------------------------------------------------------
+const QString KMMsgBase::decodeRFC2231String(const QString& _str)
+{
+  int p = _str.find("'");
+  if (p < 0) return _str;
+  QString st = _str.mid(_str.findRev("'") + 1);
+  char ch, ch2;
+  p = 0;
+  while (p < (int)st.length())
+  {
+    if (st.at(p) == 37)
+    {
+      ch = st.at(p+1).latin1() - 48;
+      if (ch > 16) ch -= 7;
+      ch2 = st.at(p+2).latin1() - 48;
+      if (ch2 > 16) ch2 -= 7;
+      st.at(p) = ch * 16 + ch2;
+      st.remove( p+1, 2 );
+    }
+    p++;
+  }
+  return st;
+}
+
+
 //-----------------------------------------------------------------------------
 const QString KMMsgBase::decodeQuotedPrintableString(const QString& aStr)
 {
@@ -412,7 +551,11 @@ const QString KMMsgBase::decodeQuotedPrintableString(const QString& aStr)
 //-----------------------------------------------------------------------------
 const QString KMMsgBase::decodeQuotedPrintable(const QString& aStr)
 {
-  DwString dwsrc(aStr.data());
+  QString bStr = aStr;
+  if (aStr.isNull())
+    bStr = "";
+
+  DwString dwsrc(bStr.data());
   DwString dwdest;
 
   DwDecodeQuotedPrintable(dwsrc, dwdest);
@@ -423,7 +566,11 @@ const QString KMMsgBase::decodeQuotedPrintable(const QString& aStr)
 //-----------------------------------------------------------------------------
 const QString KMMsgBase::encodeQuotedPrintable(const QString& aStr)
 {
-  DwString dwsrc(aStr.data(), aStr.length());
+  QString bStr = aStr;
+  if (aStr.isNull())
+    bStr = "";
+
+  DwString dwsrc(bStr.data(), bStr.length());
   DwString dwdest;
   QString result;
 
@@ -436,7 +583,12 @@ const QString KMMsgBase::encodeQuotedPrintable(const QString& aStr)
 //-----------------------------------------------------------------------------
 const QString KMMsgBase::decodeBase64(const QString& aStr)
 {
-  DwString dwsrc(aStr.data(), aStr.length());
+  QString bStr = aStr;
+  if (aStr.isNull())
+    bStr = "";
+  while (bStr.length() < 16) bStr += "=";
+
+  DwString dwsrc(bStr.data(), bStr.length());
   DwString dwdest;
   QString result;
 
@@ -449,7 +601,11 @@ const QString KMMsgBase::decodeBase64(const QString& aStr)
 //-----------------------------------------------------------------------------
 const QString KMMsgBase::encodeBase64(const QString& aStr)
 {
-  DwString dwsrc(aStr.data(), aStr.length());
+  QString bStr = aStr;
+  if (aStr.isNull())
+    bStr = "";
+
+  DwString dwsrc(bStr.data(), bStr.length());
   DwString dwdest;
   QString result;
 
