@@ -2,14 +2,18 @@
 
 #include <qdir.h>
 
-#include "kmmsgpart.h"
-#include "kmmessage.h"
 #include <kmimemagic.h>
 #include <kapp.h>
 #include <kconfig.h>
 #include <kglobal.h>
 #include <kstddirs.h>
 #include <kiconloader.h>
+#include <kdebug.h>
+
+#include "kmmsgbase.h"
+#include "kmmsgpart.h"
+#include "kmmessage.h"
+
 extern "C" {
 #include "md5.h"
 };
@@ -115,8 +119,7 @@ void KMMessagePart::setBodyEncoded(const QByteArray& aStr)
     memcpy(mBody.data(), dwResult.c_str(), len);
     break;
   default:
-    debug("WARNING -- unknown encoding `%s'. Assuming 8bit.", 
-	  (const char*)cteStr());
+    kdDebug() << "WARNING -- unknown encoding `" << (const char*)cteStr() << "'. Assuming 8bit." << endl;
   case DwMime::kCte7bit:
   case DwMime::kCte8bit:
   case DwMime::kCteBinary:
@@ -151,8 +154,7 @@ QByteArray KMMessagePart::bodyDecoded(void) const
     memcpy((void*)result.data(), (void*)dwResult.c_str(), len);
     break;
   default:
-    debug("WARNING -- unknown encoding `%s'. Assuming 8bit.", 
-	  (const char*)cteStr());
+    kdDebug() << "WARNING -- unknown encoding `" << (const char*)cteStr() << "'. Assuming 8bit." << endl;
   case DwMime::kCte7bit:
   case DwMime::kCte8bit:
   case DwMime::kCteBinary:
@@ -299,7 +301,7 @@ void KMMessagePart::setContentTransferEncoding(int aCte)
 //-----------------------------------------------------------------------------
 const QString KMMessagePart::contentDescription(void) const
 {
-  return mContentDescription;
+  return KMMsgBase::decodeRFC1522String(mContentDescription);
 }
 
 
@@ -315,19 +317,28 @@ const QString KMMessagePart::fileName(void) const
 {
   int i, j, len;
   QString str;
+  int RFC2231encoded = 0;
 
-  i = mContentDisposition.find("filename=", 0, FALSE);
-  if (i < 0) return QString::null;
+  i = mContentDisposition.find("filename*=", 0, FALSE);
+  if (i >= 0) { RFC2231encoded = 1; }
+  else {
+    i = mContentDisposition.find("filename=", 0, FALSE);
+    if (i < 0) return QString::null;
+  }
   j = mContentDisposition.find(';', i+9);
 
   if (j < 0) j = 32767;
-  str = mContentDisposition.mid(i+9, j-i-9).stripWhiteSpace();
+  str = mContentDisposition.mid(i+9+RFC2231encoded, j-i-9-RFC2231encoded).
+    stripWhiteSpace();
 
   len = str.length();
   if (str[0]=='"' && str[len-1]=='"') 
       str = str.mid(1, len-2);
 
-  str = KMMsgBase::decodeQuotedPrintableString(str);
+  if (RFC2231encoded)
+    str = KMMsgBase::decodeRFC2231String(str);
+  else
+    str = KMMsgBase::decodeQuotedPrintableString(str);
 
   return str;
 }
@@ -368,21 +379,17 @@ void KMMessagePart::setName(const QString aStr)
 }
 
 
-#if defined CHARSETS
 //-----------------------------------------------------------------------------
 const QString KMMessagePart::charset(void) const
 {
-
    return mCharset;
 }
 
 //-----------------------------------------------------------------------------
 void KMMessagePart::setCharset(const QString aStr)
 {
-
   mCharset=aStr;
 }
-#endif
 
 
 
