@@ -8,6 +8,7 @@
 #include <qdialog.h>
 #include <qtimer.h>
 #include <qcolor.h>
+#include <qtextcodec.h>
 
 class KHTMLPart;
 class KMFolder;
@@ -32,6 +33,9 @@ public:
   KMReaderWin(QWidget *parent=0, const char *name=0, int f=0);
   virtual ~KMReaderWin();
 
+  /** Read color settings and set palette */
+  virtual void readColorConfig(void);
+
   /** Read settings from app's config file. */
   virtual void readConfig(void);
 
@@ -59,6 +63,10 @@ public:
   AttachmentStyle attachmentStyle(void) const { return mAttachmentStyle;}
   virtual void setAttachmentStyle(int style);
 
+  /** Get/set codec for reader win. */
+  QTextCodec *codec(void) const { return mCodec; }
+  virtual void setCodec(QTextCodec *codec);
+
   /** Set the message that shall be shown. If NULL, an empty page is
     displayed. */
   virtual void setMsg(KMMessage* msg, bool force = false);
@@ -90,12 +98,19 @@ public:
   KMReaderWin that is created. */
   static QString attachDir(void) { return mAttachDir; }
 
-  /* Override default html mail setting */
+  /** Override default html mail setting */
   void setHtmlOverride( bool override );
 
-  /* Is html mail to be supported? Takes into account override */
+  /** Is html mail to be supported? Takes into account override */
   bool htmlMail();
 
+  /** View message part of type message/RFC822 in extra viewer window. */
+  void atmViewMsg(KMMessagePart* msgPart);
+
+  /** Display an attachment */
+  static void atmView(KMReaderWin* aReaderWin, KMMessagePart* aMsgPart,
+    bool aHTML, const QString& aFileName, const QString& pname, QTextCodec *codec);
+
 signals:
   /** Emitted to show a text on the status line. */
   void statusMsg(const QString& text);
@@ -111,6 +126,9 @@ signals:
 
 public slots:
 
+  /* Force update even if message is the same */
+  void clearCache();
+
   /* Refresh the reader window */
   void updateReaderWin();
 
@@ -132,6 +150,9 @@ public slots:
   /** The user presses the right mouse button on an URL. */
   void slotUrlPopup(const QString &, const QPoint& mousePos);
 
+  /** The user selected "Find" from the menu. */
+  void slotFind();
+
 protected slots:
   /** Some attachment operations. */
   void slotAtmOpen();
@@ -143,6 +164,9 @@ protected slots:
   void slotDelayedResize();
 
 protected:
+  /** Watch for palette changes */
+  virtual bool event(QEvent *e);
+
   /** Feeds the HTML viewer with the contents of the given message.
     HTML begin/end parts are written around the message. */
   virtual void parseMsg(void);
@@ -152,7 +176,7 @@ protected:
 
   /** Creates a nice mail header depending on the current selected
     header style. */
-  virtual void writeMsgHeader(void);
+  virtual void writeMsgHeader(int vcpartnum = -1);
 
   /** Feeds the HTML widget with the contents of the given message-body
     string. May contain body parts. */
@@ -184,9 +208,6 @@ protected:
   /** Returns id of message part from given URL or -1 if invalid. */
   virtual int msgPartFromUrl(const KURL &url);
 
-  /** View message part of type message/RFC822 in extra viewer window. */
-  virtual void atmViewMsg(KMMessagePart* msgPart);
-
  /** Create directory for attachments */
   virtual void makeAttachDir(void);
 
@@ -212,6 +233,8 @@ protected:
   QString mBackingPixmapStr;
   QTimer updateReaderWinTimer;
   QTimer mResizeTimer;
+  QTextCodec *mCodec;
+  bool mAutoDetectEncoding;
 
   int fntSize;
   QString mBodyFamily;
