@@ -15,59 +15,33 @@
  *                                                                         *
  ***************************************************************************/
 
-
 #ifndef KNGROUPPROPDLG_H
 #define KNGROUPPROPDLG_H
 
+#include <kdialogbase.h>
 
-#include <qtabdialog.h>
-#include <qgroupbox.h>
-#include <qlabel.h>
-#include <qlineedit.h>
-#include <qcheckbox.h>
-#include "knuserwidget.h"
-
+class QLineEdit;
+class KNUserWidget;
 class KNGroup;
 
-class KNGroupPropDlg : public QTabDialog  {
 
-	public:
-		KNGroupPropDlg(KNGroup *group, QWidget *parent=0, const char *name=0);
-		~KNGroupPropDlg();
-		
-		void apply();
-		bool nickHasChanged()	{ return nChanged; }	
-		
-	protected:
-	
-		class statistics : public QWidget {
-		
-			public:
-				statistics(QWidget *parent=0, const char *name=0, int *values=0);
-				~statistics();
-				
-				void resizeEvent(QResizeEvent*);
-								
-				QGroupBox *gb1;
-				QLabel *t_otal, *u_nread, *n_ew, *u_nrThr, *n_ewThr;
-		};
-		
-		class settings : public QWidget {
-			
-			public:
-				settings(QWidget *parent=0, const char *name=0);
-				~settings();
-								
-				QCheckBox *useID;
-				KNUserWidget *uw;
-				QLineEdit *nick;
-				
-		};
-		
-		settings *set;
-		statistics *sta;
-		KNGroup *grp;
-		bool nChanged;
+class KNGroupPropDlg : public KDialogBase  {
+
+  public:
+    KNGroupPropDlg(KNGroup *group, QWidget *parent=0, const char *name=0);
+    ~KNGroupPropDlg();
+    
+    bool nickHasChanged() { return nChanged; }  
+    
+  protected:
+    KNGroup *grp;
+    bool nChanged;
+    KNUserWidget *uw;
+    QLineEdit *nick;
+    
+  protected slots:
+    void slotOk();
+  
 };
 
 #endif
