@@ -1,9 +1,9 @@
 /***************************************************************************
                           knprotocolclient.cpp  -  description
                              -------------------
-
-    copyright            : (C) 2000 by Christian Thurner
-    email                : cthurner@freepage.de
+    copyright            : (C) 2000 by Christian Gebauer
+    email                : gebauer@bigfoot.com
+    version              : $Id: knprotocolclient.cpp,v 1.23 2000/09/28 21:42:30 gebauer Exp $
  ***************************************************************************/
 
 /***************************************************************************
@@ -15,22 +15,32 @@
  *                                                                         *
  ***************************************************************************/
 
+#include <config.h>
+
+#include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/time.h>
+
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <netdb.h>
-#include <fcntl.h>
-#include <stdio.h>
+
 #include <errno.h>
+#include <fcntl.h>
+#include <netdb.h>
+#include <pthread.h>
+#include <stdio.h>
 #include <signal.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <pthread.h>
+
+#include <qstrlist.h>
 #include <klocale.h>
 
+#include "knserverinfo.h"
+#include "knjobdata.h"
 #include "knprotocolclient.h"
 
+
 KNProtocolClient::KNProtocolClient(int NfdPipeIn, int NfdPipeOut, QObject *parent, const char *name)
 :  QObject(parent,name), job(0L), inputSize(10000), fdPipeIn(NfdPipeIn), fdPipeOut(NfdPipeOut), tcpSocket(-1)
 {
@@ -41,18 +51,18 @@ KNProtocolClient::~KNProtocolClient()
 {
   if (isConnected())
     closeConnection();
-  delete input;
+  delete [] input;
 }
 
 
 void* KNProtocolClient::startThread(void* pseudoThis)
 {
-  KNProtocolClient* newthis = (KNProtocolClient*) (pseudoThis);
+  KNProtocolClient *newthis = static_cast<KNProtocolClient*>(pseudoThis);
 
   if (0!=pthread_setcanceltype(PTHREAD_CANCEL_ENABLE,NULL))
-    qDebug("pthread_setcanceltype failed!");
+    qWarning("pthread_setcanceltype failed!");
   if (0!= pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,NULL))
-    qDebug("pthread_setcanceltype failed!");
+    qWarning("pthread_setcanceltype failed!");
 
   signal(SIGPIPE,SIG_IGN);   // ignore sigpipe
 
@@ -82,7 +92,7 @@ void KNProtocolClient::waitForWork()
 
   while (true) {
     if (isConnected()) {  // we are connected, hold the connection for xx secs
-    	FD_ZERO(&fdsR);
+      FD_ZERO(&fdsR);
       FD_SET(fdPipeIn, &fdsR);
       FD_SET(tcpSocket, &fdsR);
       FD_ZERO(&fdsE);
@@ -91,11 +101,15 @@ void KNProtocolClient::waitForWork()
       tv.tv_usec = 0;
       selectRet = select(FD_SETSIZE, &fdsR, NULL, &fdsE, &tv);
       if (selectRet == 0) {
-      	qDebug("KNProtocolClient::waitForWork(): hold time elapsed, closing connection.");
-				closeConnection();               // nothing happend...
+#ifndef NDEBUG
+        qDebug("knode: KNProtocolClient::waitForWork(): hold time elapsed, closing connection.");
+#endif
+        closeConnection();               // nothing happend...
       } else {
-				if (((selectRet > 0)&&(!FD_ISSET(fdPipeIn,&fdsR)))||(selectRet == -1)) {
-        	qDebug("KNProtocolClient::waitForWork(): connection broken, closing it");
+        if (((selectRet > 0)&&(!FD_ISSET(fdPipeIn,&fdsR)))||(selectRet == -1)) {
+#ifndef NDEBUG
+          qDebug("knode: KNProtocolClient::waitForWork(): connection broken, closing it");
+#endif
           closeSocket();
         }
       }
@@ -110,13 +124,13 @@ void KNProtocolClient::waitForWork()
 
     sendSignal(TSjobStarted);
     if (job) {
-    // 	qDebug("KNProtocolClient::waitForWork(): got job");  too verbose
+    //  qDebug("knode: KNProtocolClient::waitForWork(): got job");
 
-      if (!account.isEqual(job->account())) {     // server changed
-      	account.copy(job->account());
-      	if (isConnected())
-				  closeConnection();
-			}
+      if (job->net()&&!account.isEqual(job->account())) {     // server changed
+        account.copy(job->account());
+        if (isConnected())
+          closeConnection();
+      }
 
       input[0] = 0;                 //terminate string
       thisLine = input;
@@ -127,13 +141,17 @@ void KNProtocolClient::waitForWork()
       doneLines = 0;
       byteCount = 0;
 
-      if (!isConnected())
-				openConnection();
+      if (!job->net())    // job needs no net access
+        processJob();
+      else {
+        if (!isConnected())
+          openConnection();
+
+        if (isConnected())         // connection is ready
+          processJob();
+      }
+      errorPrefix = "";
 
-      if (isConnected())         // connection is ready
-				processJob();
-			errorPrefix = "";
-				
       clearPipe();
     }
     sendSignal(TSworkDone);      // emit stopped signal
@@ -148,10 +166,12 @@ void KNProtocolClient::processJob()
 // connect, handshake and authorization
 bool KNProtocolClient::openConnection()
 {
-	sendSignal(TSconnect);
+  sendSignal(TSconnect);
+
+#ifndef NDEBUG
+  qDebug("knode: KNProtocolClient::openConnection(): opening connection");
+#endif
 
- 	qDebug("KNProtocolClient::openConnection(): opening connection");
-	
   if (account.server().isEmpty()) {
     job->setErrorString(i18n("Unable to resolve hostname"));
     return false;
@@ -177,17 +197,22 @@ bool KNProtocolClient::openConnection()
 
   in_addr address;
 
-  if (inet_aton(account.server(),&address)) {
+ // can only use inet_addr because of portability problem on Solaris
+ // TODO: port to QDns/QSocket
+ // Solaris uses deprecated inet_addr instead of inet_aton (David F.)
+  address.s_addr = inet_addr(account.server().local8Bit().data());
+  // unsigned int is ok because its uint/32bit in fact
+  if ( (unsigned int) address.s_addr != (unsigned int)-1 ) {
     if (!conRawIP(&address)) {
       closeSocket();
       return false;
     }
   } else {               // host name lookup....
-    struct hostent* hostData = gethostbyname(account.server());
+    struct hostent* hostData = gethostbyname(account.server().local8Bit().data());
 
     if (NULL==hostData) {
       herror("connect(): ");
-	    job->setErrorString(i18n("Unable to resolve hostname"));
+      job->setErrorString(i18n("Unable to resolve hostname"));
       closeSocket();
       return false;
     }
@@ -210,7 +235,9 @@ void KNProtocolClient::closeConnection()
   fd_set fdsW;
   timeval tv;
 
- 	qDebug("KNProtocolClient::closeConnection(): closing connection");
+#ifndef NDEBUG
+  qDebug("knode: KNProtocolClient::closeConnection(): closing connection");
+#endif
 
   FD_ZERO(&fdsW);
   FD_SET(tcpSocket, &fdsW);
@@ -221,7 +248,7 @@ void KNProtocolClient::closeConnection()
   if (ret > 0) {    // we can write...
     QCString cmd = "QUIT\r\n";
     int todo = cmd.length();
-    send(tcpSocket,&cmd.data()[0],todo, MSG_NOSIGNAL);
+    write(tcpSocket,&cmd.data()[0],todo);
   }
   closeSocket();
 }
@@ -230,11 +257,11 @@ void KNProtocolClient::closeConnection()
 // sends a command (one line), return code is written to rep
 bool KNProtocolClient::sendCommand(const QCString &cmd, int &rep)
 {
-	if (!sendStr(cmd + "\r\n"))
-	  return false;
-	if (!getNextResponse(rep))
-		return false;
-	return true;
+  if (!sendStr(cmd + "\r\n"))
+    return false;
+  if (!getNextResponse(rep))
+    return false;
+  return true;
 }
 
 
@@ -244,52 +271,52 @@ bool KNProtocolClient::sendCommandWCheck(const QCString &cmd, int rep)
   int code;
 
   if (!sendCommand(cmd,code))
-  	return false;
-	if (code!=rep) {
-		handleErrors();
-		return false;
-	}
-	return true;
+    return false;
+  if (code!=rep) {
+    handleErrors();
+    return false;
+  }
+  return true;
 }
 
 
 // sends a message (multiple lines)
 bool KNProtocolClient::sendMsg(const DwString &msg)
 {
-	const char *line = msg.c_str();
-	char *end;
-	QCString buffer;
-	size_t length;
-	char inter[10000];
-	
-	progressValue = 100;
-	predictedLines = msg.length()/80;	  // rule of thumb
-	
-	while ((end = strstr(line,"\r\n"))) {
-	  if (line[0]=='.')                     // expand one period to double period...
-	  	buffer.append(".");
-	  length = end-line+2;
-	  if ((buffer.length()>1)&&((buffer.length()+length)>1024)) {    // artifical limit, because I don't want to generate too large blocks
-   		if (!sendStr(buffer))
-			  return false;
-			buffer = "";
-		}
-		if (length > 9500) {
+  const char *line = msg.c_str();
+  char *end;
+  QCString buffer;
+  size_t length;
+  char inter[10000];
+
+  progressValue = 100;
+  predictedLines = msg.length()/80;   // rule of thumb
+
+  while ((end = strstr(line,"\r\n"))) {
+    if (line[0]=='.')                     // expand one period to double period...
+      buffer.append(".");
+    length = end-line+2;
+    if ((buffer.length()>1)&&((buffer.length()+length)>1024)) {    // artifical limit, because I don't want to generate too large blocks
+      if (!sendStr(buffer))
+        return false;
+      buffer = "";
+    }
+    if (length > 9500) {
       job->setErrorString(i18n("Message size exceeded the size of the internal buffer."));
       closeSocket();
       return false;
     }
-		memcpy(inter,line,length);
-		inter[length]=0;             // terminate string
-		buffer += inter;
-		line = end+2;
-		doneLines++;				
-	}
-	buffer += ".\r\n";
-	if (!sendStr(buffer))
+    memcpy(inter,line,length);
+    inter[length]=0;             // terminate string
+    buffer += inter;
+    line = end+2;
+    doneLines++;
+  }
+  buffer += ".\r\n";
+  if (!sendStr(buffer))
     return false;
 
-	return true;
+  return true;
 }
 
 
@@ -319,7 +346,7 @@ bool KNProtocolClient::getNextLine()
 
     int received;
     do {
-      received = recv(tcpSocket, inputEnd, inputSize-(inputEnd-input)-1, MSG_NOSIGNAL);
+      received = read(tcpSocket, inputEnd, inputSize-(inputEnd-input)-1);
     } while ((received<0)&&(errno==EINTR));       // don't get tricked by signals
 
     if (received <= 0) {
@@ -331,12 +358,13 @@ bool KNProtocolClient::getNextLine()
     inputEnd[0] = 0;  // terminate *char
 
     byteCount += received;
-    if (predictedLines != -1)
-      progressValue = 100 + (doneLines*900/predictedLines);
-
-    sendSignal(TSprogressUpdate);
 
   } while (!(nextLine = strstr(thisLine,"\r\n")));
+
+  if (predictedLines > 0)
+    progressValue = 100 + (doneLines*900/predictedLines);
+  sendSignal(TSprogressUpdate);
+
   nextLine[0] = 0;  // terminate string
   nextLine[1] = 0;
   nextLine+=2;
@@ -347,21 +375,22 @@ bool KNProtocolClient::getNextLine()
 // receives a message (multiple lines)
 bool KNProtocolClient::getMsg(QStrList &msg)
 {
-	char *line;
-	
+  char *line;
+
   while (getNextLine()) {
-  	line = getCurrentLine();
-  	if (line[0]=='.') {
-	  	if (line[1]=='.')
-	    	line++;        // collapse double period into one
-	    else
-	      return true;   // message complete
-	  }
-	  msg.append(line);
-	  doneLines++;
-	}
-	
-	return false;        // getNextLine() failed
+    line = getCurrentLine();
+    if (line[0]=='.') {
+      if (line[1]=='.')
+        line++;        // collapse double period into one
+      else
+        if (line[1]==0)
+          return true;   // message complete
+    }
+    msg.append(line);
+    doneLines++;
+  }
+
+  return false;        // getNextLine() failed
 }
 
 
@@ -394,7 +423,7 @@ bool KNProtocolClient::checkNextResponse(int code)
 void KNProtocolClient::handleErrors()
 {
   if (errorPrefix.isEmpty())
-    job->setErrorString(i18n("An error occoured:\n"));
+    job->setErrorString(i18n("An error occured:\n%1").arg(thisLine));
   else
     job->setErrorString(errorPrefix + thisLine);
 
@@ -404,9 +433,9 @@ void KNProtocolClient::handleErrors()
 
 void KNProtocolClient::sendSignal(threadSignal s)
 {
-	int signal=(int)s;
-	// qDebug("KNProtcolClient::sendSignal() : sending signal to main thread"); too verbose
-	write(fdPipeOut, &signal, sizeof(int));
+  int signal=(int)s;
+  // qDebug("knode: KNProtcolClient::sendSignal() : sending signal to main thread");
+  write(fdPipeOut, &signal, sizeof(int));
 }
 
 
@@ -431,9 +460,9 @@ bool KNProtocolClient::waitForRead()
 
   if (ret == -1) {     // select failed
     if (job) {
-		  QString str = i18n("Communication error:\n");
-		  str += strerror(errno);
-		  job->setErrorString(str);
+      QString str = i18n("Communication error:\n");
+      str += strerror(errno);
+      job->setErrorString(str);
     }
     closeSocket();
     return false;
@@ -446,13 +475,15 @@ bool KNProtocolClient::waitForRead()
   }
   if (ret > 0) {
     if (FD_ISSET(fdPipeIn,&fdsR)) {  // stop signal
-     	qDebug("KNProtocolClient::waitForRead(): got stop signal");
+#ifndef NDEBUG
+      qDebug("knode: KNProtocolClient::waitForRead(): got stop signal");
+#endif
       closeConnection();
       return false;
     }
     if (FD_ISSET(tcpSocket,&fdsE)||FD_ISSET(fdPipeIn,&fdsE)) {  // broken pipe, etc
       if (job)
-				job->setErrorString(i18n("The connection is broken."));
+        job->setErrorString(i18n("The connection is broken."));
       closeSocket();
       return false;
     }
@@ -461,7 +492,7 @@ bool KNProtocolClient::waitForRead()
   }
 
   if (job)
-		job->setErrorString(i18n("Communication error"));
+    job->setErrorString(i18n("Communication error"));
   closeSocket();
   return false;
 }
@@ -491,28 +522,30 @@ bool KNProtocolClient::waitForWrite()
 
   if (ret == -1) {     // select failed
     if (job) {
- 		  QString str = i18n("Communication error:\n");
-		  str += strerror(errno);
-		  job->setErrorString(str);
+      QString str = i18n("Communication error:\n");
+      str += strerror(errno);
+      job->setErrorString(str);
     }
     closeSocket();
     return false;
   }
   if (ret == 0) {      // nothing happend, timeout
-	  if (job)
+    if (job)
       job->setErrorString(i18n("A delay occured which exceeded the\ncurrent timeout limit."));
     closeConnection();
     return false;
   }
   if (ret > 0) {
     if (FD_ISSET(fdPipeIn,&fdsR)) {  // stop signal
-     	qDebug("KNProtocolClient::waitForWrite(): got stop signal");
+#ifndef NDEBUG
+      qDebug("knode: KNProtocolClient::waitForWrite(): got stop signal");
+#endif
       closeConnection();
       return false;
     }
     if (FD_ISSET(tcpSocket,&fdsR)||FD_ISSET(tcpSocket,&fdsE)||FD_ISSET(fdPipeIn,&fdsE)) {  // broken pipe, etc
-	    if (job)
-				job->setErrorString(i18n("The connection is broken."));
+      if (job)
+        job->setErrorString(i18n("The connection is broken."));
       closeSocket();
       return false;
     }
@@ -521,7 +554,7 @@ bool KNProtocolClient::waitForWrite()
   }
 
   if (job)
-		job->setErrorString(i18n("Communication error"));
+    job->setErrorString(i18n("Communication error"));
   closeSocket();
   return false;
 }
@@ -533,7 +566,7 @@ bool KNProtocolClient::conRawIP(in_addr* ip)
   fd_set fdsR,fdsW;
   timeval tv;
 
-	job->setErrorString("");
+  job->setErrorString("");
   sockaddr_in address;
   address.sin_family = AF_INET;
   address.sin_port = htons(account.port());
@@ -552,56 +585,56 @@ bool KNProtocolClient::conRawIP(in_addr* ip)
       } while ((ret<0)&&(errno==EINTR));       // don't get tricked by signals
 
       if (ret == -1) {     // select failed
-				QString str = i18n("Communication error:\n");
-    		str += strerror(errno);
-    		job->setErrorString(str);
-				return false;				
+        QString str = i18n("Communication error:\n");
+        str += strerror(errno);
+        job->setErrorString(str);
+        return false;
       }
 
       if (ret == 0) {      // nothing happend, timeout
-      	job->setErrorString(i18n("A delay occured which exceeded the\ncurrent timeout limit."));
-				return false;
+        job->setErrorString(i18n("A delay occured which exceeded the\ncurrent timeout limit."));
+        return false;
       }
 
       if ((ret > 0)&&(FD_ISSET(fdPipeIn,&fdsR)))   // stop signal
-				return false;
+        return false;
 
       int err = 0;
       unsigned int len = 1;
-      if (getsockopt(tcpSocket,SOL_SOCKET,SO_ERROR,&err,&len)!=0) {
-       	QString str = i18n("Communication error:\n");
-    		str += strerror(errno);
-    		job->setErrorString(str);
-				return false;
+      if (getsockopt(tcpSocket,SOL_SOCKET,SO_ERROR,(char*)&err,&len)!=0) {
+        QString str = i18n("Communication error:\n");
+        str += strerror(errno);
+        job->setErrorString(str);
+        return false;
       } else {
         QCString str;
-				switch (err) {
-					case 0: return true;   // success!!
-					case ECONNREFUSED:
-					  job->setErrorString(i18n("The server refused the connection."));
-					  return false;
-  				case ETIMEDOUT:
-	  				job->setErrorString(i18n("A delay occured which exceeded the\ncurrent timeout limit."));
-					  return false;
-					default:
+        switch (err) {
+          case 0: return true;   // success!!
+          case ECONNREFUSED:
+            job->setErrorString(i18n("The server refused the connection."));
+            return false;
+          case ETIMEDOUT:
+            job->setErrorString(i18n("A delay occured which exceeded the\ncurrent timeout limit."));
+            return false;
+          default:
                       job->setErrorString(i18n("Unable to connect:\n%1").arg(strerror(err)));
-					  return false;
-				}
+            return false;
+        }
       }
     } else {
       QCString str;
-			switch (errno) {
-				case 0: return true;   // success!!
-				case ECONNREFUSED:
-				  job->setErrorString(i18n("The server refused the connection."));
-				  return false;
-  			case ETIMEDOUT:
-	  			job->setErrorString(i18n("A delay occured which exceeded the\ncurrent timeout limit."));
-				  return false;
-				default:
-    			  job->setErrorString(i18n("Unable to connect:\n%1").arg(strerror(errno)));
-				  return false;
-			}
+      switch (errno) {
+        case 0: return true;   // success!!
+        case ECONNREFUSED:
+          job->setErrorString(i18n("The server refused the connection."));
+          return false;
+        case ETIMEDOUT:
+          job->setErrorString(i18n("A delay occured which exceeded the\ncurrent timeout limit."));
+          return false;
+        default:
+            job->setErrorString(i18n("Unable to connect:\n%1").arg(strerror(errno)));
+          return false;
+      }
     }
   }
   return true;
@@ -627,12 +660,12 @@ bool KNProtocolClient::sendStr(const QCString &str)
   while (todo > 0) {
     if (!waitForWrite())
       return false;
-    ret = send(tcpSocket,&str.data()[done],todo, MSG_NOSIGNAL);
+    ret = write(tcpSocket,&str.data()[done],todo);
     if (ret <= 0) {
       if (job) {
-	 	    QString str = i18n("Communication error:\n");
-  	  	str += strerror(errno);
-    		job->setErrorString(str);
+        QString str = i18n("Communication error:\n");
+        str += strerror(errno);
+        job->setErrorString(str);
       }
       closeSocket();
       return false;
@@ -641,10 +674,10 @@ bool KNProtocolClient::sendStr(const QCString &str)
       todo -= ret;
     }
     byteCount += ret;
-    if (predictedLines != -1)
-      progressValue = 100 + (doneLines/predictedLines)*900;
-    sendSignal(TSprogressUpdate);
   }
+  if (predictedLines > 0)
+    progressValue = 100 + (doneLines/predictedLines)*900;
+  sendSignal(TSprogressUpdate);
   return true;
 }
 
@@ -664,7 +697,7 @@ void KNProtocolClient::clearPipe()
     FD_SET(fdPipeIn,&fdsR);
     if (1==(selectRet=select(FD_SETSIZE,&fdsR,NULL,NULL,&tv)))
       if ( ::read(fdPipeIn, &buf, 1 ) == -1 )
-	::perror( "clearPipe()" );
+  ::perror( "clearPipe()" );
   } while (selectRet == 1);
 }
 
