@@ -1,7 +1,7 @@
 /*
  *            kPPP: A pppd front end for the KDE project
  *
- * $Id: connect.cpp,v 1.130 2000/05/29 09:03:37 porten Exp $
+ * $Id: connect.cpp,v 1.134 2000/08/10 13:19:11 porten Exp $
  * 
  *            Copyright (C) 1997 Bernd Johannes Wuebben 
  *                   wuebben@math.cornell.edu
@@ -189,14 +189,13 @@ void ConnectWidget::init() {
   comlist = &gpppdata.scriptType();
   arglist = &gpppdata.script();
 
-  QString tit = i18n("Connecting to: ");
-  tit += gpppdata.accname();
+  QString tit = i18n("Connecting to: %1").arg(gpppdata.accname());
   setCaption(tit);
 
   kapp->processEvents();
 
   // run the "before-connect" command
-  if(strlen(gpppdata.command_before_connect()) > 0) {
+  if (!gpppdata.command_before_connect().isEmpty()) {
     messg->setText(i18n("Running pre-startup command..."));
     emit debugMessage(i18n("Running pre-startup command..."));
 
@@ -295,7 +294,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
       setExpect(gpppdata.modemInitResp());
       QString vol("AT");
       vol += gpppdata.volumeInitString();
-      writeline(vol.data());
+      writeline(vol);
       usleep(gpppdata.modemInitDelay() * 10000); // 0.01 - 3.0 sec
       vmain = 1;
       return;
@@ -309,17 +308,16 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
       timeout_timer->stop();
       timeout_timer->start(gpppdata.modemTimeout()*1000);
 
-      QString bm = i18n("Dialing");
-      QStrList &plist = gpppdata.phonenumbers();
-      bm += " ";
-      bm += gpppdata.dialPrefix();
-      bm += plist.at(dialnumber);
+      QStringList &plist = gpppdata.phonenumbers();
+      QString bmarg= gpppdata.dialPrefix();
+      bmarg += *plist.at(dialnumber);
+      QString bm = i18n("Dialing %1").arg(bmarg);
       messg->setText(bm);
       emit debugMessage(bm);
 
       QString pn = gpppdata.modemDialStr();
       pn += gpppdata.dialPrefix();
-      pn += plist.at(dialnumber);
+      pn += *plist.at(dialnumber);
       if(++dialnumber >= plist.count())
         dialnumber = 0;
       writeline(pn);
@@ -349,9 +347,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
       Modem::modem->hangup();
 
       if(gpppdata.busyWait() > 0) {
-	QString bm = i18n("Line Busy. Waiting: ");
-	bm += gpppdata.busyWait();
-	bm += i18n(" seconds");
+	QString bm = i18n("Line Busy. Waiting: %1 seconds").arg(gpppdata.busyWait());
 	messg->setText(bm);
 	emit debugMessage(bm);
       
@@ -407,17 +403,16 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
       timeout_timer->start(scriptTimeout);
 
       if((unsigned) scriptindex < comlist->count()) {
-        scriptCommand = comlist->at(scriptindex);
-        scriptArgument = arglist->at(scriptindex);
+        scriptCommand = *(comlist->at(scriptindex));
+        scriptArgument = *(arglist->at(scriptindex));
       } else {
         kdDebug(5002) << "End of script" << endl;
 	vmain = 10;
         return;
       }
 
-      if(strcmp(scriptCommand, "Scan") == 0) {
-	QString bm = i18n("Scanning ");
-	bm += scriptArgument;
+      if (scriptCommand == "Scan") {
+	QString bm = i18n("Scanning %1").arg(scriptArgument);
 	messg->setText(bm);
 	emit debugMessage(bm);
 
@@ -426,13 +421,12 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
         return;
       }
 
-      if(strcmp(scriptCommand, "Save") == 0) {
-	QString bm = i18n("Saving ");
-	bm += scriptArgument;
+      if (scriptCommand == "Save") {
+	QString bm = i18n("Saving %1").arg(scriptArgument);
 	messg->setText(bm);
 	emit debugMessage(bm);
 
-	if(stricmp(scriptArgument, "password") == 0) {
+	if (scriptArgument.lower() == "password") {
 	  gpppdata.setPassword(scanvar);
 	  p_kppp->setPW_Edit(scanvar);
 	  if(gpppdata.storePassword())
@@ -445,8 +439,8 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
       }
 
 
-      if(strcmp(scriptCommand, "Send") == 0 || strcmp(scriptCommand, "SendNoEcho") == 0) {
-	QString bm = i18n("Sending ");
+      if (scriptCommand == "Send" || scriptCommand == "SendNoEcho") {
+	QString bm = i18n("Sending %1");
 
 	// replace %USERNAME% and %PASSWORD%
 	QString arg = scriptArgument;
@@ -455,11 +449,11 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	arg = arg.replace(re1, gpppdata.storedUsername());
 	arg = arg.replace(re2, gpppdata.storedPassword());
 
-	if(strcmp(scriptCommand, "Send") == 0)
-	  bm += scriptArgument;
+	if (scriptCommand == "Send")
+	  bm = bm.arg(scriptArgument);
 	else {
-	  for(uint i = 0; i < strlen(scriptArgument); i++)
-	    bm += "*";
+	  for(uint i = 0; i < scriptArgument.length(); i++)
+	    bm = bm.arg("*");
 	}
 
 	messg->setText(bm);
@@ -470,9 +464,8 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
         return;
       }
 
-      if(strcmp(scriptCommand, "Expect") == 0) {
-        QString bm = i18n("Expecting ");
-        bm += scriptArgument;
+      if (scriptCommand == "Expect") {
+        QString bm = i18n("Expecting %1").arg(scriptArgument);
 	messg->setText(bm);
 	emit debugMessage(bm);
 
@@ -482,40 +475,36 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
       }
 
 
-      if(strcmp(scriptCommand, "Pause") == 0) {
-	QString bm = i18n("Pause ");
-	bm += scriptArgument;
-	bm += i18n(" seconds");
+      if (scriptCommand == "Pause") {
+	QString bm = i18n("Pause %1 seconds").arg(scriptArgument);
 	messg->setText(bm);
 	emit debugMessage(bm);
 	
 	pausing = true;
 	
-	pausetimer->start(atoi(scriptArgument)*1000, true);
+	pausetimer->start(scriptArgument.toInt()*1000, true);
 	timeout_timer->stop();
 	
 	scriptindex++;
 	return;
       }
 
-      if(strcmp(scriptCommand, "Timeout") == 0) {
+      if (scriptCommand == "Timeout") {
 
 	timeout_timer->stop();
 
-	QString bm = i18n("Timeout ");
-	bm += scriptArgument;
-	bm += i18n(" seconds");
+	QString bm = i18n("Timeout %1 seconds").arg(scriptArgument);
 	messg->setText(bm);
 	emit debugMessage(bm);
 	
-	scriptTimeout=atoi(scriptArgument)*1000;
+	scriptTimeout=scriptArgument.toInt()*1000;
         timeout_timer->start(scriptTimeout);
 	
 	scriptindex++;
 	return;
       }
 
-      if(strcmp(scriptCommand, "Hangup") == 0) {
+      if (scriptCommand == "Hangup") {
 	messg->setText(i18n("Hangup"));
 	emit debugMessage(i18n("Hangup"));
 
@@ -526,7 +515,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	return;
       }
 
-      if(strcmp(scriptCommand, "Answer") == 0) {
+      if (scriptCommand == "Answer") {
 	
 	timeout_timer->stop();
 
@@ -538,9 +527,8 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	return;
       }
 
-      if(strcmp(scriptCommand, "ID") == 0) {
-	QString bm = i18n("ID ");
-	bm += scriptArgument;
+      if (scriptCommand == "ID") {
+	QString bm = i18n("ID %1").arg(scriptArgument);
 	messg->setText(bm);
 	emit debugMessage(bm);
 
@@ -548,7 +536,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	
 	if(!idstring.isEmpty() && firstrunID) {
 	  // the user entered an Id on the main kppp dialog
-	  writeline(idstring.data());
+	  writeline(idstring);
 	  firstrunID = false;
 	  scriptindex++;
 	}
@@ -575,9 +563,8 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	}
       }
 
-      if(strcmp(scriptCommand, "Password") == 0) {
-	QString bm = i18n("Password ");
-	bm += scriptArgument;
+      if (scriptCommand == "Password") {
+	QString bm = i18n("Password %1").arg(scriptArgument);
 	messg->setText(bm);
 	emit debugMessage(bm);
 
@@ -585,7 +572,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	
 	if(!pwstring.isEmpty() && firstrunPW) {
 	  // the user entered a password on the main kppp dialog
-	  writeline(pwstring.data());
+	  writeline(pwstring);
 	  firstrunPW = false;
 	  scriptindex++;
 	}
@@ -613,8 +600,8 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	}
       }
  
-      if(strcmp(scriptCommand, "Prompt") == 0) {
-	QString bm = i18n("Prompting ");
+      if (scriptCommand == "Prompt") {
+	QString bm = i18n("Prompting %1");
 
         // if the scriptindex (aka the prompt text) includes a ## marker 
         // this marker should get substituted with the contents of our stored 
@@ -627,7 +614,7 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 		ts.insert( vstart, scanvar );
 	}
 
-	bm += ts;
+	bm = bm.arg(ts);
 	messg->setText(bm);
 	emit debugMessage(bm);
 
@@ -650,9 +637,8 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	}
       }
 
-      if(strcmp(scriptCommand, "PWPrompt") == 0) {
-	QString bm = i18n("PW Prompt ");
-	bm += scriptArgument;
+      if (scriptCommand == "PWPrompt") {
+	QString bm = i18n("PW Prompt %1").arg(scriptArgument);
 	messg->setText(bm);
 	emit debugMessage(bm);
 
@@ -675,10 +661,9 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	}
       }
 
-      if(strcmp(scriptCommand, "LoopStart") == 0) {
+      if (scriptCommand == "LoopStart") {
 
-        QString bm = i18n("Loop Start ");
-        bm += scriptArgument;
+        QString bm = i18n("Loop Start %1").arg(scriptArgument);
 
 	if ( loopnest > (MAXLOOPNEST-2) ) {
 		bm += i18n("ERROR: Nested too deep, ignored.");
@@ -699,11 +684,10 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	scriptindex++;
       }
 
-      if(strcmp(scriptCommand, "LoopEnd") == 0) {
-        QString bm = "Loop End ";
-        bm += scriptArgument;
+      if (scriptCommand == "LoopEnd") {
+        QString bm = i18n("Loop End %1").arg(scriptArgument);
 	if ( loopnest <= 0 ) {
-		bm = i18n("LoopEnd without matching Start! Line: ") + bm ;
+		bm = i18n("LoopEnd without matching Start! Line: %1").arg(bm);
 		vmain=20;
 		scriptindex++;
 		cancelbutton();
@@ -720,23 +704,22 @@ void ConnectWidget::timerEvent(QTimerEvent *) {
 	scriptindex++;
       }
 
-      if(strcmp(scriptCommand, "Mode") == 0) {
-	QString bm = "Setting mode to "; // i18n
-	bm += scriptArgument;
+      if (scriptCommand == "Mode") {
+	QString bm = i18n("Setting mode to %1").arg(scriptArgument); // i18n
 	messg->setText(bm);
 	emit debugMessage(bm);
 	scriptindex++;
 
-	if(stricmp(scriptArgument, "7bit") == 0) {
+	if (scriptArgument.lower() == "7bit") {
 	  Modem::modem->setReadMask(0x7F);
 	  return;
 	}
-	if(stricmp(scriptArgument, "8bit") == 0) {
+	if (scriptArgument.lower(), "8bit") {
 	  Modem::modem->setReadMask(0xFF);
 	  return;
 	}
 	cancelbutton();
-	QString msg = i18n("Unknown mode `%1' !\n\n"
+	QString msg = i18n("Unknown mode `%1'!\n\n"
                            "Valid modes are `7bit' and `8bit'.")
                            .arg(scriptArgument);
 	KMessageBox::error(0, msg);
@@ -891,8 +874,7 @@ void ConnectWidget::checkBuffers() {
       scanvar = scanvar.stripWhiteSpace();
 
       // Show the Variabel content in the debug window
-      QString sv = i18n("Scan Var: ");
-      sv += scanvar;
+      QString sv = i18n("Scan Var: %1").arg(scanvar);
       emit debugMessage(sv);
   }
 
@@ -902,8 +884,7 @@ void ConnectWidget::checkBuffers() {
       // keep everything after the expected string
       readbuffer.remove(0, readbuffer.find(expectstr) + expectstr.length());
 
-      QString ts = i18n("Found: ");
-      ts += expectstr;
+      QString ts = i18n("Found: %1").arg(expectstr);
       emit debugMessage(ts);
 
       if (loopend) {
@@ -914,8 +895,7 @@ void ConnectWidget::checkBuffers() {
     if (loopend && readbuffer.contains(loopstr[loopnest])) {
       expecting = false;
       readbuffer = "";
-      QString ts = i18n("Looping: ");
-      ts += loopstr[loopnest];
+      QString ts = i18n("Looping: %1").arg(loopstr[loopnest]);
       emit debugMessage(ts);
       scriptindex = loopstartindex[loopnest];
       loopend = false;
@@ -1005,23 +985,21 @@ void ConnectWidget::debugbutton() {
 }
 
 
-void ConnectWidget::setScan(const char *n) {
+void ConnectWidget::setScan(const QString &n) {
   scanning = true;
   scanstr = n;
   scanbuffer = "";
 
-  QString ts = i18n("Scanning: ");
-  ts += n;
+  QString ts = i18n("Scanning: %1").arg(n);
   emit debugMessage(ts);
 }
 
 
-void ConnectWidget::setExpect(const char *n) {
+void ConnectWidget::setExpect(const QString &n) {
   expecting = true;
   expectstr = n;
 
-  QString ts = i18n("Expecting: ");
-  ts += n;
+  QString ts = i18n("Expecting: %1").arg(n);
   ts.replace(QRegExp("\n"), "<LF>");
   emit debugMessage(ts);
 
@@ -1118,7 +1096,7 @@ void ConnectWidget::if_waiting_slot() {
     p_kppp->con_win->show();
     
     if(gpppdata.get_iconify_on_connect()) {
-      p_kppp->con_win->iconify();
+      p_kppp->con_win->showMinimized();
     }
   }
 
@@ -1186,10 +1164,13 @@ bool ConnectWidget::execppp() {
   if(gpppdata.autoDNS())
     command += " usepeerdns";
 
-  QStrList &arglist = gpppdata.pppdArgument();
-  for (char *arg = arglist.first(); arg; arg = arglist.next()) {
+  QStringList &arglist = gpppdata.pppdArgument();
+  for ( QStringList::Iterator it = arglist.begin();
+        it != arglist.end();
+        ++it )
+  {
     command += " ";
-    command += arg;
+    command += *it;
   }
 
   // PAP settings
@@ -1219,7 +1200,7 @@ bool ConnectWidget::execppp() {
 
   kapp->flushX();
 
-  return Requester::rq->execPPPDaemon(command.data());
+  return Requester::rq->execPPPDaemon(command);
 }
 
 
@@ -1233,8 +1214,8 @@ void ConnectWidget::setMsg(const QString &msg) {
   messg->setText(msg);
 }
 
-void ConnectWidget::writeline(const char *s) {
-  Modem::modem->writeLine(s);
+void ConnectWidget::writeline(const QString &s) {
+  Modem::modem->writeLine(s.local8Bit());
 }
 
 // Set the hostname and domain from DNS Server
@@ -1257,7 +1238,7 @@ void auto_hostname() {
       new_hostname=hostname_entry->h_name;
       dot=new_hostname.find('.');
       new_hostname=new_hostname.remove(dot,new_hostname.length()-dot);
-      Requester::rq->setHostname(new_hostname.data());
+      Requester::rq->setHostname(new_hostname);
       modified_hostname = TRUE;
 
       new_hostname=hostname_entry->h_name;
@@ -1271,13 +1252,13 @@ void auto_hostname() {
 
 // Replace the DNS domain entry in the /etc/resolv.conf file and
 // disable the nameserver entries if option is enabled
-void add_domain(const char *domain) {
+void add_domain(const QString &domain) {
 
   int fd;
   char c;
   QString resolv[MAX_RESOLVCONF_LINES];
 
-  if (domain == 0L || strlen(domain) == 0) 
+  if (domain.isEmpty())
     return;
 
   if((fd = Requester::rq->openResolv(O_RDONLY)) >= 0) {
@@ -1295,9 +1276,10 @@ void add_domain(const char *domain) {
     if ((c != '\n') && (i < MAX_RESOLVCONF_LINES)) i++;
 
     if((fd = Requester::rq->openResolv(O_WRONLY|O_TRUNC)) >= 0) {
+      QCString tmp = domain.local8Bit();
 
       write(fd, "domain ", 7);
-      write(fd, domain, strlen(domain));
+      write(fd, tmp.data(), tmp.length());
       write(fd, " \t\t#kppp temp entry\n", 20);
 
       for(int j=0; j < i; j++) {
@@ -1308,11 +1290,13 @@ void add_domain(const char *domain) {
 	        && !resolv[j].contains("#entry disabled by kppp")) {
 
           write(fd, "# ", 2);
-	  write(fd, resolv[j].data(), resolv[j].length());
+          QCString tmp = resolv[j].local8Bit();
+	  write(fd, tmp, tmp.length());
           write(fd, " \t#entry disabled by kppp\n", 26);
 	}
 	else {
-	  write(fd, resolv[j].data(), resolv[j].length());
+          QCString tmp = resolv[j].local8Bit();
+	  write(fd, tmp, tmp.length());
 	  write(fd, "\n", 1);
 	}
       }
@@ -1323,14 +1307,19 @@ void add_domain(const char *domain) {
 
 
 // adds the DNS entries in the /etc/resolv.conf file
-void adddns() {
+void adddns()
+{
   int fd;
 
-  if((fd = Requester::rq->openResolv(O_WRONLY|O_APPEND)) >= 0) {
-    QStrList &dnslist = gpppdata.dns();
-    for(char *dns = dnslist.first(); dns; dns = dnslist.next()) {
+  if ((fd = Requester::rq->openResolv(O_WRONLY|O_APPEND)) >= 0) {
+    QStringList &dnslist = gpppdata.dns();
+    for ( QStringList::Iterator it = dnslist.begin();
+          it != dnslist.end();
+          ++it )
+    {
       write(fd, "nameserver ", 11);
-      write(fd, dns, strlen(dns));
+      QCString dns = (*it).local8Bit();
+      write(fd, dns.data(), dns.length());
       write(fd, " \t#kppp temp entry\n", 19);
     }
     close(fd);
@@ -1383,11 +1372,13 @@ void removedns() {
       for(int j=0; j < i; j++) {
 	if(resolv[j].contains("#kppp temp entry")) continue;
 	if(resolv[j].contains("#entry disabled by kppp")) {
-	  write(fd, resolv[j].data()+2, resolv[j].length() - 27);
+          QCString tmp = resolv[j].local8Bit();
+	  write(fd, tmp.data()+2, tmp.length() - 27);
 	  write(fd, "\n", 1);
 	}
 	else {
-	  write(fd, resolv[j].data(), resolv[j].length());
+          QCString tmp = resolv[j].local8Bit();
+	  write(fd, tmp, tmp.length());
 	  write(fd, "\n", 1);
 	}
       }
@@ -1397,7 +1388,7 @@ void removedns() {
   }
 
   if (  modified_hostname ) {
-    Requester::rq->setHostname(old_hostname.data());
+    Requester::rq->setHostname(old_hostname);
     modified_hostname = FALSE;
   }
 
