@@ -1,7 +1,7 @@
 /*
  *            kPPP: A front end for pppd for the KDE project
  *
- * $Id: modeminfo.cpp,v 1.29 1999/10/09 00:26:50 kalle Exp $
+ * $Id: modeminfo.cpp,v 1.31 2000/07/30 21:09:57 bieker Exp $
  * 
  * Copyright (C) 1997 Bernd Johannes Wuebben 
  *                    wuebben@math.cornell.edu
@@ -29,7 +29,7 @@
 #include <qregexp.h>
 #include <qlayout.h>
 #include <kapp.h> 
-#include <kwm.h>
+#include <kwin.h>
 #include <kmessagebox.h>
 #include "modeminfo.h"
 #include "modem.h"
@@ -39,7 +39,7 @@ ModemTransfer::ModemTransfer(QWidget *parent, const char *name)
   : QDialog(parent, name,TRUE, WStyle_Customize|WStyle_NormalBorder)
 {
   setCaption(i18n("ATI Query"));
-  KWM::setMiniIcon(winId(), kapp->miniIcon());
+  KWin::setIcons(winId(), kapp->icon(), kapp->miniIcon());
 
   QVBoxLayout *tl = new QVBoxLayout(this, 10, 10);
   
@@ -187,7 +187,7 @@ void ModemTransfer::do_script() {
     query.sprintf("ATI%d\n", step);
     statusBar->setText(msg);
     progressBar->advance(1);
-    Modem::modem->writeLine(query.data());
+    Modem::modem->writeLine(query.local8Bit());
     break;
 
   default:
@@ -246,7 +246,7 @@ ModemInfo::ModemInfo(QWidget *parent, const char* name)
   QString label_text;
 
   setCaption(i18n("Modem Query Results"));
-  KWM::setMiniIcon(winId(), kapp->miniIcon());
+  KWin::setIcons(winId(), kapp->icon(), kapp->miniIcon());
 
   QVBoxLayout *tl = new QVBoxLayout(this, 10, 10);
 
