@@ -10,7 +10,7 @@
 #include "kscolourpicker.h"
 #include "../config.h"
 #include "kconfig.h"
-#include "../kspainter.h"
+#include "../KSircListBox/kspainter.h"
 #include <klocale.h>
 
 extern KConfig *kConfig;
@@ -198,3 +198,4 @@ void kscolourpicker::slot_cancel()
   delete this;
 }
 
+#include "kscolourpicker.moc"
