@@ -15,9 +15,9 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-#include <qmessagebox.h>
 #include <qevent.h>
 #include <qkeycode.h>
+#include "../config.h"
 
 // #include <qapplication.h>
 #include <iostream.h>
@@ -49,7 +49,7 @@ open_ksirc::open_ksirc
   serverFileParser::readDatafile( filename );
 
   Groups.setAutoDelete(TRUE);
-  
+
   // TODO add "Recent" to global listing servers here..
   // Now we read in the Recent group from the config file
   // remove all recent servers first
@@ -97,21 +97,22 @@ open_ksirc::open_ksirc
 
   PB_Connect->setDefault(TRUE);
   PB_Connect->setAutoDefault(TRUE);
+  PB_Edit->setEnabled(false); // Not yet implemented.
 
   ComboB_ServerName->setFocus();
   connect(ComboB_ServerName, SIGNAL(enterPressed()), this, SLOT(clickConnect()));
 }
 
-// insert a sorted list of groups into ComboB_ServerGroup, note that 
-// we want to get the recent servers in first so we insert "Recent" 
+// insert a sorted list of groups into ComboB_ServerGroup, note that
+// we want to get the recent servers in first so we insert "Recent"
 // in first, then we want Random.
 
 void open_ksirc::insertGroupList()
 {
   QStrList tempgroups;
-  Server *serv; 
- 
-  for ( serv=Groups.first(); serv != 0; serv=Groups.next() ) { 
+  Server *serv;
+
+  for ( serv=Groups.first(); serv != 0; serv=Groups.next() ) {
     if (tempgroups.find(serv->group()) == -1)
       tempgroups.inSort( serv->group() );
   }
@@ -133,9 +134,11 @@ void open_ksirc::insertServerList( const char * group )
   Server *serv;
 
   for ( serv=Groups.first(); serv != 0; serv=Groups.next() ) {
-    if (strcmp(serv->group(), group) == 0)
-      newListBox->inSort( serv->server() );
+    if ( !qstrcmp(serv->group(), group) )
+      newListBox->insertItem( serv->server() );
   }
+  newListBox->sort();
+
   ComboB_ServerName->setListBox(newListBox);
 //  ComboB_ServerName->setCurrentItem( 0 ); // this don't work =(
   if (newListBox->count() > 0)
@@ -151,9 +154,9 @@ void open_ksirc::setServer( const QString &serveraddress )
 {
   QListBox *newListBox  = new("QListBox") QListBox();
   Server *serv;
-  QList<port> portlist; 
+  QList<port> portlist;
   port *p;
-  bool defaultport = FALSE; 
+  bool defaultport = FALSE;
 
   for ( serv=Groups.first(); serv != 0; serv=Groups.next() ) {
     if (strcmp(serv->server(), serveraddress) == 0) {
@@ -161,7 +164,7 @@ void open_ksirc::setServer( const QString &serveraddress )
       portlist = serv->ports();
       for ( p=portlist.last(); p != 0; p=portlist.prev() ) {
         newListBox->insertItem( p->portnum() );
-        if (strcmp(p->portnum(), "6667") == 0) 
+        if (strcmp(p->portnum(), "6667") == 0)
           defaultport = TRUE;
       }
       break;
@@ -207,6 +210,9 @@ void open_ksirc::setGroup( const QString &group )
 
 void open_ksirc::clickConnect()
 {
+  if ( ComboB_ServerName->currentText().isEmpty() )
+      return; // sorry, no error message currently (message freeze)
+    
   QString server;
   int port;
   QString script;
@@ -248,10 +254,10 @@ void open_ksirc::clickConnect()
     conf->writeEntry("RecentServers", recent);
     conf->sync();
   }
-  
+
   emit open_ksircprocess( server, port, script );
   emit open_ksircprocess(str);
-  
+
   accept();
 }
 
@@ -263,10 +269,6 @@ void open_ksirc::clickCancel()
 void open_ksirc::clickEdit()
 {
   // TODO open new("server") server editor
-  QMessageBox::information(this, "kSirc - Open Server",
-                           "Feature not implemented",
-                           "&Ok", "", "", 0, 0);
-
 }
 
 open_ksirc::~open_ksirc()
