@@ -19,11 +19,7 @@ typedef QList<filterRule> filterRuleList;
 class KSircMessageReceiver
 {
 public:
-  KSircMessageReceiver(KSircProcess *_proc)
-    {
-      proc = _proc;
-      broadcast = TRUE;
-    }
+  KSircMessageReceiver(KSircProcess *_proc);
   virtual ~KSircMessageReceiver();
 
   virtual void sirc_receive(QString str);
