@@ -4,7 +4,9 @@
 #include <stdlib.h>
 #include <time.h>
 #include <iostream.h>
+#include "config.h"
 
+#include <kdebug.h>
 
 QDict<QObject> *objFinder::objList = new("QDict<QObject>") QDict<QObject>;
 
@@ -26,7 +28,7 @@ void objFinder::insert(QObject *obj, const char *key){ /*FOLD00*/
   QString name;
 
   if(obj == 0x0){
-    warning("objFinder: Passed Null Object");
+    qWarning("objFinder: Passed Null Object");
     return;
   }
   
@@ -39,7 +41,7 @@ void objFinder::insert(QObject *obj, const char *key){ /*FOLD00*/
       name = randString();
     }
   }
-  objList->insert(name.data(), obj);
+  objList->insert(name, obj);
   connect(obj, SIGNAL(destroyed()),
           objFind, SLOT(objDest()));
   
@@ -93,7 +95,7 @@ void objFinder::dumpTree(){ /*fold00*/
   QWidgetList *all = QApplication::allWidgets();
   QWidgetListIt itW(*all);
   while(itW.current()){
-    cerr << itW.current()->className() << "::" << itW.current()->name("unnamed") << endl;
+    kdDebug() << itW.current()->className() << "::" << itW.current()->name("unnamed") << endl;
     ++itW;
   }
 
@@ -146,7 +148,6 @@ void objFinder::objDest(){ /*fold00*/
   if(sender() == 0x0){
     return;
   }
-  debug("Destroyed: %s", sender()->name());
   QDictIterator<QObject> it(*objList);
   while(it.current()){
     if(it.current() == sender()){
