@@ -1,4 +1,5 @@
 #include <stdio.h>
+#include "../config.h"
 
 #include "playout.h"
 #include "commands.h"
@@ -12,7 +13,7 @@ PLayout::PLayout(QObject *pobject)
 
 PLayout::~PLayout()
 {
-  //  debug("PObject: in destructor");
+  //  kdDebug() << "PObject: in destructor" << endl;
   /*
   delete widget();
   layout = 0;
@@ -33,12 +34,12 @@ PObject *PLayout::createWidget(CreateArgs &ca)
   if((ca.parent != 0) &&
      (ca.parent->widget()->isWidgetType() == TRUE)){
     qbl = new("QBoxLayout") QBoxLayout((QWidget *) ca.parent->widget(), (QBoxLayout::Direction) direction, border);
-    //      debug("Creating layout with parent: %d", parent.iWinId);
+    //      kdDebug() << "Creating layout with parent: " << parent.iWinId << endl;
 
   }
   else{
     qbl = new("QBoxLayout") QBoxLayout((QBoxLayout::Direction) direction, border);
-    //      debug("Creating layout NO PARENT", parent.iWinId);
+    //      kdDebug() << "Creating layout NO PARENT" << endl;
   }
   pw->setWidget(qbl);
   pw->setWidgetId(ca.pwI);
@@ -50,11 +51,11 @@ void PLayout::messageHandler(int fd, PukeMessage *pm)
 {
   PukeMessage pmRet;
 
-//  debug("In PLayout: %d", pm->iCommand);
+//  kdDebug() << "In PLayout: " << pm->iCommand << endl;
   
   if(pm->iCommand == PUKE_LAYOUT_ADDWIDGET){
     if(pm->iTextSize != 2*sizeof(char)){
-      warning("PLayout/addwidget: incorrent cArg size, bailing out.  Needed: %u wanted: %d\n", sizeof(int), pm->iTextSize);
+      qWarning("PLayout/addwidget: incorrent cArg size, bailing out.  Needed: %u wanted: %d\n", sizeof(int), pm->iTextSize);
       pmRet.iCommand = PUKE_LAYOUT_ADDWIDGET_ACK; // ack the add widget
       pmRet.iWinId = pm->iWinId;
       pmRet.iArg = 1;
@@ -66,8 +67,7 @@ void PLayout::messageHandler(int fd, PukeMessage *pm)
     wiWidget.fd = fd;
     wiWidget.iWinId = pm->iArg;
     PWidget *pw = controller()->id2pwidget(&wiWidget);
-    //    debug("Adding widget with stretch: %d and align: %d", (int) pm->cArg[0],
-    //	  (int) pm->cArg[1]);
+    //    kdDebug() << "Adding widget with stretch: " << (int) pm->cArg[0] << " and align: " << //	  (int) pm->cArg[1] << endl;
     widget()->addWidget(pw->widget(), pm->cArg[0], pm->cArg[1]);
 
     pmRet.iCommand = PUKE_LAYOUT_ADDWIDGET_ACK; // ack the add widget
@@ -78,7 +78,7 @@ void PLayout::messageHandler(int fd, PukeMessage *pm)
   }
   else if(pm->iCommand == PUKE_LAYOUT_ADDLAYOUT){
     if(pm->iTextSize != sizeof(char)){
-      warning("PLayout: incorrent cArg size, bailing out.  Needed: %u wanted: %d\n", sizeof(int), pm->iTextSize);
+      qWarning("PLayout: incorrent cArg size, bailing out.  Needed: %u wanted: %d\n", sizeof(int), pm->iTextSize);
       pmRet.iCommand = PUKE_LAYOUT_ADDLAYOUT_ACK; // ack the add widget
       pmRet.iWinId = pm->iWinId;
       pmRet.iArg = 1;
@@ -135,9 +135,12 @@ void PLayout::messageHandler(int fd, PukeMessage *pm)
 
 void PLayout::setWidget(QObject *_layout)
 {
-  //  debug("PObject setwidget called");
+  //  kdDebug() << "PObject setwidget called" << endl;
   if(_layout != 0 && _layout->inherits("QBoxLayout") == FALSE)
-      throw(errorInvalidSet(_layout, className()));
+  {
+     errorInvalidSet(_layout);
+     return;
+  }
 
   layout = (QBoxLayout *) _layout;
   PObject::setWidget(_layout);
