@@ -1,6 +1,10 @@
+#include <kdebug.h>
+#include "../config.h"
+
 #include "plistbox.h"
 
-PObject *createWidget(CreateArgs &ca) /*FOLD00*/
+PObject *
+PListBox::createWidget(CreateArgs &ca) /*FOLD00*/
 {
   PListBox *plb = new("PListBox") PListBox(ca.parent);
   QListBox *lb;
@@ -21,14 +25,14 @@ PObject *createWidget(CreateArgs &ca) /*FOLD00*/
 PListBox::PListBox(PObject *parent) /*FOLD00*/
   : PFrame(parent)
 {
-  //  debug("PListBox PListBox called");
+  //  kdDebug() << "PListBox PListBox called" << endl;
   lb = 0;
   setWidget(lb);
 }
 
 PListBox::~PListBox() /*FOLD00*/
 {
-  //  debug("PListBox: in destructor");
+  //  kdDebug() << "PListBox: in destructor" << endl;
   /*
   delete widget();     // Delete the frame
   lb=0;          // Set it to 0
@@ -55,7 +59,8 @@ void PListBox::messageHandler(int fd, PukeMessage *pm) /*FOLD00*/
     if(!checkWidget())
       return;
     
-    widget()->inSort(pm->cArg);
+    widget()->insertItem(pm->cArg);
+    widget()->sort();
     pmRet.iCommand = - pm->iCommand;
     pmRet.iWinId = pm->iWinId;
     pmRet.iArg = widget()->count();
@@ -117,7 +122,7 @@ void PListBox::messageHandler(int fd, PukeMessage *pm) /*FOLD00*/
     }
     break;
   case PUKE_LISTBOX_SET_SCROLLBAR:
-    widget()->setScrollBar((bool) pm->iArg);
+    widget()->setVScrollBarMode( (bool) pm->iArg ? QListBox::AlwaysOn : QListBox::AlwaysOff );
     pmRet.iCommand = PUKE_LISTBOX_SET_SCROLLBAR_ACK;
     pmRet.iWinId = pm->iWinId;
     pmRet.iArg = 0;
@@ -125,7 +130,7 @@ void PListBox::messageHandler(int fd, PukeMessage *pm) /*FOLD00*/
     emit outputMessage(fd, &pmRet);
     break;
   case PUKE_LISTBOX_SET_AUTO_SCROLLBAR:
-    widget()->setAutoScrollBar((bool) pm->iArg);
+    widget()->setVScrollBarMode( (bool) pm->iArg ? QListBox::Auto : QListBox::AlwaysOff );
     pmRet.iCommand = PUKE_LISTBOX_SET_AUTO_SCROLLBAR_ACK;
     pmRet.iWinId = pm->iWinId;
     pmRet.iArg = 0;
@@ -148,7 +153,10 @@ void PListBox::messageHandler(int fd, PukeMessage *pm) /*FOLD00*/
 void PListBox::setWidget(QObject *_lb) /*FOLD00*/
 {
   if(_lb != 0 && _lb->inherits("QListBox") == FALSE)
-    throw(errorInvalidSet(_lb, className()));
+  {
+    errorInvalidSet(_lb);
+    return;
+  }
 
   lb = (QListBox *) _lb;
   if(lb != 0){
@@ -171,7 +179,7 @@ QListBox *PListBox::widget() /*FOLD00*/
 void PListBox::highlighted(int index) { /*FOLD00*/
   PukeMessage pmRet;
 
-  debug("Got highlight");
+  kdDebug() << "Got highlight" << endl;
   pmRet.iCommand = PUKE_LISTBOX_HIGHLIGHTED_ACK;
   pmRet.iWinId = widgetIden().iWinId;
   pmRet.iArg = index;
@@ -189,7 +197,7 @@ void PListBox::highlighted(int index) { /*FOLD00*/
 void PListBox::selected(int index) { /*FOLD00*/
   PukeMessage pmRet;
 
-  debug("Got selected");
+  kdDebug() << "Got selected" << endl;
   pmRet.iCommand = PUKE_LISTBOX_SELECTED_ACK;
   pmRet.iWinId = widgetIden().iWinId;
   pmRet.iArg = index;
@@ -206,7 +214,7 @@ void PListBox::selected(int index) { /*FOLD00*/
 
 bool PListBox::checkWidget(){ /*FOLD00*/
   if(widget() == 0){
-    debug("PListBox: No Widget set");
+    kdDebug() << "PListBox: No Widget set" << endl;
     return FALSE;
   }
   return TRUE;
