@@ -1,8 +1,10 @@
 #include "pobjfinder.h"
 #include "pobjfinder-cmd.h"
 #include <qstrlist.h>
+#include "../config.h"
 
-PObject *createWidget(CreateArgs &ca)
+PObject *
+PObjFinder::createWidget(CreateArgs &ca)
 {
   PObjFinder *pw = new("PObjFinder") PObjFinder(ca.parent);
   pw->setWidget(0x0);
@@ -18,7 +20,7 @@ PObjFinder::PObjFinder(PObject *parent)
   // We don't actually encase a widget since all the ObjFinder interface
   // is static
   setWidget(0x0);
-  connect(objFind, SIGNAL(inserted(QObject *)),
+  connect(controller(), SIGNAL(inserted(QObject *)),
           this, SLOT(newObject(QObject *)));
 }
 
@@ -33,7 +35,7 @@ void PObjFinder::messageHandler(int fd, PukeMessage *pm)
   case PUKE_OBJFINDER_ALLOBJECTS:
     {
       QString qscArg;
-      QStrList allObj = objFinder::allObjects();
+      QStrList allObj = controller()->allObjects();
       for(uint i = 0; i <= allObj.count(); i++){
         qscArg += allObj.at(i);
         qscArg += "\n";
@@ -81,3 +83,5 @@ void PObjFinder::newObject(QObject *name){
   emit outputMessage(widgetIden().fd, &pmRet);
   delete pmRet.cArg;
 }
+
+#include "pobjfinder.moc"
