@@ -1,7 +1,9 @@
 #include "ppushbt.h"
 #include <stdio.h>
+#include "../config.h"
 
-PObject *createWidget(CreateArgs &ca)
+PObject *
+PPushButton::createWidget(CreateArgs &ca)
 {
   PPushButton *pb = new("PPushButton") PPushButton(ca.parent);
   QPushButton *qb;
@@ -18,14 +20,14 @@ PObject *createWidget(CreateArgs &ca)
 PPushButton::PPushButton(PObject *parent)
   : PButton(parent)
 {
-  //  debug("PLineEdit PLineEdit called");
+  //  kdDebug() << "PLineEdit PLineEdit called" << endl;
   button = 0;
   setWidget(button);
 }
 
 PPushButton::~PPushButton()
 {
-  //  debug("PLineEdit: in destructor"); 
+  //  kdDebug() << "PLineEdit: in destructor" << endl;
 /*  delete widget();     // Delete the frame
   button=0;          // Set it to 0
   setWidget(button); // Now set all widget() calls to 0.
@@ -44,7 +46,10 @@ void PPushButton::messageHandler(int fd, PukeMessage *pm)
 void PPushButton::setWidget(QObject *_qb)
 {
   if(_qb != 0 && _qb->inherits("QPushButton") == FALSE)
-    throw(errorInvalidSet(_qb, className()));
+  {
+     errorInvalidSet(_qb);
+     return;
+  }
 
   button = (QPushButton *) _qb;
   if(_qb != 0){
