@@ -1,10 +1,12 @@
 #include <stdio.h>
+#include "../config.h"
 
 #include "ptabdialog.h"
 #include "commands.h"
 #include <iostream.h>
 
-PObject *createWidget(CreateArgs &ca)
+PObject *
+PTabDialog::createWidget(CreateArgs &ca)
 {
   PTabDialog *ptd = new("PTabDialog") PTabDialog(ca.parent);
   QTabDialog *qtd;
@@ -33,7 +35,7 @@ PTabDialog::PTabDialog(QObject *)
 
 PTabDialog::~PTabDialog()
 {
-  //  debug("PObject: in destructor");
+  //  kdDebug() << "PObject: in destructor" << endl;
   /*
   delete widget();
   tab = 0;
@@ -47,7 +49,7 @@ void PTabDialog::messageHandler(int fd, PukeMessage *pm)
 
   if(pm->iCommand == PUKE_TABDIALOG_ADDTAB){
     if(!(pm->iTextSize > 0)){
-      warning("QTabDialog/addtab: incorrent cArg size, bailing out.  Needed: >0 got: %d\n", pm->iTextSize);
+      qWarning("QTabDialog/addtab: incorrent cArg size, bailing out.  Needed: >0 got: %d\n", pm->iTextSize);
       pmRet.iCommand = PUKE_TABDIALOG_ADDTAB_ACK; // ack the add widget
       pmRet.iWinId = pm->iWinId;
       pmRet.iArg = 1;
@@ -59,8 +61,7 @@ void PTabDialog::messageHandler(int fd, PukeMessage *pm)
     wiWidget.fd = fd;
     wiWidget.iWinId = pm->iArg;
     PWidget *pw = controller()->id2pwidget(&wiWidget);
-    //    debug("Adding widget with stretch: %d and align: %d", (int) pm->cArg[0],
-    //	  (int) pm->cArg[1]);
+    //    kdDebug() << "Adding widget with stretch: " << (int) pm->cArg[0] << " and align: " << //	  (int) pm->cArg[1] << endl;
     widget()->addTab(pw->widget(), pm->cArg);
 
     pmRet.iCommand = PUKE_TABDIALOG_ADDTAB_ACK; // ack the add widget
@@ -78,7 +79,10 @@ void PTabDialog::messageHandler(int fd, PukeMessage *pm)
 void PTabDialog::setWidget(QObject *tb)
 {
   if(tb != 0 && tb->inherits("QTabDialog") == FALSE)
-    throw(errorInvalidSet(tb, className()));
+  {
+     errorInvalidSet(tb);
+     return;
+  }
 
   tab = (QTabDialog *) tb;
   if(tab != 0){
@@ -89,7 +93,7 @@ void PTabDialog::setWidget(QObject *tb)
 
 QTabDialog *PTabDialog::widget()
 {
-  //  debug("PObject widget called");
+  //  kdDebug() << "PObject widget called" << endl;
   return tab;
 }
 
