@@ -12,9 +12,10 @@
 #include <qtextstream.h>
 
 #include "config.h"
-#include "irclistitem.h"
+#include "KSircListBox/irclistitem.h"
 #include <klocale.h>
 #include <kstddirs.h>
+#include "config.h"
 
 extern global_config *kSircConfig; 
 
